/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.access;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL .<br/>
 * Access Control List
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: AccessControlList.java 8214 2006-08-23 12:12:55Z peterit $
 */

public class AccessControlList {

  public static final String DELIMITER = " ";
  
  private String owner;

  private Map <String, String[]>permissions;

  public AccessControlList(boolean initDefault) {
    this.owner = SystemIdentity.SYSTEM;
    this.permissions = new HashMap<String, String[]>();
    if(initDefault)
      permissions.put(SystemIdentity.ANY, PermissionType.ALL);
  }
  
  public AccessControlList(String owner, Map<String, String[]>permissions) {
    this.owner = owner;
    this.permissions = permissions;
  }
  
  public AccessControlList(PropertyData ownerProp, PropertyData permissionsProp)
      throws RepositoryException {

    if(ownerProp != null) // if property already initialized in storage
      this.owner = new String(((ValueData) ownerProp.getValues().get(0))
        .getAsByteArray());
    
    if (permissionsProp != null) {
      permissions = new HashMap<String, String[]>();
      List permValues = permissionsProp.getValues();

      for (int i = 0; i < permValues.size(); i++) {
        String p = new String(((ValueData) permValues.get(i)).getAsByteArray());
        // TODO
        StringTokenizer parser = new StringTokenizer(p, DELIMITER);
        addPermission(parser.nextToken(), parser.nextToken());
        //////
      }
    }
  }

  /**
   * Assign new owner and permissions map to this ACL
   * 
   * @param owner
   * @param permissions
   */
  public void assign(String owner, Map<String, String[]>permissions) {
    this.owner = owner;
    this.permissions = permissions;
  }
  
  /**
   * Reset this ACL to default state. Owner SYSTEM. Permission ANY/ANY
   * 
   * @param owner
   * @param permissions
   */
  public void reset() {
    this.owner = SystemIdentity.SYSTEM;
    this.permissions = new HashMap<String, String[]>();
    this.permissions.put(SystemIdentity.ANY, PermissionType.ALL);
  }
  
  private void addPermission(String identity, String perm) {
    if (permissions.containsKey(identity)) {
      String[] oldPerm = (String[]) permissions.get(identity);
      String[] newPerm = new String[oldPerm.length + 1];
      for (int i = 0; i < oldPerm.length; i++)
        newPerm[i] = oldPerm[i];
      newPerm[newPerm.length - 1] = perm;
      permissions.put(identity, newPerm);
    } else {
      permissions.put(identity, new String[] { perm });
    }
  }

  /**
   * @return Returns the owner.
   */
  public String getOwner() {
    return owner;
  }

  /**
   * Returns map key=(String)identity; value=(String[])permissions
   * 
   * @return Returns the permissions.
   */
  public Map<String, String[]> getPermissions() {
    return permissions;
  }

  /**
   * @return list of PermissionEntry objects
   */
  public List <String> getPermissionEntries() {
    List <String> list =  new ArrayList<String>();
    Iterator ids = permissions.keySet().iterator();
    while(ids.hasNext()) {
      String id = (String)ids.next();
      String[] perm = permissions.get(id);
      for(int i=0; i<perm.length; i++)
       list.add(id+" "+perm[i]);
      // better ?
      // list.add(id+DELIMITER+perm[i]);

    }
    return list;
  }
  
  public String dump() {
    String res = "OWNER: "+owner+"\n";
    Iterator ids = permissions.keySet().iterator();
    while(ids.hasNext()) {
      String id = (String)ids.next();
      String[] perm = (String[])permissions.get(id);
      String permStr = "";
      for(int i=0; i<perm.length; i++)
        permStr+=perm[i]+" ";
      res+=" "+id +" --->" +permStr+"\n";
    }
    return res;
  }
}