/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.dataflow;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;

/**
 * Created by The eXo Platform SARL        .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: ItemState.java 6078 2006-06-08 15:14:03Z peterit $
 */

public class ItemState {

  public static final int ADDED = 1;

  public static final int UPDATED = 2;

  public static final int DELETED = 3;

  public static final int UNCHANGED = 4;
  
  public static final int MIXIN_CHANGED = 5;

  protected final ItemData data;
  protected final int state;
  protected final boolean eventFire;
  /**
   * for Session.move()
   */
  private final InternalQPath ancestorToSave;


  /**
   * The constructor
   * @param data underlying data
   * @param state
   * @param eventFire - if the state cause some event firing
   * @param ancestorToSave - path of item which should be called in save (usually for session.move())
   */
  public ItemState(ItemData data, int state, boolean eventFire, InternalQPath ancestorToSave) {
    this.data = data;
    this.state = state;
    this.eventFire = eventFire;
    if(ancestorToSave == null)
      this.ancestorToSave = data.getQPath();
    else
      this.ancestorToSave = ancestorToSave;

  }
  

  /**
   * @return data.
   */
  public ItemData getData() {
    return data;
  }
  
  /**
   * @return state.
   */
  public int getState() {
    return state;
  }
  
  public boolean isNode() {
    return data.isNode();
  }
  
  public boolean isAdded() {
    return (state == ADDED);
  }

  public boolean isUpdated() {
    return (state == UPDATED);
  }

  public boolean isDeleted() {
    return (state == DELETED);
  }

  public boolean isUnchanged() {
    return (state == UNCHANGED);
  }
  
  public boolean isMixinChanged() {
    return (state == MIXIN_CHANGED);
  }
  
  public boolean isEventFire() {
    return eventFire;
  }

  public InternalQPath getAncestorToSave() {
    return ancestorToSave;
  }


  /**
   * creates ADDED item state
   * shortcut for new ItemState(data, ADDED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createAdded(ItemData data) {
    return new ItemState(data, ADDED, true, null); 
  }

  /**
   * creates UPDATED item state
   * shortcut for new ItemState(data, UPDATED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createUpdated(ItemData data) {
    return new ItemState(data, UPDATED, true, null); 
  }

  /**
   * creates DELETED item state
   * shortcut for new ItemState(data, DELETED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createDeleted(ItemData data) {
    return new ItemState(data, DELETED, true, null); 
  }

  /**
   * creates UNCHANGED item state
   * shortcut for new ItemState(data, UNCHANGED, false, false, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createUnchanged(ItemData data) {
    return new ItemState(data, UNCHANGED, false, null); 
  }

  public static String getStateName(int state) {
    if (state == ADDED)
      return "ADDED";
    else if (state == UPDATED)
      return "UPDATED";
    else if (state == DELETED)
      return "DELETED";
    else if (state == UNCHANGED)
      return "UNCHANGED";
    else if (state == MIXIN_CHANGED)
      return "MIXIN CHANGED";
//    else if (state == AUTO_DELETED)
//      return "AUTO DELETED";
    else
      return "UNDEFINED";
  }



}
