/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.dataflow.persistent;

import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.ItemDataVisitor;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.NodeData;
/**
 * Created by The eXo Platform SARL        .</br>
 * 
 * Immutable NodeData from persistense storage
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: PersistedNodeData.java 7556 2006-08-03 19:52:16Z geaz $
 */

public class PersistedNodeData extends PersistedItemData implements NodeData {
  
  protected final int orderNumber;
  protected final InternalQName primaryTypeName;
  protected final InternalQName[] mixinTypeNames;
  
  public PersistedNodeData(String id, InternalQPath qpath, String parentId, int version,
      int orderNumber, InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      AccessControlList acl) {
    super(id, qpath, parentId, version, acl);
    this.primaryTypeName = primaryTypeName;
    this.mixinTypeNames = mixinTypeNames;
    this.orderNumber = orderNumber;
  }

  public int getOrderNumber() {
    return orderNumber;
  }

  public InternalQName getPrimaryTypeName() {
    return primaryTypeName;
  }

  public InternalQName[] getMixinTypeNames() {
    return mixinTypeNames;
  }

  public boolean isNode() {
    return true;
  }

  public AccessControlList getACL() {
    return acl;
  }
  
  public void accept(ItemDataVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  
  public List getChildNodes() {
    throw new RuntimeException("DO NOT call getChildNodes! ");
  }

  public List getChildProperties() {
    throw new RuntimeException("DO NOT call getChildProperties! ");
  }
  
  public int getOrderDirection() {
    return 0;
  }
  
  public boolean isOrdered() {
    // For persisted node data - not ordered always 
    return getOrderDirection() != 0;
  }

}
