/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.dataflow.persistent;

import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.ItemDataVisitor;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL        .</br>
 * 
 * Immutable PropertyData
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: PersistedPropertyData.java 7950 2006-08-15 06:47:56Z peterit $
 */

public class PersistedPropertyData extends PersistedItemData implements PropertyData {

  protected List<ValueData> values;
  protected final int type;
  protected final boolean multiValued;
  
  public PersistedPropertyData(String id, InternalQPath qpath, String parentId, int version,
      int type, boolean multiValued, List<ValueData> values, AccessControlList acl) {
    super(id, qpath, parentId, version, acl);
    this.values = values;
    this.type = type;
    this.multiValued = multiValued;
  }
  
  public PersistedPropertyData(String id, InternalQPath qpath, String parentId, int version,
      int type, boolean multiValued, AccessControlList acl) {
    super(id, qpath, parentId, version, acl);
    this.values = null;
    this.type = type;
    this.multiValued = multiValued;
  }

  
  public int getType() {
    return type;
  }

  public List<ValueData> getValues() {
    return values;
  }

  public boolean isMultiValued() {
    return multiValued;
  }
  
  public boolean isNode() {
    return false;
  }

  public void setType(int type) {
    throw new RuntimeException("DO NOT call setType! ");
  }

  public void setValues(List<ValueData> values) throws RepositoryException {
    if(this.values == null)
      this.values = values;
    else
      throw new RuntimeException("The values can not be changed ");
  }

  public void accept(ItemDataVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

}
