/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl;

import java.util.List;

import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NodeTypeManager;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NamespaceRegistryImpl;
import org.exoplatform.services.jcr.impl.core.WorkspaceInitializer;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.util.UUIDGenerator;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL 11.05.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: RepositoryInitializer.java 8586 2006-09-05 10:25:14Z peterit $
 */
public class RepositoryInitializer {

  static protected Log log = ExoLogger.getLogger("jcr.WorkspaceStorageInitializer");
  
  private RepositoryContainer container;

  private RepositoryEntry     config;
  
  RepositoryInitializer(RepositoryContainer container, RepositoryEntry config)
      throws RepositoryException, RepositoryConfigurationException {
    this.container = container;
    this.config = config;
    
  }
  
  void init() throws RepositoryException, RepositoryConfigurationException {
    List wsEntries = config.getWorkspaceEntries();
    //Collections.sort(wsEntries, new WorkspaceOrderComparator(config.getSystemWorkspaceName()));
    for (int i = 0; i < wsEntries.size(); i++) {
      initWorkspace((WorkspaceEntry) wsEntries.get(i));
    }        
  }
  
  void load() throws RepositoryException {
    NamespaceRegistryImpl nsRegistry = (NamespaceRegistryImpl) container.getComponentInstanceOfType(NamespaceRegistry.class);
    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) container.getComponentInstanceOfType(NodeTypeManager.class);
          
    // Load from persistence
    nsRegistry.loadFromStorage();
    ntManager.loadFromStorage();
  }
  
  private void initWorkspace(WorkspaceEntry wsConfig) throws RepositoryException {
    
    WorkspaceContainer workspaceContainer = (WorkspaceContainer) container.getComponentInstance(wsConfig.getName());
    //boolean isSystem = config.getSystemWorkspaceName().equals(wsConfig.getName());

    WorkspaceInitializer wsInitializer = (WorkspaceInitializer) workspaceContainer.getComponentInstanceOfType(WorkspaceInitializer.class);
    
    UUIDGenerator uuidGen = (UUIDGenerator) container.getComponentInstanceOfType(UUIDGenerator.class);
    LocationFactory locationFactory = (LocationFactory) container.getComponentInstanceOfType(LocationFactory.class);
    
    // Init Root and jcr:system if workspace is system workspace
    if (wsConfig.getAutoInitializedRootNt() != null && !wsInitializer.isWorkspaceInitialized()) {
      InternalQName rootNodeTypeName = locationFactory.parseJCRName(wsConfig.getAutoInitializedRootNt()).getInternalName();
      wsInitializer.initWorkspace(rootNodeTypeName);
    }
    
  }

}
