/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.io.InputStream;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.Item;
import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.ItemVisitor;
import javax.jcr.MergeException;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.Lock;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.ItemDefinition;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;
import javax.jcr.version.VersionHistory;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.access.SystemIdentity;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.itemfilters.ItemFilter;
import org.exoplatform.services.jcr.impl.core.itemfilters.NamePatternFilter;
import org.exoplatform.services.jcr.impl.core.lock.LockImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.ItemDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.PermissionValue;
import org.exoplatform.services.jcr.impl.core.version.MergeVisitor;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionImpl;
import org.exoplatform.services.jcr.impl.dataflow.MutableItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;
import org.exoplatform.services.jcr.impl.dataflow.data.ItemDataFactory;
import org.exoplatform.services.jcr.impl.dataflow.session.SessionChangesLog;
import org.exoplatform.services.jcr.impl.util.EntityCollection;
import org.exoplatform.services.jcr.util.UUIDGenerator;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL.
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: NodeImpl.java 8545 2006-09-04 13:49:41Z peterit $
 */

public class NodeImpl extends ItemImpl implements ExtendedNode {
  
  protected static Log log = ExoLogger.getLogger("jcr.NodeImpl");

  protected LocationFactory sysLocFactory;

  private NodeDefinition    definition;
   
  /**
   * New constructor
   * @param id
   * @param session
   */
  public NodeImpl(NodeData data, SessionImpl session) throws RepositoryException {
    super(data, session);
    sysLocFactory = session.getSystemLocationFactory();
    loadData(data);
  }  
   

  public void loadData(ItemData data) throws RepositoryException, InvalidItemStateException,
      ConstraintViolationException {

    if (data == null)
      throw new InvalidItemStateException("Data is null for " + this.getPath()
          + " Probably was deleted by another session and can not be loaded from container ");

    if (!data.isNode())
      throw new RepositoryException("Load data: Node expected");

    NodeData nodeData = (NodeData) data;
    if (nodeData.getPrimaryTypeName() == null)
      throw new RepositoryException("Load data: NodeData has no primaryTypeName. Null value found. " 
          + (nodeData.getQPath() != null ? nodeData.getQPath().getAsString() : "[null path node]") 
          + " " + nodeData);
    if (nodeData.getMixinTypeNames() == null)
      throw new RepositoryException("Load data: NodeData has no mixinTypeNames. Null value found. " 
          + (nodeData.getQPath() != null ? nodeData.getQPath().getAsString() : "[null path node]"));
    
    this.data = nodeData;

  }

  private void initDefinition() throws RepositoryException, ConstraintViolationException {

    if (this.isRoot()) { // root - no parent
      NodeDefinitionImpl defNodeDef = new NodeDefinitionImpl();
      defNodeDef.setRequiredNodeTypes(new NodeType[] {nodeType(Constants.NT_BASE)});
      this.definition = defNodeDef;
      return;
    }

    //NodeImpl parent = (NodeImpl) getParent();

    NodeType[] nodetypes = parent().getAllNodeTypes();
    NodeDefinition residualDef = null;
    for (int i = 0; i < nodetypes.length; i++) {

      NodeDefinition def = ((NodeTypeImpl) nodetypes[i]).getChildNodeDefinition(getInternalName());

      if (def != null) {
        if (def.getName() == ItemDefinitionImpl.RESIDUAL_SET)
          residualDef = def;
        else {
          log.debug("Node definition found " + def.getName());
          this.definition = def;
          return;
          // return def;
        }
      }
    }

    if (residualDef != null)
      this.definition = residualDef;
    else
      throw new ConstraintViolationException(
          "NodeImpl.getDefinition failed. Definition not found for " + getPath());

  }

  private AccessControlList initACL(NodeData nodeData) {
    AccessControlList dataACL = nodeData.getACL();
    if (dataACL == null) { // default one
      return new AccessControlList(true);
    } else {
      return dataACL;
    }
  }


  /**
   * @see javax.jcr.Node#getNode
   */
  public Node getNode(String relPath) throws PathNotFoundException, RepositoryException {

    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);
    //NodeImpl myParent = findParent(itemPath);
    if(log.isDebugEnabled())
      log.debug("getNode() " + itemPath.getAsString(false));
    Item node = item(itemPath);
    if(node == null || !node.isNode())
      throw new PathNotFoundException("Node not found " + itemPath.getAsString(false));
    
    return (Node)node;
  }

  /**
   * @see javax.jcr.Node#getNodes
   */
  public NodeIterator getNodes() throws RepositoryException {

    return new EntityCollection(getChildNodes());
  }


  /**
   * @return list with actual nodes, that stored in persisten storage
   */
  protected List <NodeImpl> getChildNodes() throws RepositoryException, AccessDeniedException {

    List <NodeImpl> storedNodes = dataManager.getChildNodes(nodeData(), true);
    Collections.sort(storedNodes, new NodesOrderComparator());
    return storedNodes;
  }

  private static class NodesOrderComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      NodeImpl n1 = (NodeImpl) o1;
      NodeImpl n2 = (NodeImpl) o2;
      return n1.getOrderNumber() - n2.getOrderNumber();
    }
  }

  /**
   * @see javax.jcr.Node#getNodes
   */
  public NodeIterator getNodes(String namePattern) throws RepositoryException {
    ItemFilter filter = new NamePatternFilter(namePattern);
    ArrayList list = new ArrayList();
    NodeIterator children = getNodes();
    for(NodeImpl item:getChildNodes()) {
      if (filter.accept(item))
        list.add(item);
    }
    return new EntityCollection(list);
  }

  /**
   * @see javax.jcr.Node#getProperty
   */
  public Property getProperty(String relPath) throws PathNotFoundException, RepositoryException {

    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);
  
    if(log.isDebugEnabled())
      log.debug("getProperty() " + itemPath.getAsString(false));
    
    Item prop = item(itemPath);
    if(prop == null || prop.isNode())
      throw new PathNotFoundException("Property not found " + itemPath.getAsString(false));
    
    return (Property)prop;
  }
  
  protected PropertyImpl property(InternalQName name) throws IllegalPathException, PathNotFoundException, RepositoryException {
    Item prop = item(InternalQPath.makeChildPath(getInternalPath(), name));
    if(prop == null || prop.isNode())
      throw new PathNotFoundException("Property not found " + name);
    return (PropertyImpl) prop; 
  }
  
  private boolean hasProperty(InternalQName name) {
    try {
      if(property(name) != null)
        return true;
    } catch (RepositoryException e) {
    }
    return false;

  }


  /**
   * @see javax.jcr.Node#getProperties
   */
  public PropertyIterator getProperties() throws RepositoryException {
    
    return new EntityCollection(getChildProperties());
  }


  /**
   * @return list with actual properties, that stored in persisten storage
   */
  protected List getChildProperties() throws RepositoryException, AccessDeniedException {

    List <PropertyImpl> storedProperties = dataManager.getChildProperties((NodeData)getData(), true);
    Collections.sort(storedProperties, new PropertiesOrderComparator());
    return storedProperties;
  }

  private static class PropertiesOrderComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      int r = 0;
      try {
        PropertyImpl p1 = (PropertyImpl) o1;
        InternalQName qname1 = p1.getLocation().getName().getInternalName();
        PropertyImpl p2 = (PropertyImpl) o2;
        InternalQName qname2 = p2.getLocation().getName().getInternalName();
        if (qname1.equals(Constants.JCR_PRIMARYTYPE)) {
          r = Integer.MIN_VALUE;
        } else if (qname2.equals(Constants.JCR_PRIMARYTYPE)) {
          r = Integer.MAX_VALUE;
        } else if (qname1.equals(Constants.JCR_MIXINTYPES)) {
          r = Integer.MIN_VALUE + 1;
        } else if (qname2.equals(Constants.JCR_MIXINTYPES)) {
          r = Integer.MAX_VALUE - 1;
        } else if (qname1.equals(Constants.JCR_UUID)) {
          r = Integer.MIN_VALUE + 2;
        } else if (qname2.equals(Constants.JCR_UUID)) {
          r = Integer.MAX_VALUE - 2;
        } else {
          r = qname1.getAsString().compareTo(qname2.getAsString());
        }
      } catch (Exception e) {
        System.err.println("PropertiesOrderComparator error: " + e);
      }
      return r;
    }
  }

  /**
   * @see javax.jcr.Node#getProperties
   */
  public PropertyIterator getProperties(String namePattern) throws RepositoryException {
    ItemFilter filter = new NamePatternFilter(namePattern);
    ArrayList list = new ArrayList();
    PropertyIterator children = getProperties();
    while (children.hasNext()) {
      PropertyImpl item = (PropertyImpl) children.nextProperty();
      if (filter.accept(item))
        list.add(item);
    }

    return new EntityCollection(list);
  }

  /**
   * @see javax.jcr.Node#getIndex
   */
  public int getIndex() throws RepositoryException {
    return getLocation().getIndex();
  }

  /**
   * @see javax.jcr.Node#getPrimaryItem
   */
  public Item getPrimaryItem() throws ItemNotFoundException, RepositoryException {

    NodeType[] types = this.getAllNodeTypes();
    for (int i = 0; i < types.length; i++) {
      String itemName = types[i].getPrimaryItemName();
      if (itemName != null) {
        Item primaryItem = dataManager.getItem(getInternalPath(), true);
        if (primaryItem != null)
          return primaryItem;
      }
    }

    throw new ItemNotFoundException("Primary item not found for " + getPath());
  }

  /**
   * @see javax.jcr.Node#getUUID
   */
  public String getUUID() throws UnsupportedRepositoryOperationException, RepositoryException {
    if (isNodeType(Constants.MIX_REFERENCEABLE)) {
      return getInternalUUID();
    } else {
      throw new UnsupportedRepositoryOperationException("Node " + getPath()
          + " is not referenceable");
    }

  }

  /**
   * Returns saved only references (allowed by specs)
   * 
   * @see javax.jcr.Node#getReferences
   */
  public PropertyIterator getReferences() throws RepositoryException {
    return new EntityCollection(dataManager.getReferences(getUUID()));
  }

  /**
   * @see javax.jcr.Node#hasNode
   */
  public boolean hasNode(String relPath) throws RepositoryException {
    try {
      getNode(relPath);
    } catch (PathNotFoundException e) {
      return false;
    }
    return true;
  }

  /**
   * @see javax.jcr.Node#hasNodes
   */
  public boolean hasNodes() throws RepositoryException {
    return getNodes().hasNext();
  }

  /**
   * @see javax.jcr.Node#hasProperty
   */
  public boolean hasProperty(String relPath) throws RepositoryException {
    try {
      getProperty(relPath);
      return true;
    } catch (RepositoryException e) {
      return false;
    }
  }

  /**
   * @see javax.jcr.Node#hasProperties
   */
  public boolean hasProperties() throws RepositoryException {
    return getProperties().hasNext();
  }

  /**
   * @see javax.jcr.Node#addNode
   */
  public Node addNode(String relPath) throws  PathNotFoundException,
      ConstraintViolationException, RepositoryException, VersionException, LockException {

    // Parent can be not the same as this node
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);

    // Check if there no final index
    if (itemPath.isIndexSetExplicitly())
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + itemPath.getAsString(false));

    NodeImpl parent = (NodeImpl)dataManager.getItem(itemPath.makeParentPath().getInternalPath(), false); //this.findParent(itemPath);
    InternalQName name = itemPath.getName().getInternalName();
    // Check if parent exists
    if (parent == null)
      throw new PathNotFoundException("Parent not found for " + itemPath.getAsString(true));

    // find node type
    //NodeType[] nodeTypes = parent.getAllNodeTypes();
    JCRName nodeTypeName = parent.findNodeType(itemPath.getName().getAsString());
    
    // try to make new node
    return doAddNode(parent, name, nodeTypeName.getInternalName());

  }

  public JCRName findNodeType(String name) throws RepositoryException,
      ConstraintViolationException {
    
    NodeType[] nodeTypes = getAllNodeTypes();
    String residualNodeTypeName = null;
    for (int j = 0; j < nodeTypes.length; j++) {
      NodeDefinition[] nodeDefs = nodeTypes[j].getChildNodeDefinitions();
      for (int i = 0; i < nodeDefs.length; i++) {
        NodeDefinition nodeDef = nodeDefs[i];
        if (nodeDef.getName().equals(name)) {
          return sysLocFactory.parseJCRName(nodeDef.getDefaultPrimaryType().getName());
        } else if (nodeDef.getName().equals(NodeDefinitionImpl.RESIDUAL_SET)) {
          residualNodeTypeName = nodeDef.getDefaultPrimaryType().getName();
        }
      }
    }
    if (residualNodeTypeName == null)
      throw new ConstraintViolationException("Can not define node type for " + name);
    return sysLocFactory.parseJCRName(residualNodeTypeName);
  }
  

  /**
   * @see javax.jcr.Node#addNode
   */
  public Node addNode(String relPath, String nodeTypeName) throws ItemExistsException,
      PathNotFoundException, NoSuchNodeTypeException, ConstraintViolationException,
      RepositoryException, VersionException, LockException {

    // Parent can be not the same as this node
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);

    // Check if there no final index
    if (itemPath.isIndexSetExplicitly())
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + itemPath.getAsString(false));

    NodeImpl parent = (NodeImpl)dataManager.getItem(itemPath.makeParentPath().getInternalPath(), false);
    // Check if parent exists
    if (parent == null)
      throw new PathNotFoundException("Parent not found for " + itemPath.getAsString(true));

    InternalQName name = itemPath.getName().getInternalName();
    InternalQName ptName = locationFactory.parseJCRName(nodeTypeName).getInternalName();

    
    // try to make new node
    return doAddNode(parent, name, ptName);
  }

  
  public void validateChildNode(InternalQName name, InternalQName primaryTypeName)  
  throws ItemExistsException, RepositoryException, ConstraintViolationException,
  VersionException, LockException {
    
    // Check if nodeType exists and not mixin
    String ptStr = sysLocFactory.createJCRName(primaryTypeName).getAsString();
    NodeTypeImpl type = nodeType(primaryTypeName);

    if (type.isMixin())
      throw new ConstraintViolationException("Add Node failed: Node Type <" + ptStr
          + "> is MIXIN type!");

    // Check if new node's node type is allowed by its parent definition
    NodeType[] types = getAllNodeTypes();
    for (int i = 0; i < types.length; i++) {
      NodeTypeImpl t = (NodeTypeImpl) types[i];
      if (t.isChildNodePrimaryTypeAllowed(ptStr)) {
        break;
      } else if (i == types.length - 1) {
        throw new ConstraintViolationException("Can't add node " + name.getAsString() + " to "
            + getPath() + " node type " + primaryTypeName.getAsString()
            + " is not allowed as child's node type for parent node type ");
      }
    }

    // Check if node is not protected 
    if (findChildNodeDefinition(name).isProtected()) 
      throw new ConstraintViolationException("Can't add protected node " + name.getAsString()
            + " to " + getPath());

    // Check if versionable ancestor is not checked-in
    if (!isCheckedOut())
      throw new VersionException("Node " + getPath()
          + " or its nearest ancestor is checked-in");

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");
    
  }

  private NodeImpl doAddNode(NodeImpl parentNode, InternalQName name, InternalQName primaryTypeName)  
      throws ItemExistsException, RepositoryException, ConstraintViolationException,
      VersionException, LockException {
    
    parentNode.validateChildNode(name, primaryTypeName);
    
    // Initialize data
    InternalQName[] mixinTypeNames = new InternalQName[0];
    String uuid = UUIDGenerator.generate();
    String parentUUID = parentNode.getInternalUUID();
    int orderNum = parentNode.getNextChildOrderNum();
    int index = parentNode.getNextChildIndex(name);
    InternalQPath path = InternalQPath.makeChildPath(parentNode.getInternalPath(), name, index);
    AccessControlList acl = new AccessControlList(true);

    // create new nodedata
    NodeData nodeData = new TransientNodeData(path, uuid, index, primaryTypeName,
        mixinTypeNames, orderNum, parentNode.getInternalUUID(), acl);

    // Create new Node
    ItemState state = ItemState.createAdded(nodeData);
    NodeImpl node = (NodeImpl)dataManager.update(state, true);
    
    addAutoCreatedItems(primaryTypeName);

    if(log.isDebugEnabled())
      log.debug("new node : " + node.getPath() + " name: "
        + " primaryType: " + node.getPrimaryNodeType().getName() + " index: " + node.getIndex()
        + " parent: " + parentNode);

    return node;

  }

  public int getNextChildOrderNum() throws RepositoryException {
    return getChildNodes().size();
  }

  public int getNextChildIndex(InternalQName nameToAdd)
      throws RepositoryException, ItemExistsException {
    
    //NodeDefinition def = childToAdd.getDefinition();
    int ind = 0;
    for(NodeImpl sibling: getChildNodes()) {
      //NodeImpl child = (NodeImpl) children.nextNode();
      
      if (sibling.getInternalName().equals(nameToAdd)) {
        NodeDefinition def = sibling.getDefinition();
        if(log.isDebugEnabled())
          log.debug("Calculate index for " + nameToAdd + " " + 
              sibling.getPath());

        if (def.allowsSameNameSiblings())
          ind++;
        else
          throw new ItemExistsException("The node " + nameToAdd + " already exists in "
              + getPath() + " and same name sibling is not allowed ");

      }
    }
    return ind + 1;

  }

  
  protected int getValuesType(Value[] values) throws ValueFormatException {

    int type = PropertyType.UNDEFINED;

    if (values != null && values.length > 0) {
      for (Value v : values) {
        if (v != null) {
          if (type != PropertyType.UNDEFINED) {
            if (v.getType() != type) {
              throw new ValueFormatException(
                  "Can not set multi-valued property with array of different type values  "
                      + getPath());
            }
          } else {
            type = v.getType();
          }
        }
      }
    }

    return type;
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value[] values, int type) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    List <Value> newValues = values != null ? new ArrayList<Value>(): null;
    for(Value val:values)
      newValues.add(val);
    return doSetProperty(locationFactory.parseJCRName(name).getInternalName(), 
        newValues, type, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value[] values) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    int type = getValuesType(values);

    //return doSetProperty(name, values, type, true, true, true);
    return setProperty(name, values, type);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String[] values) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    return setProperty(name, values, PropertyType.STRING);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String[] values, int type) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {
    List <Value> strValues = null;
    if (values != null){
      List valList = packStrValues(values);
      strValues = new ArrayList<Value>();
      for (int i = 0; i < valList.size(); i++) {
        strValues.add(valueFactory.createValue((String) valList.get(i)));
      }
    }
    return doSetProperty(locationFactory.parseJCRName(name).getInternalName(), 
        strValues, type, true);
  }

  private List packStrValues(String[] values) {
    ArrayList valList = new ArrayList();
    for (int i = 0; i < values.length; i++) {
      if (values[i] != null)
        valList.add(values[i]);
    }
    return valList;
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {
    int valType;
    Value[] values;
    if (value == null) {
      valType = PropertyType.UNDEFINED;
      values = null;
    } else {
      valType = value.getType();
      values = new Value[] { value };
    }
    
    List <Value> newValues = value != null ? new ArrayList<Value>(): null;
    newValues.add(value);
    return doSetProperty(locationFactory.parseJCRName(name).getInternalName(), 
        newValues, valType, false);

//    return doSetProperty(name, values, valType, false, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value value, int type) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    List <Value> newValues = value != null ? new ArrayList<Value>(): null;
    newValues.add(value);
    return doSetProperty(locationFactory.parseJCRName(name).getInternalName(), 
        newValues, type, false);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String value, int type) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    Value val = null;
    // Hm, It's copy of code used in PropertyImpl.doSetValue(),
    // for TCK's SetPropertyAssumeTypeTest
    try {
      val = value == null ? null : valueFactory.createValue(value, type);
    } catch (ValueFormatException e) {
      // We have to know what type of Exception tyo throw,
      // ValueFormatException or ConstraintViolationException for property type
      // violated the node type definition

      throw new ConstraintViolationException(
          "The change would violate a requred node type with property '" + name
              + "'. A property value '" + value + "' can't be of type "
              + PropertyType.nameFromValue(type) + ". Converting is impossible: " + e.getMessage(),
          e);
    }

    return setProperty(name, val, type);

  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    Value val = value == null ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.STRING);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, InputStream value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    Value val = value == null ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.BINARY);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, boolean value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value), PropertyType.BOOLEAN);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Calendar value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    Value val = value == null ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.DATE);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, double value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value), PropertyType.DOUBLE);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, long value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value), PropertyType.LONG);

  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Node value) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.REFERENCE);
  }

  protected PropertyImpl doSetProperty(InternalQName qname, List<Value> values, int type, boolean multiValued) 
      throws ValueFormatException, VersionException, LockException, ConstraintViolationException,
      RepositoryException {

    String name = locationFactory.createJCRName(qname).getAsString();
    JCRPath path = locationFactory.createJCRPath(getLocation(), name);
    
    TransientPropertyData newData = doUpdateProperty(this, qname,
        values, multiValued, type);
    ItemState state;
    
    if((PropertyImpl)dataManager.getItem(path.getInternalPath(), false) != null) {
      state = ItemState.createUpdated(newData);
    } else {
      state = ItemState.createAdded(newData);
    }
    return (PropertyImpl)dataManager.update(state, true);
    
  }
  

  /**
   * @see javax.jcr.Node#getPrimaryNodeType
   */
  public NodeType getPrimaryNodeType() throws RepositoryException {
    //System.out.println("Primary type "+primaryType.getInternalName());
    return nodeType(nodeData().getPrimaryTypeName());
  }

  /**
   * @see javax.jcr.Node#getMixinNodeTypes
   */
  public NodeType[] getMixinNodeTypes() throws RepositoryException {

    // should not be null
    if (nodeData().getMixinTypeNames() == null)
      throw new RepositoryException("Data Container implementation error getMixinTypeNames == null");
    NodeType[] mixinNodeTypes = new NodeType[nodeData().getMixinTypeNames().length];
    for (int i = 0; i < mixinNodeTypes.length; i++) {
      mixinNodeTypes[i] = nodeType(nodeData().getMixinTypeNames()[i]);
    }

    return mixinNodeTypes;
  }

  /**
   * @see javax.jcr.Node#isNodeType
   */
  public boolean isNodeType(String nodeTypeName) throws RepositoryException {
    return isNodeType(locationFactory.parseJCRName(nodeTypeName).getInternalName());
  }
  
  public boolean isNodeType(InternalQName qName) throws RepositoryException {
    NodeType testNodeType;
    try {
      testNodeType = nodeType(qName);
    } catch (NoSuchNodeTypeException e) {
      //e.printStackTrace();
      log.warn("Node.isNodeType() No such nodetype: " + qName.getAsString());
      return false;
    }
    
    //System.out.println("IS TYPE >>> "+testNodeType+" "+qName.getAsString());
    
    NodeType[] nodetypes = getAllNodeTypes();
    for (int i = 0; i < nodetypes.length; i++) {
      //System.out.println("IS TYPE >>> "+testNodeType.getName()+" "+nodetypes[i]);
      if (NodeTypeImpl.isSameOrSubType(testNodeType, nodetypes[i]))
        return true;
    }
    return false;
  }


  /**
   * @see javax.jcr.Node#getDefinition
   */
  public NodeDefinition getDefinition() throws RepositoryException {

    if (definition == null)
      initDefinition();

    return definition;

  }

  /**
   * @see javax.jcr.Node#getCorrespondingNodePath
   */
  public String getCorrespondingNodePath(String workspaceName) throws ItemNotFoundException,
      NoSuchWorkspaceException, AccessDeniedException, RepositoryException {
    SessionImpl correspSession = (SessionImpl) ((RepositoryImpl) session.getRepository()).login(
        session.getCredentials(), workspaceName);

    return getCorrespondingNode(correspSession).getPath();
  }

  public NodeImpl getCorrespondingNode(SessionImpl correspSession) throws ItemNotFoundException,
      NoSuchWorkspaceException, AccessDeniedException, RepositoryException {
    if (this.isNodeType("mix:referenceable")) {
      try {
        return (NodeImpl) correspSession.getNodeByUUID(getUUID());
      } catch (ItemNotFoundException e) {
      }
    } else {
      for (int i = getDepth(); i >= 0; i--) {
        NodeImpl ancestor = (NodeImpl) getAncestor(i);
        if (ancestor.isNodeType("mix:referenceable")) {
          NodeImpl correspAncestor = (NodeImpl) correspSession.getNodeByUUID(ancestor.getUUID());
          JCRPath.PathElement[] relJCRPath = getLocation().getRelPath(getDepth() - i);
          // System.out.println("LOC. getRelPath "+(getDepth() - i)+" =
          // "+getRelPath(relJCRPath)+" "+getPath());
          try {
            return (NodeImpl) correspAncestor.getNode(getRelPath(relJCRPath));
          } catch (ItemNotFoundException e) {
          }
        }
      }
    }
    try {
      return (NodeImpl) correspSession.getItem(getPath());
    } catch (PathNotFoundException e) {
      throw new ItemNotFoundException("No corresponding path for " + getPath() + " in "
          + correspSession.getWorkspace().getName());
    }
  }

  private String getRelPath(JCRPath.PathElement[] relPath) throws RepositoryException {
    String path = "";
    for (int i = 0; i < relPath.length; i++) {
      path += relPath[i].getAsString(false);
      if (i < relPath.length - 1)
        path += "/";
    }
    return path;
  }

  /**
   * @see javax.jcr.Node#update
   */
  public void update(String srcWorkspaceName) throws NoSuchWorkspaceException,
      AccessDeniedException, InvalidItemStateException, LockException, RepositoryException {

    // Check pending changes
    if (session.hasPendingChanges())
      throw new InvalidItemStateException("Session has pending changes ");

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");

    String srcPath;
    try {
      srcPath = getCorrespondingNodePath(srcWorkspaceName);
      remove();
      save();
    } catch (ItemNotFoundException e) {
      log.debug("No corresponding node in workspace: " + srcWorkspaceName);
      return;
    }
    session.getWorkspace().clone(srcWorkspaceName, srcPath, this.getPath(), true);
//    reloadNodeData();
  }


  /**
   * @see javax.jcr.Node#addMixin
   */
  public void addMixin(String mixinName) throws NoSuchNodeTypeException,
      ConstraintViolationException, VersionException, LockException, RepositoryException {

    InternalQName name = locationFactory.parseJCRName(mixinName).getInternalName(); 
    doAddMixin(name);
  }

  public void doAddMixin(InternalQName mixinName) throws NoSuchNodeTypeException,
      ConstraintViolationException, VersionException, LockException, RepositoryException {

    
    // Add both to mixinNodeTypes and to jcr:mixinTypes property
    NodeType type = nodeType(mixinName);
    if(log.isDebugEnabled())
      log.debug("Node.addMixin " + mixinName + " " + getPath());
    
    // Mixin or not
    if (type == null || !type.isMixin())
      throw new NoSuchNodeTypeException("Node Type " + mixinName
          + " not found or not mixin type.");

    // Validate
    if (hasSameOrSubtypeMixin((NodeTypeImpl) type))
      throw new ConstraintViolationException("Can not add mixin type "
          + mixinName);

    if (getDefinition().isProtected())
      throw new ConstraintViolationException(
          "Can not add mixin type. Node is protected " + getPath());

    // Check if versionable ancestor is not checked-in
    if (!isCheckedOut())
      throw new VersionException("Node " + getPath()
          + " or its nearest ancestor is checked-in");

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");

    InternalQName[] mixinTypes = nodeData().getMixinTypeNames();
    for (int i = 0; i < mixinTypes.length; i++) {
      if (type.equals(nodeType(mixinTypes[i]))) {
        // we already have this mixin type 
        log.warn("Node.addMixin node already has this mixin type " + mixinName + " " + getPath());
        return;
      }
    }

    // Prepare mixin values
    InternalQName[] newMixin = new InternalQName[mixinTypes.length + 1];
    List <Value> values = new ArrayList<Value>();

    for (int i = 0; i < mixinTypes.length; i++) {
      newMixin[i] = mixinTypes[i];
      JCRName jcrName = locationFactory.createJCRName(mixinTypes[i]);
      values.add(valueFactory.createValue(jcrName));
    }
    newMixin[mixinTypes.length] = mixinName;
    values.add(valueFactory.createValue(locationFactory.createJCRName(mixinName)));

    // Set mixin property and nodeData
    //boolean firstMixin = (mixinTypes.length == 0);
    //mixinTypes = newMixin;
    // Should register jcr:mixinTypes and autocreated items if node is not added
    updateMixin(newMixin);
    PropertyImpl prop = doSetProperty(Constants.JCR_MIXINTYPES, values, PropertyType.NAME, true);
    addAutoCreatedItems(mixinName);
    
    if(log.isDebugEnabled())
      log.debug("Node.addMixin Property " + prop.getPath() + " values " + mixinTypes.length);

    // Add autocreated items for this nodeType
//    if (addAutoCreated)
//      addAutoCreatedItems(nodeType(locationFactory.parseJCRName(mixinName).getInternalName()), registerInDataManager);

  }

  /**
   * @see javax.jcr.Node#removeMixin
   */
  public void removeMixin(String mixinName) throws NoSuchNodeTypeException,
      ConstraintViolationException, RepositoryException {

    InternalQName[] mixinTypes = nodeData().getMixinTypeNames();

    // find mixin
    int ind = -1;
    for (int i = 0; i < mixinTypes.length; i++) {
      if (mixinTypes[i].getAsString().equals(mixinName))
        ind = i;
    }
    // no mixin found
    if (ind < 0)
      throw new NoSuchNodeTypeException("No mixin type found " + mixinName + " for node "
          + getPath());

    // A ConstraintViolationException will be thrown either
    // immediately or on save if the removal of a mixin is not
    // allowed. Implementations are free to enforce any policy
    // they like with regard to mixin removal and may differ on
    // when this validation is done.

    // Check if versionable ancestor is not checked-in
    if (!isCheckedOut())
      throw new VersionException("Node " + getPath() + " or its nearest ancestor is checked-in");

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");

    // Prepare mixin values
    InternalQName[] newMixin = new InternalQName[mixinTypes.length - 1];
    List <Value> values = new ArrayList<Value>();
    for (int i = 0; i < newMixin.length; i++) {
      if (i != ind) {
        newMixin[i] = mixinTypes[i];
        JCRName mixinJCRName = locationFactory.createJCRName(newMixin[i]);
        values.add(valueFactory.createValue(mixinJCRName));
      }
    }

    // Set mixin property and locally
//    PropertyImpl prop = doSetProperty("jcr:mixinTypes", values, PropertyType.NAME, true,
//        registerInDataManager, false);
    updateMixin(newMixin);
    PropertyImpl prop = doSetProperty(Constants.JCR_MIXINTYPES, values, PropertyType.NAME, true);
  }
  
  private void updateMixin(InternalQName[] newMixin) throws RepositoryException {
    TransientNodeData newData =  new TransientNodeData(getInternalPath(), 
        getInternalUUID(), nodeData().getPersistedVersion(),
        nodeData().getPrimaryTypeName(), newMixin,
        nodeData().getOrderNumber(), nodeData().getParentUUID(), 
        nodeData().getACL());
    ItemState mixinChanged =  new ItemState(newData, ItemState.MIXIN_CHANGED, true, null);
    dataManager.update(mixinChanged, false);
  }

  /**
   * @see javax.jcr.Node#canAddMixin
   */
  public boolean canAddMixin(String mixinName) throws RepositoryException {

//    if (hasSameOrSubtypeMixin((NodeTypeImpl) nodeType(mixinName))) {
    if (hasSameOrSubtypeMixin(nodeType(locationFactory.parseJCRName(mixinName).getInternalName()))) {
      return false;
    }

    if (getDefinition().isProtected()) {
      return false;
    }

    if (!isCheckedOut())
      return false;

    if (!checkLocking())
      return false;

    return true;
  }

  private boolean hasSameOrSubtypeMixin(NodeTypeImpl type) throws RepositoryException,
      ValueFormatException {
    
    InternalQName[] mixinTypes = nodeData().getMixinTypeNames();

    for (int i = 0; i < mixinTypes.length; i++) {
      
      if (NodeTypeImpl.isSameOrSubType(type, nodeType(mixinTypes[i]))) {
        return true;
      }
    }
    return false;

  }

  /**
   * @see javax.jcr.Node#orderBefore
   */
  public void orderBefore(String srcName, String destName)
      throws UnsupportedRepositoryOperationException, ConstraintViolationException,
      ItemNotFoundException, RepositoryException {

    if (!getPrimaryNodeType().hasOrderableChildNodes())
      throw new UnsupportedRepositoryOperationException("Node does not support child ordering "
          + getPrimaryNodeType().getName());

    if (srcName.equals(destName))
      return;

    EntityCollection nodes = (EntityCollection) getNodes();
    NodeImpl srcNode = null;
    NodeImpl destNode = null;
    int size = (int) nodes.getSize();
    if (size < 2)
      throw new UnsupportedRepositoryOperationException("Nothing to order Count of child nodes "
          + size);

    // index number of each element equlal of NodeImpl.orderNumber
    List<NodeImpl> list = nodes.getList();
    List<NodeImpl> prevOrderList = new ArrayList<NodeImpl>(list);

    if (destName == null)
      destNode = list.get(size - 1);

    // TODO
    // reorder nodes making new TransientNodeData/State and use updateOrder for each changed
    
    // find dest and src nodes
    while (nodes.hasNext()) {
      NodeImpl node = (NodeImpl) nodes.nextNode();
      boolean withIndex = srcName.endsWith("]");
      if (node.getLocation().getAsString(withIndex).endsWith(srcName)) {
        srcNode = node;
      }      
      if (destName != null) {
        withIndex = destName.endsWith("]");
        if (node.getLocation().getAsString(withIndex).endsWith(destName)) {
          destNode = node;
        }
      }
    }

    if (srcNode == null)
      throw new ItemNotFoundException("Src Node not found " + srcName);
    if (destNode == null)
      throw new ItemNotFoundException("Dest Node not found " + destName);
    
    // if sameNameSablings and one name nodes ordered swaping lock state
    // it's a play for next reordering 
    if (srcNode.getName().equals(destNode.getName()) 
        && srcNode.getIndex() != destNode.getIndex()
        && srcNode.isLocked() && !destNode.isLocked()) {
      // make locked current node
      Lock lock = srcNode.getLock();
      destNode.lockInSession(lock.isDeep(), lock.isSessionScoped());
      // unclock prevNode
      srcNode.unlockInSession();
    }    

    // System.out.println("ORDER "+srcNode.getName()+"
    // "+srcNode.getOrderNumber()+" "+destNode.getName()+"
    // "+destNode.getOrderNumber());

    // reorder nodes
    list.remove(srcNode.getOrderNumber());            
    list.add(destNode.getOrderNumber(), srcNode);
    
    int srcOrderNumber = -1;
    int destOrderNumber = -1;
    if (destName == null) {
      // TODO [PN] srcNode.setOrderDirection(Event.NODE_ADDED); // order source node up to head
      //srcOrderNumber = destNode.getOrderNumber() + 1;
      destOrderNumber = srcNode.getOrderNumber() + 1;
    } else if (srcNode.getName().equals(destNode.getName())) {
      // TODO [PN] srcNode.setOrderDirection(Event.NODE_ADDED); // order source node up to head
      srcOrderNumber = srcNode.getOrderNumber();
      destOrderNumber = destNode.getOrderNumber();
    } else {
      // TODO [PN] srcNode.setOrderDirection(Event.NODE_ADDED | Event.NODE_REMOVED); // order source node up to head      
    }
    // log.info("==== reorder source: " + srcNode.getPath() + " order numb: " + srcNode.getOrderNumber());
    // log.info("==== reorder dest: " + destNode.getPath() + " order numb: " + destNode.getOrderNumber());
    for (int i = 0; i < list.size(); i++) {
      NodeImpl node = list.get(i);
      // log.info("==== reorder node: " + node.getLocation().getAsString(true) + " from " + node.getOrderNumber() + " to " + i);
      if (srcNode.getName().equals(node.getName())
          && node.getOrderNumber() >= destOrderNumber
          && node.getOrderNumber() < srcOrderNumber) {
        // Same name sibs for source node
        // TODO [PN] node.setOrderDirection(Event.NODE_REMOVED); // order destination node down to end
      }
      // TODO [PN] node.setOrderNumber(i);
      dataManager.update(ItemState.createUpdated(node.getData()), true);
    }
  }
  
  
  private int getOrderNumber() {
    return nodeData().getOrderNumber();
  }


  // //////////////////////// OPTIONAL

  // VERSIONING

  public Version checkin() throws VersionException, UnsupportedRepositoryOperationException,
      InvalidItemStateException, RepositoryException {
    
    if (!this.isNodeType(Constants.MIX_VERSIONABLE))
      throw new UnsupportedRepositoryOperationException(
          "Node.checkin() is not supported for not mix:versionable node ");
    if (!this.isCheckedOut())
      return this.getBaseVersion();
    if (session.getTransientNodesManager().hasPendingChanges(getInternalPath()))
      throw new InvalidItemStateException("Node has pending changes " + getPath());
    if (hasProperty(Constants.JCR_MERGEFAILED))
      throw new VersionException("Node has jcr:mergeFailed " + getPath());

    String verUuid = UUIDGenerator.generate();
    SessionChangesLog changesLog = new SessionChangesLog(session.getId());

    PropertyData propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_ISCHECKEDOUT, PropertyType.BOOLEAN, 
        false, new TransientValueData(false));
    changesLog.add(ItemState.createAdded(propData)); 
    
    propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_BASEVERSION, PropertyType.REFERENCE, 
        false, new TransientValueData(new Uuid(verUuid)));
    changesLog.add(ItemState.createAdded(propData));

    propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_PREDECESSORS, PropertyType.REFERENCE, 
        true);
    changesLog.add(ItemState.createAdded(propData));

    ((VersionHistoryImpl)getVersionHistory()).addVersion(this, verUuid, changesLog);
    
    dataManager.getTransactManager().save(changesLog);

    VersionImpl version = (VersionImpl)dataManager.getItemByUUID(verUuid, true);
 
    return version;
  }

  public void checkout() throws RepositoryException, UnsupportedRepositoryOperationException {
    if (!this.isNodeType(Constants.MIX_VERSIONABLE))
      throw new UnsupportedRepositoryOperationException(
          "Node.checkout() is not supported for not mix:versionable node ");

    SessionChangesLog changesLog = new SessionChangesLog(session.getId());

    PropertyData propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_ISCHECKEDOUT, PropertyType.BOOLEAN, 
        false, new TransientValueData(true));
    changesLog.add(ItemState.createAdded(propData)); 
    
    ValueData baseVersion = (ValueData)((PropertyData)dataManager.getItemData(InternalQPath.makeChildPath(
          getInternalPath(), Constants.JCR_BASEVERSION))).getValues().get(0);

//    List<ValueData> baseVersion = dataManager.getPropertyValueData(
//        InternalQPath.makeChildPath(
//          getInternalPath(), 
//          Constants.JCR_BASEVERSION));
    
    propData = TransientPropertyData.createPropertyData(
        nodeData(), 
        Constants.JCR_PREDECESSORS, 
        PropertyType.REFERENCE, 
        false, 
        baseVersion /*.get(0)*/);
    changesLog.add(ItemState.createAdded(propData));
    
    dataManager.getTransactManager().save(changesLog);
  }

  public boolean isCheckedOut() throws UnsupportedRepositoryOperationException, RepositoryException {

    if (isRoot())
      return true;

    if (this.isNodeType(Constants.MIX_VERSIONABLE)) {
      return property(Constants.JCR_ISCHECKEDOUT).getBoolean();
    }

    NodeImpl ancestor = (NodeImpl) getParent();
    while (!ancestor.isRoot()) {
      if (ancestor.isNodeType("mix:versionable"))
        return ancestor.isCheckedOut();
      else
        ancestor = (NodeImpl) ancestor.getParent();
    }
    if (ancestor.isNodeType("mix:versionable"))
      return ancestor.isCheckedOut();
    else
      return true;
  }

  /* (non-Javadoc)
   * @see javax.jcr.Node#getVersionHistory()
   */
  public VersionHistory getVersionHistory() throws UnsupportedRepositoryOperationException,
      RepositoryException {

    if (!this.isNodeType(Constants.MIX_VERSIONABLE))
      throw new UnsupportedRepositoryOperationException("Node is not mix:versionable " + getPath());

    PropertyImpl versionHistoryProp = property(Constants.JCR_VERSIONHISTORY);
    if(versionHistoryProp == null)
      throw new UnsupportedRepositoryOperationException("Node does not have jcr:versionHistory: " + getPath());
    //String versionHistory = getProperty("jcr:versionHistory").getString();

    return (VersionHistory) session.getNodeByUUID(versionHistoryProp.getString());
  }

  public Version getBaseVersion() throws UnsupportedRepositoryOperationException,
      RepositoryException {

    if (!this.isNodeType(Constants.MIX_VERSIONABLE))
      throw new UnsupportedRepositoryOperationException("Node is not versionable " + getPath());

    Version v = (Version) session.getNodeByUUID(property(Constants.JCR_BASEVERSION).getString());

    return v;
  }

  public void restore(Version version, boolean removeExisting) throws VersionException,
      ItemExistsException, UnsupportedRepositoryOperationException, LockException,
      RepositoryException, InvalidItemStateException {

    try {
    
    if (!this.isNodeType(Constants.MIX_VERSIONABLE))
      throw new UnsupportedRepositoryOperationException("Node is not versionable " + getPath());

    if (session.hasPendingChanges())
      throw new InvalidItemStateException("Session has pending changes ");

    if (((VersionImpl)version).getInternalName().equals(Constants.JCR_ROOTVERSION))
      throw new VersionException("It is illegal to call restore() on jcr:rootVersion");

    if (!((VersionHistoryImpl) getVersionHistory()).isVersionBelongToThis(version))
      throw new VersionException("Bad version " + version.getPath());

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");

    NodeImpl restored = ((VersionImpl) version).restore(this, removeExisting);

    
    } catch(UnsupportedRepositoryOperationException e) {
      //log.error("RESTORE: " + e.getMessage(), e);
      throw e;
    } catch(VersionException e) {
      //log.error("RESTORE: " + e.getMessage(), e);
      throw e;  
    } catch(InvalidItemStateException e) {
      //log.error("RESTORE: " + e.getMessage(), e);
      throw e;  
    } catch(RepositoryException e) {
      //log.error("RESTORE: " + e.getMessage(), e);
      throw e;
    }
  }

  public void restore(String versionName, boolean removeExisting) throws VersionException,
      ItemExistsException, UnsupportedRepositoryOperationException, LockException,
      RepositoryException, InvalidItemStateException {

    // throw VersionException if not found
    VersionImpl version = (VersionImpl) getVersionHistory().getVersion(versionName);
    restore(version, removeExisting);

  }

  public void restore(Version version, String relPath, boolean removeExisting)
      throws VersionException, ItemExistsException, UnsupportedRepositoryOperationException,
      LockException, RepositoryException, InvalidItemStateException {

    NodeImpl node;
    //JCRPath newPath = locationFactory.createJCRPath(getLocation(), relPath);
    InternalQPath newPath = locationFactory.createJCRPath(getLocation(), relPath).getInternalPath();
    try {
      //node = (NodeImpl) getNode(relPath);
      node = (NodeImpl) dataManager.getItem(newPath, true);
    } catch (PathNotFoundException e) {
      // throw if not found
      //NodeImpl pNode = (NodeImpl) session.getItem(newPath.makeParentPath().getAsString(false));
      // make dummy node in memory (nodeType will be replaced)
      //node = this.createChildNode(pNode.getName(), "nt:base", true);
      
      NodeData parentData = (NodeData) dataManager.getItemData(
          newPath.makeParentPath() 
          /*false*/);
      NodeData nodeData = TransientNodeData.createNodeData(
          parentData, 
          newPath.getName(), 
          Constants.NT_BASE);
      
      dataManager.update(ItemState.createAdded(nodeData), true);
      
      node = (NodeImpl) dataManager.getItem(newPath, true);
    }

    node.restore(version, removeExisting);
  }

  public void restoreByLabel(String versionLabel, boolean removeExisting) throws VersionException,
      ItemExistsException, UnsupportedRepositoryOperationException, LockException,
      RepositoryException, InvalidItemStateException {

    VersionImpl version = (VersionImpl) getVersionHistory().getVersionByLabel(versionLabel);
    restore(version, removeExisting);

  }

  public NodeIterator merge(String srcWorkspace, boolean bestEffort)
      throws UnsupportedRepositoryOperationException, NoSuchWorkspaceException,
      AccessDeniedException, MergeException, RepositoryException, InvalidItemStateException {

    if (session.hasPendingChanges())
      throw new InvalidItemStateException("Session has pending changes ");

    Map<String, String> failed = new HashMap<String, String>();

    // get corresponding node
    SessionImpl srcSession = (SessionImpl) ((RepositoryImpl) session.getRepository()).login(session
        .getCredentials(), srcWorkspace);
    NodeImpl correspNode;
    try {
      correspNode = getCorrespondingNode(srcSession);
    } catch (ItemNotFoundException e) {
      log.debug("No corresponding node in workspace: " + srcWorkspace);
      return new EntityCollection();
    }

    MergeVisitor visitor = new MergeVisitor(this, failed, bestEffort);
    correspNode.accept(visitor);
    
    // [PN] 21.08.06
    //----------- END SESSION BATCH ----------    
//    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
//    state = ItemChangeState.DELETED;
//    dataManager.getTransactManager().saveItem(this);
//    
//    state = ItemChangeState.ADDED;
//    dataManager.getTransactManager().saveItem(this);
//    dataManager.getWorkspaceDataManager().endSessionBatch(session);
    
//  changesLog.add(ItemState.createDeleted(this.getData()));
    MutableItemDataChangesLog changesLog = new SessionChangesLog(session.getId())
      .add(ItemState.createAdded(this.getData()));

    dataManager.getTransactManager().save(changesLog);    
    //----------- END SESSION BATCH ----------
    
    NodeIterator failedIter = createMergeFailed(failed); 
    
    // Clean both session and ws cache
//    for (ItemImpl cItem: visitor.getCleanupList()) {
//      dataManager.removeFromCache(cItem);
//    }
    
    return failedIter;
  }

  private NodeIterator createMergeFailed(Map<String, String> failed) throws RepositoryException {

    EntityCollection res = new EntityCollection();

    for (String uuid: failed.keySet()) {  
      NodeImpl versionable = (NodeImpl) session.getNodeByUUID(uuid);
      res.add(versionable);
      String offendingUuid = failed.get(uuid);
      //Value[] mergeFailed;
      List<ValueData> mergeFailedRefs = new ArrayList<ValueData>();
      try {
        mergeFailedRefs = (List <ValueData>)((PropertyData)dataManager.getItemData(
            InternalQPath.makeChildPath(getInternalPath(), Constants.JCR_MERGEFAILED))).getValues();
      } catch (PathNotFoundException e) {
      }

//      List<ValueData> mergeFailedRefs = dataManager.getPropertyValueData(
//          InternalQPath.makeChildPath(versionable.getInternalPath(), Constants.JCR_MERGEFAILED)); 
//      try {
//        //mergeFailed = versionable.getProperty("jcr:mergeFailed").getValues();
//        mergeFailedRefs = dataManager.getPropertyValueData(
//            InternalQPath.makeChildPath(versionable.getInternalPath(), Constants.JCR_MERGEFAILED));
//      } catch (PathNotFoundException e) {
//        //mergeFailed = new Value[0];
//        mergeFailedRefs = new ArrayList<ValueData>();
//      }

      List<ValueData> newMergeFailedRefs = new ArrayList<ValueData>();
      for (ValueData vd: mergeFailedRefs) {
        Value val = valueFactory.loadValue(vd, PropertyType.REFERENCE);
        if (val.getString().equals(offendingUuid)) {
          newMergeFailedRefs = null;
          break;
        }
        newMergeFailedRefs.add(ItemDataFactory.makeValueData(vd));
      }
      
      // if not exists add "new" value and refresh jcr:mergeFailed property
      //if (!mf.contains(offendingUuid)) {
      if (newMergeFailedRefs != null) {
        newMergeFailedRefs.add(new TransientValueData(offendingUuid, mergeFailedRefs.size()));
        
        MutableItemDataChangesLog changesLog = new SessionChangesLog(session.getId());
        
        PropertyData mergeFailed = TransientPropertyData.createPropertyData(
            (NodeData) getData(), 
            Constants.JCR_MERGEFAILED, 
            PropertyType.REFERENCE, 
            true, 
            newMergeFailedRefs);
        
        changesLog.add(ItemState.createUpdated(mergeFailed)); 
        dataManager.getTransactManager().save(changesLog);
      }
    }

    return res;
  }

  public void doneMerge(Version version) throws VersionException, InvalidItemStateException,
      UnsupportedRepositoryOperationException, RepositoryException {
    
    MutableItemDataChangesLog changesLog = new SessionChangesLog(session.getId());
    
    VersionImpl base = (VersionImpl) getBaseVersion();
    base.addPredecessor(version, changesLog);
    removeMergeFailed(version, changesLog);
    
    dataManager.getTransactManager().save(changesLog);
  }

  public void cancelMerge(Version version) throws VersionException, InvalidItemStateException,
      UnsupportedRepositoryOperationException, RepositoryException {
    
    MutableItemDataChangesLog changesLog = new SessionChangesLog(session.getId());
    
    removeMergeFailed(version, changesLog);
    
    dataManager.getTransactManager().save(changesLog);
  }

  private void removeMergeFailed(Version version, MutableItemDataChangesLog changesLog) throws RepositoryException {
    
    PropertyData mergeFailed = (PropertyData) dataManager.getItem(
        InternalQPath.makeChildPath(getInternalPath(), Constants.JCR_MERGEFAILED), true);  
    if (mergeFailed == null)
      return;
    
//    if (!hasProperty("jcr:mergeFailed"))
//      return;    
    //PropertyImpl mergeFailed = (PropertyImpl) getProperty("jcr:mergeFailed");
    //Value[] mergeFailedRefs = mergeFailed.getValues();
//    List<String> mf = new ArrayList<String>();
//    for (Value mfv: mergeFailedRefs) {
//      if (!mfv.getString().equals(version.getUUID())) {
//        mf.add(mfv.getString());
//      }
//    }
    List<ValueData> mf = new ArrayList<ValueData>();
    for (ValueData mfvd: (List <ValueData>)mergeFailed.getValues()) {
      Value val = valueFactory.loadValue(mfvd, PropertyType.REFERENCE);
      if (!val.getString().equals(version.getUUID())) {
        mf.add(mfvd);
      }
    }
   
    changesLog.add(ItemState.createAdded(this.getData()));
    
    if (mf.size() > 0) {
//      mergeFailedRefs = new Value[mf.size()];
//      for (int i = 0; i < mergeFailedRefs.length; i++) {
//        mergeFailedRefs[i] = valueFactory.createValue(mf.get(i), PropertyType.REFERENCE);
//      }
      // remove prev object stores prop jcr:mergeFailed
      //mergeFailed.removeFromParent(ItemChangeState.UNCHANGED);
      // one new for prop jcr:mergeFailed
      //mergeFailed = createChildProperty("jcr:mergeFailed", mergeFailedRefs, PropertyType.REFERENCE);
      PropertyData mergeFailedRef = TransientPropertyData.createPropertyData(
          nodeData(), 
          Constants.JCR_MERGEFAILED, 
          PropertyType.REFERENCE, 
          true, 
          mf);
      changesLog.add(ItemState.createUpdated(mergeFailedRef));
    }
    
    // save outside the method in log owner
//    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
//    dataManager.getTransactManager().saveItem(mergeFailed);
//    dataManager.getWorkspaceDataManager().endSessionBatch(session);
  }

  // Locks
  public Lock lock(boolean isDeep, boolean isSessionScoped)
      throws UnsupportedRepositoryOperationException, LockException, AccessDeniedException,
      RepositoryException {

    if (!isNodeType(Constants.MIX_LOCKABLE))
      throw new LockException("Node is not lockable " + getPath());

    Lock lock = session.getLockManager().getLock(this, false);
    if (lock != null)
      throw new LockException("Node is already locked " + getPath() + " on "
          + lock.getNode().getPath());

    if (isDeep && session.getLockManager().getChildLock(this) != null)
      throw new LockException("Lock is deep and node already has locked child " + getPath());

    if (dataManager.hasPendingChanges(getInternalPath()))
      throw new InvalidItemStateException("Node has pending unsaved changes " + getPath());

    SessionChangesLog changesLog = new SessionChangesLog(session.getId());

    PropertyData propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_LOCKOWNER, PropertyType.STRING, 
        false, new TransientValueData(session.getUserID()));
    changesLog.add(ItemState.createAdded(propData)); 

    propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_LOCKISDEEP, PropertyType.BOOLEAN, 
        false, new TransientValueData(isDeep));
    changesLog.add(ItemState.createAdded(propData)); 

    dataManager.getTransactManager().save(changesLog);

//    PropertyImpl lockOwner = createChildProperty("jcr:lockOwner", new Value[] { valueFactory
//        .createValue(session.getUserID()) }, PropertyType.STRING);
//    PropertyImpl lockIsDeep = createChildProperty("jcr:lockIsDeep", new Value[] { valueFactory
//        .createValue(isDeep) }, PropertyType.BOOLEAN);
//    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
//    dataManager.getTransactManager().saveItem(lockOwner);
//    dataManager.getTransactManager().saveItem(lockIsDeep);
//    //dataManager.getWorkspaceDataManager().add(lockOwner);
//    //dataManager.getWorkspaceDataManager().add(lockIsDeep);
//    dataManager.getWorkspaceDataManager().endSessionBatch(session);

    Lock newLock = session.getLockManager().addLock(this, isDeep, isSessionScoped);
    return newLock;

  }
  
  /**
   * Lock node in session using SessionDataManager instead WorkspaceManager. 
   * Used in orderBefore, for relock nodes with sameNameSibs and index change.
   * We can't use Node.lock() because it works direct with workspace and needs save after that.
   * But we need one session scope in orderBefore for observation works true.
   * */
  protected Lock lockInSession(boolean isDeep, boolean isSessionScoped)
      throws UnsupportedRepositoryOperationException, LockException, AccessDeniedException,
      RepositoryException {

    if (!isNodeType(Constants.MIX_LOCKABLE))
      throw new LockException("Node is not lockable " + getPath());

    Lock lock = session.getLockManager().getLock(this, false);
    if (lock != null)
      throw new LockException("Node is already locked " + getPath() + " on "
          + lock.getNode().getPath());

    if (isDeep && session.getLockManager().getChildLock(this) != null)
      throw new LockException("Lock is deep and node already has locked child " + getPath());

    if (dataManager.hasPendingChanges(getInternalPath()))
      throw new InvalidItemStateException("Node has pending unsaved changes " + getPath());

    PropertyData propData = TransientPropertyData.createPropertyData(nodeData(), 
        Constants.JCR_LOCKOWNER, PropertyType.STRING, false, new TransientValueData(session.getUserID()));
    dataManager.update(ItemState.createAdded(propData), true);

    propData = TransientPropertyData.createPropertyData(nodeData(), Constants.JCR_LOCKISDEEP, PropertyType.BOOLEAN, 
        false, new TransientValueData(isDeep));
    dataManager.update(ItemState.createAdded(propData), true);
        
    Lock newLock = session.getLockManager().addLock(this, isDeep, isSessionScoped);
    return newLock;
  }

  public Lock getLock() throws UnsupportedRepositoryOperationException, LockException,
      AccessDeniedException, RepositoryException {
    LockImpl lock = session.getLockManager().getLock(this, false);
    if (lock == null)
      throw new LockException("Lock not found " + getPath());
    return lock;

  }

  public void unlock() throws UnsupportedRepositoryOperationException, LockException,
      AccessDeniedException, RepositoryException {

    if (!session.hasLockToken(session.getLockManager().getLockToken(this)))
      throw new LockException("There are no permission to unlock the node or node is not locked "
          + getPath());

    if (dataManager.hasPendingChanges(getInternalPath()))
      throw new InvalidItemStateException("Node has pending unsaved changes " + getPath());

    SessionChangesLog changesLog = new SessionChangesLog(session.getId());

    changesLog.add(ItemState.createDeleted(
        TransientPropertyData.createPropertyData(
            nodeData(), 
            Constants.JCR_LOCKOWNER, 
            PropertyType.STRING, 
            false)));
    changesLog.add(ItemState.createDeleted(
        TransientPropertyData.createPropertyData(
            nodeData(), 
            Constants.JCR_LOCKISDEEP, 
            PropertyType.BOOLEAN, 
            false)));

    dataManager.getTransactManager().save(changesLog);

//    PropertyImpl lockOwner = (PropertyImpl) getProperty("jcr:lockOwner");
//    PropertyImpl lockIsDeep = (PropertyImpl) getProperty("jcr:lockIsDeep");
//
//    // [PN] 09.04.06
//    //removeChildProperty(lockOwner);
//    //removeChildProperty(lockIsDeep);
//
//    //----------- BEGIN SESSION BATCH ----------    
//    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
//
//    lockOwner.removeFromParent();
//    dataManager.getTransactManager().saveItem(lockOwner);
//    //dataManager.getWorkspaceDataManager().delete(lockOwner);
//    
//    lockIsDeep.removeFromParent();
//    dataManager.getTransactManager().saveItem(lockIsDeep);
//    //dataManager.getWorkspaceDataManager().delete(lockIsDeep);
//    
//    dataManager.getWorkspaceDataManager().endSessionBatch(session);
//    //----------- END SESSION BATCH ----------    
//
//    dataManager.resetItemState(lockOwner);
//    dataManager.resetItemState(lockIsDeep);
//    
//    dataManager.removeFromCache(this);
//    
    session.getLockManager().removeLock(this);
  }

  /**
   * Unlock node in session using SessionDataManager instead WorkspaceManager. 
   * Used in orderBefore, for relock nodes with sameNameSibs and index change.
   * We can't use Node.unlock() because it works direct with workspace and needs save after that.
   * But we need one session scope in orderBefore for observation works true.
   * */
  protected void unlockInSession() throws UnsupportedRepositoryOperationException, LockException,
      AccessDeniedException, RepositoryException {

    if (!session.hasLockToken(session.getLockManager().getLockToken(this)))
      throw new LockException("There are no permission to unlock the node or node is not locked "
          + getPath());

    if (dataManager.hasPendingChanges(getInternalPath()))
      throw new InvalidItemStateException("Node has pending unsaved changes " + getPath());

    // TODO use existing property instead of one new  
    dataManager.update(ItemState.createDeleted(
        TransientPropertyData.createPropertyData(nodeData(), 
            Constants.JCR_LOCKOWNER, PropertyType.STRING, false)), true);
    dataManager.update(ItemState.createDeleted(
        TransientPropertyData.createPropertyData(nodeData(), 
            Constants.JCR_LOCKISDEEP, PropertyType.BOOLEAN, false)), true);

    session.getLockManager().removeLock(this);
  }

  public boolean holdsLock() throws RepositoryException {
    return (session.getLockManager().getLock(this, true) != null);
  }

  public boolean isLocked() throws RepositoryException {
    return (session.getLockManager().getLock(this, false) != null);
  }

  boolean checkLocking() throws RepositoryException {
    return (!isLocked() || session.hasLockToken(session.getLockManager().getLockToken(this)) || session
        .getUserID().equals(SystemIdentity.SYSTEM));
  }

  // ////////////////// Item implementation ////////////////////

  /**
   * @see javax.jcr.Item#accept
   */
  public void accept(ItemVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  /**
   * @see javax.jcr.Item#isNode
   */
  public boolean isNode() {
    return true;
  }

  // ////////////////////////////////////////////////////

  /**
   * Add autocreated items to this node. 
   * No checks will be passed for autocreated items. 
   */
  public void addAutoCreatedItems(InternalQName nodeTypeName)
      throws RepositoryException, ConstraintViolationException {

    NodeTypeImpl type = nodeType(nodeTypeName);
    NodeDefinition[] nodeDefs = type.getChildNodeDefinitions();
    PropertyDefinition[] propDefs = type.getPropertyDefinitions();

    // Add autocreated child properties
    for (int i = 0; i < propDefs.length; i++) {

      if (propDefs[i] == null) // it is possible for not mandatory propDef
        continue;

      if (propDefs[i].isAutoCreated()) {
        PropertyDefinitionImpl pdImpl = (PropertyDefinitionImpl)  propDefs[i];
        List<ValueData> vals = new ArrayList<ValueData>();  
        Value[] propVal = pdImpl.getDefaultValues();
        // there can be null in definition but should not be null value
//        if (propVal == null) {
//          propVal = new Value[1];
//          propVal[0] = null;
//        } 
        if (propVal != null) {
          for (Value v: propVal) {
            if (v != null)
              vals.add(((BaseValue) v).getData());
          }
        }
//        JCRPath loc = session.getLocationFactory().createJCRPath(getLocation(), pdImpl.getName());
//        PropertyImpl prop = doSetProperty(pdImpl.getName(), propVal, pdImpl.getRequiredType(), pdImpl.isMultiple(),
//            registerInDataManager, false);
        
        dataManager.update(ItemState.createAdded(
            TransientPropertyData.createPropertyData(
                nodeData(), 
                pdImpl.getQName(), 
                pdImpl.getRequiredType(), 
                pdImpl.isMultiple(),
                vals)),
                true);
      }
    }

    // Add autocreated child nodes
    for (int i = 0; i < nodeDefs.length; i++) {
      if (nodeDefs[i].isAutoCreated()) {
//        doAddNode(this, InternalQName.parse(nodeDefs[i].getName()),
//            InternalQName.parse(nodeDefs[i].getDefaultPrimaryType().getName()), 
//            null, null, -1, // mixin, uuid, index
//            registerInDataManager, 
//            false, // doExternalValidation=false 
//            true);

        NodeDefinitionImpl ndImpl = (NodeDefinitionImpl) nodeDefs[i];
        
        dataManager.update(ItemState.createAdded(
            TransientNodeData.createNodeData(
                nodeData(), 
                ndImpl.getQName(), 
                ((NodeTypeImpl) ndImpl.getDefaultPrimaryType()).getQName(), 
                UUIDGenerator.generate())),
                true);
      }
    }
    initAutoCreatedItems(type);
  }

  private NodeTypeImpl nodeType(InternalQName qName) throws NoSuchNodeTypeException, RepositoryException {
    NodeTypeImpl nt = ((NodeTypeManagerImpl)session.getWorkspace().getNodeTypeManager()).findNodeType(qName);
    if(nt == null)
      throw new NoSuchNodeTypeException("NodeTypeManager.getNodeType(): NodeType '" + qName.getAsString()
          + "' not found.");
    return nt;
  }

  private NodeType[] getAllNodeTypes() throws RepositoryException {
    NodeType primaryType = getPrimaryNodeType();
    NodeType[] mixinTypes = getMixinNodeTypes();
    NodeType[] types = new NodeType[mixinTypes.length + 1];
    types[0] = primaryType;
    for (int j = 0; j < mixinTypes.length; j++) {
      types[j + 1] = mixinTypes[j];
    }
    return types;
  }
  
  public PropertyDefinition findChildPropertyDefinition(InternalQName propertyName) 
  throws RepositoryException, ConstraintViolationException {
    NodeType[] allTypes = getAllNodeTypes();
    PropertyDefinition pdResidual = null; 
    for (NodeType nt: allTypes) {
      PropertyDefinitions pds = ((NodeTypeImpl) nt).getPropertyDefinitions(propertyName);
      PropertyDefinition pd = pds.getAnyDefinition();
      if ( pd != null) {
        if (((PropertyDefinitionImpl)pd).isResidualSet()) {
          pdResidual = pd;
        } else {
          return pd;
        }
      }
    }
    if(pdResidual == null)
      throw new ConstraintViolationException("Child property definition not found for "+propertyName+" "+getPath());
    return pdResidual;
  }

  public NodeDefinition findChildNodeDefinition(InternalQName nodeName) 
  throws RepositoryException, ConstraintViolationException {
    NodeDefinition ndResidual = null; 
    for (NodeType nt: getAllNodeTypes()) {
      NodeDefinitionImpl nd = (NodeDefinitionImpl)((NodeTypeImpl) nt).getChildNodeDefinition(nodeName); 
      if ( nd != null) {
        if (nd.isResidualSet()) {
          ndResidual = nd;
        } else {
          return nd;
        }
      }
    }
    if(ndResidual == null)
      throw new ConstraintViolationException("Child node definition not found for "+nodeName+" "+getPath());

    return ndResidual;
  }

  
  public String[] getMixinTypeNames() throws RepositoryException {
    NodeType[] mixinTypes = getMixinNodeTypes();
    String[] mtNames = new String[mixinTypes.length];
    for (int i = 0; i < mtNames.length; i++)
      mtNames[i] = mixinTypes[i].getName();
    return mtNames;
  }

  public void validateMandatoryChildren() throws ConstraintViolationException, RepositoryException {
    ArrayList mandatoryItemDefs = ((NodeTypeImpl) getPrimaryNodeType()).getManadatoryItemDefs();
    NodeType[] mixinTypes = getMixinNodeTypes();
    for (int i = 0; i < mixinTypes.length; i++)
      mandatoryItemDefs.addAll(((NodeTypeImpl) mixinTypes[i]).getManadatoryItemDefs());
    Iterator defs = mandatoryItemDefs.iterator();
    while (defs.hasNext()) {
      ItemDefinition def = (ItemDefinition) defs.next();
      if ((def instanceof NodeDefinition && !hasNode(def.getName()))
          || (def instanceof PropertyDefinition && !hasProperty(def.getName())))

        throw new ConstraintViolationException("Mandatory item " + def.getName()
            + " not found. Node [" + getPath() + " primary type: "
            + this.getPrimaryNodeType().getName() + "]");
    }
  }

  public void initAutoCreatedItems(NodeTypeImpl type)
      throws RepositoryException {

    if (type.isNodeType(Constants.NT_BASE)) {
//      PropertyImpl prop = doSetProperty("jcr:primaryType", new Value[] { valueFactory
//          .createValue(primaryType) }, PropertyType.NAME, false, false, false);

      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_PRIMARYTYPE, 
              PropertyType.NAME, 
              false,
              new TransientValueData(nodeData().getPrimaryTypeName()))),
              false); // no pool
    }
    if (type.isNodeType(Constants.MIX_REFERENCEABLE)) {
//      doSetProperty("jcr:uuid", new Value[] { valueFactory.createValue(internalUUID) },
//          PropertyType.STRING, false, false, false);
      
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_UUID, 
              PropertyType.STRING, 
              false,
              new TransientValueData(nodeData().getUUID()))),
              false); // no pool
    }
    if (type.isNodeType(Constants.NT_HIERARCHYNODE)) {
//      Value[] values = new Value[] { valueFactory.createValue(dataManager.getWorkspaceDataManager()
//          .getCurrentTime()) };
//      doSetProperty("jcr:created", values, PropertyType.DATE, false, false, false);
      
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_CREATED, 
              PropertyType.DATE, 
              false,
              new TransientValueData(dataManager.getTransactManager().getStorageDataManager().getCurrentTime()))),
              false); // no pool
    }
    if (!session.getAccessManager().getAccessControlPolicy().equals(AccessControlPolicy.DISABLE)
        && type.isNodeType(Constants.EXO_ACCESS_CONTROLLABLE)) {
      Map<String, String[]> perms = new HashMap<String, String[]>();
      perms.put(SystemIdentity.ANY, PermissionType.DEFAULT_AC);
      setPermissions(perms);
    }
    if (type.isNodeType(Constants.MIX_VERSIONABLE)) {

      // Generate uuids
      //Uuid versionHistoryUUID = uuidGenerator.generateUUID(getPath() + "/jcr:versionHistory");
      //Uuid baseVersionUUID = uuidGenerator.generateUUID(getPath() + "/jcr:baseVersion");

//      doSetProperty("jcr:versionHistory", new Value[] { valueFactory
//          .createValue(versionHistoryUUID) }, PropertyType.REFERENCE, false, registerInDataManager,
//          false);
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_VERSIONHISTORY, 
              PropertyType.REFERENCE, 
              false,
              new TransientValueData(UUIDGenerator.generate()))),
              false); // no pool
      
//      doSetProperty("jcr:baseVersion", new Value[] { valueFactory.createValue(baseVersionUUID) },
//          PropertyType.REFERENCE, false, registerInDataManager, false);
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_BASEVERSION, 
              PropertyType.REFERENCE, 
              false,
              new TransientValueData(UUIDGenerator.generate()))),
              false); // no pool
      
//      doSetProperty("jcr:predecessors", new Value[] { valueFactory.createValue(baseVersionUUID) },
//          PropertyType.REFERENCE, true, registerInDataManager, false);
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_PREDECESSORS, 
              PropertyType.REFERENCE, 
              false,
              new TransientValueData(UUIDGenerator.generate()))),
              false); // no pool
      
//      doSetProperty("jcr:isCheckedOut", new Value[] { valueFactory.createValue(true) },
//          PropertyType.BOOLEAN, false, registerInDataManager, false);
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.JCR_ISCHECKEDOUT, 
              PropertyType.BOOLEAN, 
              false,
              new TransientValueData(true))),
              false); // no pool      
    }
  }

  // ExtendedNode -----------------------------

  public void setPermissions(Map<String, String[]> permissions) throws RepositoryException,
      AccessDeniedException, AccessControlException {

    checkPermission(PermissionType.CHANGE_PERMISSION);

    String ownerName;
    // First who make it accessControllable is owner
    if (!isNodeType(Constants.EXO_ACCESS_CONTROLLABLE)) {
      ownerName = session.getUserID();
      doAddMixin(Constants.EXO_ACCESS_CONTROLLABLE);
    } else {
      if (!getACL().getOwner().equals(SystemIdentity.SYSTEM))
        ownerName = getACL().getOwner();
      else
        ownerName = session.getUserID();
    }
    
    //childrenACL = new AccessControlList(ownerName, permissions);
    getACL().assign(ownerName, permissions);
    
    // log.debug("ACL owner"+childrenACL.getOwner());

    // populate exo:accessControllable
    //Value[] owner = new Value[] { valueFactory.createValue(childrenACL.getOwner()) };
    //boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    
    //doSetProperty("exo:owner", owner, PropertyType.STRING, false, registerInDataManager, false);
    dataManager.update(ItemState.createAdded(
        TransientPropertyData.createPropertyData(
            nodeData(), 
            Constants.EXO_OWNER, 
            PropertyType.STRING, 
            false,
            new TransientValueData(getACL().getOwner()))),
            true);
    
    savePermissionsProperty();
  }

  private void savePermissionsProperty() throws RepositoryException {

    //boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    
    Map<String, String[]> permissions = getACL().getPermissions();
    List<ValueData> permValues = new ArrayList<ValueData>();
    for (Iterator<String> i = permissions.keySet().iterator(); i.hasNext();) {
      String identity = i.next();
      String[] perm = permissions.get(identity);
      for (int j = 0; j < perm.length; j++)
        //permValues.add(new PermissionValue(identity, perm[j]));
        permValues.add(new TransientValueData(PermissionValue.asString(identity, perm[j])));
    }

//    Value[] permArray = new Value[permValues.size()];
//    for (int i = 0; i < permArray.length; i++)
//      permArray[i] = (Value) permValues.get(i);

//    doSetProperty("exo:permissions", permArray, ExtendedPropertyType.PERMISSION, true,
//        registerInDataManager, false);
    
    dataManager.update(ItemState.createAdded(
        TransientPropertyData.createPropertyData(
            nodeData(), 
            Constants.EXO_PERMISSIONS, 
            ExtendedPropertyType.PERMISSION, 
            false,
            permValues)),
            true);
  }

  public AccessControlList getACL() {
    return nodeData().getACL();
  }
  
  protected NodeData nodeData() {
    //return (NodeData) state.getData();
    return (NodeData) data;
  }

  public void clearACL() throws RepositoryException, AccessControlException {
    checkPermission(PermissionType.CHANGE_PERMISSION);
    if (isNodeType(Constants.EXO_ACCESS_CONTROLLABLE)) {
      removeMixin(Constants.EXO_ACCESS_CONTROLLABLE.getAsString());
    }
    //childrenACL = new AccessControlList(true);
    getACL().reset();
  }

  public void removePermission(String identity) throws RepositoryException,
      AccessControlException {
    checkPermission(PermissionType.CHANGE_PERMISSION);
    // remove default permission
    if (!isNodeType(Constants.EXO_ACCESS_CONTROLLABLE)) {
      throw new AccessControlException(
          "It is impossible to remove default permissions. Use setPermission(s) instead");
    }
    getACL().getPermissions().remove(identity);
    savePermissionsProperty();
  }

  public void setPermission(String identity, String[] permission)
      throws RepositoryException, AccessControlException {
    // check if changing permission allowed
    checkPermission(PermissionType.CHANGE_PERMISSION);
    // remove default permission
    if (!isNodeType(Constants.EXO_ACCESS_CONTROLLABLE)) {
      Map<String, String[]> perms = new HashMap<String, String[]>();
      perms.put(identity, permission);
      setPermissions(perms);
      // childrenACL = new AccessControlList(false);
    } else {
      getACL().getPermissions().put(identity, permission);
      savePermissionsProperty();
    }
  }

  public void checkPermission(String actions) throws AccessControlException,
      RepositoryException {
    if (!session.getAccessManager().hasPermission(getACL(), actions, session.getUserID()))
      throw new AccessControlException("Permission denied " + getPath() + " : " + actions);
  }
}
