/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.jcr.nodetype.ConstraintViolationException;

import org.exoplatform.services.jcr.datamodel.InternalQPath;

/**
 * @deprecated
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: SessionMoveStorage.java 7543 2006-08-03 09:54:54Z geaz $ Storage for Session.move() location srcPath and destPath
 *          pairs responsible for not allowing to save them separately
 */

public class SessionMoveStorage {

  private List<MovedPair> movedPairs;

  SessionMoveStorage() {
    movedPairs = new ArrayList<MovedPair>();
  }

  /**
   * Adds moved src/dest pair's path. Called by Session.move()
   * 
   * @param path1
   * @param path2
   * @throws RepositoryException
   */
  void addMovedPaths(InternalQPath path1, InternalQPath path2) { // throws
    // RepositoryException
    movedPairs.add(new MovedPair(path1, path2));
    // System.out.println("MOVE ADDED >>> "+path1.getAsString()+"
    // "+path2.getAsString());

  }

  /**
   * Checks if the path is the same or descendant of either destination or
   * source node only. Throws ConstraintViolationException if so. Called by
   * SessionDataManager.validate()
   * 
   * @param path
   * @throws ConstraintViolationException
   */
  void checkViolation(InternalQPath path) throws ConstraintViolationException {

    for (int i = 0; i < movedPairs.size(); i++) {
      MovedPair pair = movedPairs.get(i);

      if (((pair.path1.equals(path) || pair.path1.isDescendantOf(path, false))
          && (!pair.path2.equals(path) && !pair.path2.isDescendantOf(path, false)) || ((pair.path2
          .equals(path) || pair.path2.isDescendantOf(path, false)) && (!pair.path1.equals(path) && !pair.path1
          .isDescendantOf(path, false))))) {

        throw new ConstraintViolationException(
            "path is the same or descendant of either Session.move()'s destination or source node only "
                + path.getAsString());
      }
    }
  }

  /**
   * Return map with source/destination pairs was used for move operation before 
   * Called by SessionDataManager.commit()
   * 
   * @param path
   */
  Map<InternalQPath,InternalQPath> getMoveMap() {
    HashMap<InternalQPath,InternalQPath> map = new HashMap<InternalQPath,InternalQPath>();  
    for (MovedPair pair : movedPairs) {
      map.put(pair.path1, pair.path2);
    }
    return map;
  }

  /**
   * Remove src/descendant pairs if path is ancestor of both src and descendant
   * Called by SessionDataManager.commit()
   * 
   * @param path
   */
  void removeDescendantsIfAny(InternalQPath path) {
    for (int i = 0; i < movedPairs.size(); i++) {
      MovedPair pair = movedPairs.get(i);
      if ((pair.path1.equals(path) || pair.path1.isDescendantOf(path, false))
          && (pair.path2.equals(path) || pair.path2.isDescendantOf(path, false))) {
        movedPairs.remove(i);
      }
    }
  }

  /**
   * Clear all changes. Called by SessionDataManager.rollback()
   */
  void clear() {
    movedPairs.clear();
  }
  
  boolean isMoveSource(InternalQPath path) {
    for (MovedPair pair: movedPairs) {
      if (pair.path1.equals(path)) return true;  
    }
    return false;
  }
  
  boolean isMoveSourceDescendant(InternalQPath path) {
    for (MovedPair pair: movedPairs) {
      if (path.isDescendantOf(pair.path1, false)) return true;  
    }
    return false;
  }

  boolean isMoveDestination(InternalQPath path) {
    for (MovedPair pair: movedPairs) {
      if (pair.path2.equals(path)) return true;  
    }
    return false;
  }

  boolean isMoveDestinationDescendant(InternalQPath path) {
    for (MovedPair pair: movedPairs) {
      if (path.isDescendantOf(pair.path2, false)) return true;  
    }
    return false;
  }
  
  public boolean isMoveDescendant(InternalQPath path) {
    for (MovedPair pair: movedPairs) {
      if (path.isDescendantOf(pair.path1, false) || path.isDescendantOf(pair.path2, false)) return true;  
    }
    return false;
  }  

  private class MovedPair {
    private InternalQPath path1;

    private InternalQPath path2;

    private MovedPair(InternalQPath path1, InternalQPath path2) {
      this.path1 = path1;
      this.path2 = path2;
    }
  }

}
