/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.dataflow;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Calendar;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.datamodel.AbstractValueData;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.DateValue;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .<br/>
 * 
 * Newly created value data
 * TODO avoid using BaseValue and  DateValue here
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransientValueData.java 8595 2006-09-05 14:42:58Z peterit $
 */

public class TransientValueData extends AbstractValueData {

  private static Log log = ExoLogger.getLogger("jcr.TransientValueData");

  static protected byte[] stringToBytes(final String value) {
    try {
      return value.getBytes(BaseValue.DEFAULT_ENCODING);
    } catch (UnsupportedEncodingException e) {
      throw new RuntimeException("FATAL ERROR Charset "
          + BaseValue.DEFAULT_ENCODING + " is not supported!");
    }
  }
  
  /**
   * Create binary TransientValueData with data in bytes array
   */
  public TransientValueData(byte[] value, int orderNumber) {
    super(value, orderNumber);
  }

  /**
   * Create binary TransientValueData with in-memory buffer option.
   * Used in ItemDataFactory.makeValueData(ValueData).
   */
  public TransientValueData(InputStream stream, int orderNumber, boolean inBuffer) throws IOException {
    super(stream, orderNumber, inBuffer);
  }
  
  /**
   * Create binary TransientValueData with in-memory buffer option.
   */
  public TransientValueData(InputStream stream, boolean inBuffer) throws IOException {
    this(stream, 0, inBuffer);
  }
  
  /**
   * Create binary TransientValueData with swap option.
   */
  public TransientValueData(InputStream stream, String swapFilePath, int minSwapFileSize) throws IOException {
    super(stream, 0, swapFilePath, minSwapFileSize);
  }
  
  /**
   * Constructor for stored value data
   * @param value
   */
  public TransientValueData(byte[] value) {
    super(value, 0);
  }

  /**
   * Constructor for String value data
   * @param value
   */
  public TransientValueData(String value) {
    super(stringToBytes(value), 0);
  }
  
  /**
   * Constructor for String value data
   * @param value
   */
  public TransientValueData(String value, int orderNumber) {
    super(stringToBytes(value), orderNumber);
  }

  /**
   * Constructor for boolean value data
   * @param value
   */
  public TransientValueData(boolean value) {
    super(Boolean.valueOf(value).toString().getBytes(), 0);
  }

  /**
   * Constructor for Calendar value data
   * @param value
   */
  public TransientValueData(Calendar value) {
    super(DateValue.serializeDate(value), 0);
  }

  /**
   * Constructor for double value data
   * @param value
   */
  public TransientValueData(double value) {
    super(Double.valueOf(value).toString().getBytes(), 0);
  }
  
  /**
   * Constructor for long value data
   * @param value
   */
  public TransientValueData(long value) {
    super(Long.valueOf(value).toString().getBytes(), 0);
  }

  /**
   * Constructor for Name value data
   * @param value
   */
  public TransientValueData(InternalQName value) {
    super(value.getAsString().getBytes(), 0);
  }

  /**
   * Constructor for Path value data
   * @param value
   */
  public TransientValueData(InternalQPath value) {
    super(value.getAsString().getBytes(), 0);
  }

  /**
   * Constructor for Reference value data
   * @param value
   */
  public TransientValueData(Uuid value) {
    super(value.getString().getBytes(), 0);
  }
}