/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.sql.Connection;
import java.sql.SQLException;

import javax.jcr.RepositoryException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.impl.util.UUIDGenerator;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 29.08.2006
 * @version $Id: JDBCConnection.java 8506 2006-09-01 15:48:45Z peterit $
 */
abstract public class JDBCConnection extends DBConstants {
  
  protected static Log log = ExoLogger.getLogger("jcr.JDBCConneсtionHelper");
  
  protected final DataSource dataSource;
  protected final String containerName;
  protected final int containerNameLength;
  protected final boolean multiDb;
  
  public JDBCConnection(DataSource dataSource, String containerName, boolean multiDb)  throws SQLException {
    
    this.dataSource = dataSource;
    this.multiDb = multiDb;
    prepareQueries();
    prepareStatements();
    
    this.containerName = containerName; 
    this.containerNameLength = containerName.length();
  }
  
  /**
   * Prepared queries at start time
   * @throws SQLException
   */
  abstract protected void prepareQueries() throws SQLException;
  
  /**
   * Prepared statemens at statr time as it recommends by JDBC API doc
   * @throws SQLException
   */
  abstract protected void prepareStatements() throws SQLException;
  
  /**
   * Return JDBC connection obtained from initialized data source.
   * NOTE: Helper can obtain one new connection per each call of the method  or return one obtained once.
   *  
   * @throws SQLException
   */
  abstract protected Connection getConnection() throws SQLException;
  
  /**
   * Used in Single Db Connection classes for UUID related queries 
   * @param uuid
   * @return
   */
//  protected String id2cid(String uuid) {
//    if (!multiDb && uuid != null && uuid.length() == UUIDGenerator.UUID_LENGTH) {
//        return containerName + uuid;
//    }
//    // uuid already prepared for single db
//    return uuid;
//  }
  
  /**
   * Used in loadXYZRecord methods for extract real UUID from container value.
   * @param uuid
   * @return
   */
//  protected String cid2id(String uuid) {
//    if (!multiDb && uuid != null && uuid.length() > UUIDGenerator.UUID_LENGTH) {
//      return uuid.substring(containerNameLength);
//    }
//    return uuid;
//  }  
  
  /**
   * @throws IllegalStateException if connection is closed
   */
  protected void checkIfOpened() throws IllegalStateException {
    if(!isOpened())
      throw new IllegalStateException("Connection is closed");
  }
  
  /** Will be used as implementation of
   * @see org.exoplatform.services.jcr.storage.WorkspaceStorageConnection#isOpened()
   */
  public boolean isOpened() {
    try {
      return !getConnection().isClosed();
    } catch (SQLException e) {
      log.error(e);
      return false;
    }
  }

  /** Will be used as implementation of
   * @see org.exoplatform.services.jcr.storage.WorkspaceStorageConnection#rollback()
   */
  public final void rollback() throws IllegalStateException, RepositoryException {
    checkIfOpened();
    try {
      Connection con = getConnection();
      con.rollback();
      con.close();
    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }
  
  /** Will be used as implementation of 
   * @see org.exoplatform.services.jcr.storage.WorkspaceStorageConnection#commit()
   */
  public final void commit() throws IllegalStateException, RepositoryException {
    checkIfOpened();
    try {
      Connection con = getConnection();
      con.commit();
      con.close();
    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }  
}
