/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.io.IOException;
import java.sql.SQLException;

import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.services.database.jdbc.DBSchemaCreator;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.impl.storage.value.StandaloneStoragePluginProvider;
import org.exoplatform.services.jcr.impl.storage.value.jdbc.JDBCValueStoragePluginProvider;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;
import org.exoplatform.services.jcr.storage.value.ValueStoragePluginProvider;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: JDBCWorkspaceDataContainer.java 8556 2006-09-04 15:32:25Z peterit $
 */
public class JDBCWorkspaceDataContainer extends WorkspaceDataContainerBase {

  public final static String CONTAINER_NAME = "containerName";
  public final static String SOURCE_NAME = "sourceName";
  public final static String MULTIDB = "multi-db";
  public final static String BLOB_SWAP = "blob-swap";
  
  protected static Log log = ExoLogger.getLogger("jcr.JDBCWorkspaceDataContainer");
  
  protected String containerName;
  protected String sourceName;
  protected boolean multiDb;
  private DataSource dataSource;
  private ValueStoragePluginProvider valueStorageProvider;
  
//  /**
//   * Constructor with no value storage plugins configured
//   * @param wsConfig
//   * @param dbCreator
//   * @throws RepositoryConfigurationException
//   * @throws NamingException
//   */
//  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
//      DBSchemaCreator dbCreator) 
//  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
//    this(wsConfig, dbCreator, null);
//  }
  
  /**
   * Constructor with value storage plugins
   * @param wsConfig
   * @param dbCreator
   * @param valueStrorageProvider
   * @throws RepositoryConfigurationException
   * @throws NamingException
   */
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      DBSchemaCreator dbCreator,
      StandaloneStoragePluginProvider valueStorageProvider) 
  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
    
    this.containerName = wsConfig.getName();
    this.sourceName = wsConfig.getContainer().getParameterValue(SOURCE_NAME);    
    this.multiDb = Boolean.parseBoolean(wsConfig.getContainer().getParameterValue(MULTIDB));
    
    Context context = new InitialContext();
    dataSource = (DataSource) context.lookup(sourceName);

    this.valueStorageProvider = new JDBCValueStoragePluginProvider(
        containerName, 
        multiDb,
        dataSource, 
        valueStorageProvider);
    
    log.info("JDBCWorkspaceDataContainer (cEntry): " + containerName + ", dataSource: " + sourceName);
  }

  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#openConnection()
   */
  public WorkspaceStorageConnection openConnection() throws RepositoryException {
    
    try {

      if (multiDb) 
        return new MultiDbJDBCConnection(dataSource, containerName, valueStorageProvider);
      else 
        return new SingleDbJDBCConnection(dataSource, containerName, valueStorageProvider);

    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return containerName;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getInfo()
   */
  public String getInfo() {
    String str = "Info: JDBC based JCR Workspace Data container \n";
    return str;
  }

}