/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.value.fs;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedValueData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.storage.swap.BinaryValueSwapStorage;
import org.exoplatform.services.jcr.storage.value.ValueIOChannel;
import org.exoplatform.services.jcr.util.io.LazyFileInputStream;
import org.exoplatform.services.log.ExoLogger;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: FileIOChannel.java 8661 2006-09-07 10:17:32Z peterit $
 */

public class FileIOChannel implements ValueIOChannel {
  
  protected static Log log = ExoLogger.getLogger("jcr.FileIOChannel");
  
  protected final File rootDir;
  
  protected final BinaryValueSwapStorage binarySwap;
  
  public FileIOChannel(File rootDir, BinaryValueSwapStorage binarySwap) {
    this.rootDir = rootDir;
    this.binarySwap = binarySwap;
  }
  
  public FileIOChannel(File rootDir) {
    this.rootDir = rootDir;
    this.binarySwap = null;
  }

  public List<ValueData> read(String propertyId, boolean inBuffer) throws IOException {
    
    // [PN] 18.08.06
    // FileNotFoundException can occur if file will be removed externaly (e.g. manually or by virus)
    // [GA] Sure, the same situation can occur in any type of storage :)),
    // just do not delete externally to keep data integrity
    
    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId));
    ArrayList<ValueData> values = new ArrayList<ValueData>(valueFiles.length);
    // Buffering is enabled for this property
    for (int i = 0; i<valueFiles.length; i++) {
      if (inBuffer) {
        FileInputStream fis = new FileInputStream(valueFiles[i]);
        
//        FileChannel channel = null;
//        try {
//          channel = fis.getChannel();
//          int size = (int) channel.size();
//          ByteBuffer buf = ByteBuffer.allocate(size);
//          int numRead = channel.read(buf);
//
//          byte[] arr = new byte[numRead]; // buf.position()
//          buf.rewind();
//          buf.get(arr);
        
        // read in bytes buffer
        values.add(new PersistedValueData(fis, i, true)); // read in bytes buffer
        
//        } finally {
//          if (channel != null)
//            channel.close();
//        }
      } else {
        // use lazy readable stream
        LazyFileInputStream fis = new LazyFileInputStream(valueFiles[i]);
        values.add(new PersistedValueData(fis, i));
      }
    }
    return values;
  }
  
// protected File getValueFile(String propertyId, int orderNumb) throws
// IOException {
//    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId + orderNumb));
//    if (valueFiles.length > 0) {
//      if (valueFiles.length > 1) {
//        log.warn("Too many files found (" + valueFiles.length 
//            + ") for property id: " + propertyId + ", order number: " + orderNumb);
//      }
//      return valueFiles[0];
//    }
//    return null;
//  }  

  public void write(String propertyId, List<ValueData> values)  throws IOException {
    // TODO ByteArray or Stream value data
    for(ValueData value: values) {
      try {
        String fileName = propertyId + value.getOrderNumber();
        File file = new File(rootDir, fileName);
        FileOutputStream fos = new FileOutputStream(file);
        FileChannel channel = fos.getChannel();
        
        ByteBuffer buf = ByteBuffer.wrap(value.getAsByteArray());
        channel.write(buf);
        channel.close();
        fos.close();
//        } else {
//          throw new ClassCastException("ValueData type "+value.getClass()+" is not supported yet");
//        }
      } catch (FileNotFoundException e) {
        log.error("Write operation error: " + e.getMessage(), e);
      }
    }
  }
  
  public void delete(String propertyId)  throws IOException {
    File[] valueFiles = rootDir.listFiles(new PropertyIDFilter(propertyId));
    // remove old files
    for(File valueFile: valueFiles) {
      boolean res = valueFile.delete();
    }
  }

  public void close() {
  }
  
  private class PropertyIDFilter implements FileFilter {
    
    private String id;

    public PropertyIDFilter(String id) {
      this.id = id;
    }

    public boolean accept(File filename) {
      return filename.getName().startsWith(id);
    }
  }
}
