/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.util;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL.
 *          
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: StringConverter.java 8543 2006-09-04 12:37:41Z peterit $
 */

public class StringConverter {

  private static String ILLEGAL_DNCHAR = "StringConverter: empty string for denormalization to char";
  
  private static Log log = ExoLogger.getLogger("jcr.StringConverter");
  
  private static class DNChar {
    private char dnChar;
    private int dnLength;
    
    public DNChar(char dnChar, int dnLength) {
      this.dnChar = dnChar;
      this.dnLength = dnLength;
    }

    public char getDnChar() {
      return dnChar;
    }

    public int getDnLength() {
      return dnLength;
    }    
  }
  
  /**
   * Normalizes and prints the given string.
   */
  public static String normalizeString(String s, boolean canonical) {

    StringBuffer strBuf = new StringBuffer();
    int len = (s != null) ? s.length() : 0;
    for (int i = 0; i < len; i++) {
      char c = s.charAt(i);
      String spart = s.substring(i);
      if (spart.startsWith("_") && !spart.startsWith("_x")) {
        strBuf.append(c);
      } else {
        strBuf.append(normalizeChar(c, canonical));
      }
    }
    return new String(strBuf);
  } // normalizeAndPrint(String)
  
  public static String denormalizeString(String s) {

    StringBuffer strBuf = new StringBuffer();
    int len = (s != null) ? s.length() : 0;
    int i = 0;
    while (i < len) {
      try {
        DNChar dnc = denormalize(s.substring(i));
        strBuf.append(dnc.getDnChar());
        //log.info("Denormalize as char. '" + s.substring(i, i + dnc.getDnLength()) + "'-->'" + dnc.getDnChar() + "'"
        //    + "; " + strBuf.toString() + "; pos:" + i + "; skip:" + dnc.getDnLength());
        i += dnc.getDnLength();
      } catch(IllegalArgumentException e) {
        if (!e.getMessage().equals(ILLEGAL_DNCHAR)) {
          throw e;
        }
        char c = s.charAt(i);
        strBuf.append(c);
        i++;
      }
    }
    return new String(strBuf);
  }

  /**
   * Normalizes and print the given character.
   */
  public static String normalizeChar(char c, boolean canonical) {

    switch (c) {
    case '<':
      return "&lt;";
    case '>':
      return "&gt;";
    case '&':
      return "&amp;";
    case '"':
      return "&quot;";
    case '\r':
      if (canonical) 
        return "_x000D_";
    case '\n':
      if (canonical) 
        return "_x000A_";
    case '\t':
      if (canonical) 
        return "_x0009_";
    case ' ':
      if (canonical) 
        return "_x0020_";
    case '_':
      if (canonical) 
        return "_x005f_";  
    // else, default print char
    default:
      return "" + c;
    }
  }
  
  /**
   * Denormalizes and print the given character.
   */
  public static char denormalizeChar(String string) {
    return denormalize(string).getDnChar();
  }
  
  /**
   * Denormalizes and print the given character.
   */
  private static DNChar denormalize(String string) {

    if (string.equals("&lt;")) return new DNChar('<', 4);
    else if (string.startsWith("&gt;")) return new DNChar('>', 4);
    else if (string.startsWith("&quot;")) return new DNChar('"', 6);
    else if (string.startsWith("_x000D_")) return new DNChar('\r', 7);
    else if (string.startsWith("_x000A_")) return new DNChar('\n', 7);
    else if (string.startsWith("_x0020_")) return new DNChar(' ', 7);
    else if (string.startsWith("_x005f_")) return new DNChar('_', 7);
    //else if (string.length()>0) return string.toCharArray()[0];
    else throw new IllegalArgumentException(ILLEGAL_DNCHAR);
  }  
}