/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.xml;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.log.ExoLogger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SysNodeImporter.java 6078 2006-06-08 15:14:03Z peterit $
 */

class SysNodeImporter implements ContentHandler {

  private Stack     tree;

  private String    curPropName;

  private String    curPropType;

  private ArrayList curPropValues;

  // private JCRPath curLocation;
  private List      nodeInfos;

  private Session   session;

  private NodeImpl parent;
  private NodeImpl rootNode;
  
  private Exception causeException = null;

  protected static Log log = ExoLogger.getLogger("jcr.SysNodeImporter");
  
  SysNodeImporter(NodeImpl parent) throws RepositoryException {
    //super(parent);
    this.parent = parent;
    this.tree = new Stack();
    this.nodeInfos = new ArrayList();
    this.session = parent.getSession();
    this.curPropValues = new ArrayList();
  }

  NodeImpl getNode() throws RepositoryException {
    
    return rootNode;
  }
  
  private NodeImpl buildNode() throws RepositoryException {

    // build node from parsed infos
    NodeImpl node = null;

    for (int i = 0; i < nodeInfos.size(); i++) {
      NodeInfo info = (NodeInfo) nodeInfos.get(i);
               
      SessionImpl sessionImpl = (SessionImpl) session;
      LocationFactory loFactory = sessionImpl.getLocationFactory();
      
      JCRPath relPath = loFactory.createJCRPath(parent.getLocation(), info.getRelPath());
      JCRPath parentPath = relPath.makeParentPath();
      NodeImpl parentNode = (NodeImpl) sessionImpl.getItem(parentPath.getAsString(false));
      // [PN] 01.06.06 looking for same-name-siblings
      try {
        Node parentOfParentNode = parentNode.getParent();
        NodeIterator snsParentNodes = parentOfParentNode.getNodes(parentNode.getName());
        
        if (snsParentNodes.getSize()>0) {
          snsParentNodes.skip(snsParentNodes.getSize() - 1);
          parentNode = (NodeImpl) snsParentNodes.nextNode();
        }
        
        NodeIterator snsNodes = parentNode.getNodes(relPath.getName().getName());
        relPath.setIndex((int) snsNodes.getSize() + 1);
      } catch(RepositoryException e) {
        log.warn("Error of same-name-sibling nodes processing: " + e.getMessage(), e);
        // if exception will be thrown a node will be created at relPath
      }
      
      InternalQName name = relPath.getInternalPath().getName(); 
      InternalQName primaryTypeName = loFactory.parseJCRName(info.getPrimaryTypeName()).getInternalName();
      InternalQName[] mixinTypeNames = new InternalQName[info.getMixinTypeNames().length];
      for (int mi=0; mi<mixinTypeNames.length; mi++) {
        InternalQName mixinTypeName = loFactory.parseJCRName(info.getMixinTypeNames()[mi]).getInternalName();
        mixinTypeNames[mi] = mixinTypeName;
      }
      
      //log.info("BUILD NODE: " + name.getAsString() + ":" + relPath.getIndex() + " " + primaryTypeName.getAsString() + " " + parentNode.getPath());
      
      NodeImpl n = parentNode.createChildNodeInmemory(name, primaryTypeName, mixinTypeNames,
          null,   // uuid 
          relPath.getIndex(),      // [PN] 01.06.06 index, relPath.getIndex()
          true,  // registerInDataManager
          true,  // doExternalValidation
          false);  // addAutoCreated 
      
      List props = info.getProperties();
      for (int j = 0; j < props.size(); j++) {
        PropertyInfo prop = (PropertyInfo) props.get(j);
        
        
        List valueList = prop.getValues();
        Value[] values = new Value[valueList.size()];
        for (int k = 0; k < valueList.size(); k++) {
          String val = new String((StringBuffer) valueList.get(k));
          if (prop.getType() == PropertyType.BINARY) {
            values[k] = session.getValueFactory().createValue(
                new ByteArrayInputStream(Base64.decodeBase64(val.getBytes())));
          } else {
            values[k] = session.getValueFactory().createValue(val, prop.getType());
          }
        }
        n.createChildProperty(prop.getName(), values, prop.getType()); 
      }
      if (i == 0) {
        node = n;
      }
    }
    return node;
  }

  public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
      throws SAXException {
    if (qName.equals("sv:node")) {
      try {
        String name = atts.getValue("sv:name");
        String relPath;
        if (!tree.isEmpty())
          relPath = ((NodeInfo) tree.peek()).getRelPath() + "/" + name;
        else
          relPath = name;
        
        NodeInfo info = new NodeInfo(relPath);
        tree.push(info);
        nodeInfos.add(info);
      } catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:property")) {
      // TODO manage UUID
      try {
        curPropType = atts.getValue("sv:type");
        curPropName = atts.getValue("sv:name");
        curPropValues = new ArrayList();        
      } catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:value")) {
      // [PN] 30.05.06
      StringBuffer curPropValue = new StringBuffer();
      curPropValues.add(curPropValue);
    } else {
      throw new SAXException("'" + qName
          + "' is not allowed. Only sv:node, sv:property and sv:value are allowed");
    }
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    if (qName.equals("sv:node")) {
      // nodeInfos.add(tree.pop());
      tree.pop();
    } else if (qName.equals("sv:property")) {
      ((NodeInfo) tree.peek()).addProperty(curPropName, curPropType, curPropValues);
    } else if (qName.equals("sv:value")) {
      // no thing to do 
    }
  }

  public void characters(char[] ch, int start, int length) throws SAXException {
    // [PN] 30.05.06
    // property values
    if (curPropValues.size()>0) {
      StringBuffer curPropValue = (StringBuffer) curPropValues.get(curPropValues.size()-1);
      curPropValue.append(ch, start, length);
      //curPropValues.add(curPropValue);
    } else {
      // wrong XML, no sv:value visited before
      log.warn("Wrong XML content. Element 'sv:value' expected, but SAX event 'characters' occured. characters:["
          + new String(ch) + "]");
    }
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
    nodeInfos.clear();
  }

  public void endDocument() throws SAXException {
    // so, build the node parsed before
    // and put it in the session
    try {
      rootNode = buildNode();
    } catch(Exception e) {
      e.printStackTrace();
      //causeException = e;
      throw new SAXException("Error create node from System View document. Exception: " + 
          e.getMessage(), e);
    }
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  private class NodeInfo {
    private String    relPath;

    private ArrayList properties = new ArrayList();

    public NodeInfo(String relPath) {
      this.relPath = relPath;
    }

    public void addProperty(String name, String typeName, ArrayList strValues) {
      int type = ExtendedPropertyType.valueFromName(typeName);
      PropertyInfo prop = new PropertyInfo(name, type, strValues);
      properties.add(prop);
    }

    public ArrayList getProperties() {
      return properties;
    }

    public String getRelPath() {
      return relPath;
    }

    public String getPrimaryTypeName() throws RepositoryException {
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = (PropertyInfo) properties.get(i);
        if (prop.getName().equals("jcr:primaryType"))
          return new String((StringBuffer) prop.getValues().get(0));
      }
      throw new RepositoryException("No primary type ");
    }

    public String[] getMixinTypeNames() {
      ArrayList mixins = new ArrayList();
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = (PropertyInfo) properties.get(i);
        if (prop.getName().equals("jcr:mixinTypes")) {
          for (int j = 0; j < prop.getValues().size(); j++)
            mixins.add(new String((StringBuffer) prop.getValues().get(j)));
        }
      }

      String[] mixinProps = new String[mixins.size()];
      for (int i = 0; i < mixins.size(); i++) {
        mixinProps[i] = (String) mixins.get(i);
      }
      return mixinProps;
    }

  }

  private class PropertyInfo {

    private String name;

    private int    type;

    private List   values;

    public PropertyInfo(String name, int type, List values) {
      super();
      this.name = name;
      this.type = type;
      this.values = values;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
      return name;
    }

    /**
     * @return Returns the type.
     */
    public int getType() {
      return type;
    }

    /**
     * @return Returns the values.
     */
    public List getValues() {
      return values;
    }
  }

}
