/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.Socket;
import java.util.ArrayList;
import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.Const;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class HttpClient {

  private static Log log = ExoLogger.getLogger("jcr.HttpClient");
  
  private boolean enableTrace = false;
  public void EnableTrace() {
      enableTrace = true;
  }
  
  private String server = "";
  private int port = 0;
  
  private Socket clientSocket = null;
  private PrintStream outPrintStream = null;
  private OutputStream outStream = null;
  private InputStream inputStream = null;

  // Request
  private String httpCommand = "GET";
  private String httpRequestStr = "";
  private ArrayList requestHeaders = new ArrayList();
  
  private String httpRequestBodyStr = null;
  private byte []httpRequestBodyBytes = null;
  
  // Response
  private String mainHeader = "";
  private ArrayList responseHeaders = new ArrayList();

  private byte []contentBytes = null;
  
  public HttpClient(String server, int port) {
      this.server = server;
      this.port = port;
  }
  
  public void conect() throws Exception {
      clientSocket = new Socket(server, port);
      outStream = clientSocket.getOutputStream();
      outPrintStream = new PrintStream(clientSocket.getOutputStream());
      inputStream = clientSocket.getInputStream();     
  }
  
  public void setHttpCommand(String httpCommand) {
      this.httpCommand = httpCommand;
  }
  
  public void setRequestPath(String httpRequestStr) {
      this.httpRequestStr = httpRequestStr;
  }
  
  public void setRequestHeader(String headerName, String headerValue) throws Exception {
      int existedIndex = -1;
      for (int i = 0; i < requestHeaders.size(); i++) {
          String curHeader = (String)requestHeaders.get(i);
          String []curHeaderValues = curHeader.split(Const.Http.HEADER_DELIM);
          if (curHeaderValues[0].toUpperCase().equals(headerName.toUpperCase())) {                
              existedIndex = i;
              break;
          }            
      }
      if (existedIndex != -1) {
          requestHeaders.remove(existedIndex);
      }        
      String newHeader = headerName + Const.Http.HEADER_DELIM + headerValue;
      requestHeaders.add(newHeader);
  }
  
  public void setRequestBody(String httpRequestBodyStr) {
      this.httpRequestBodyStr = httpRequestBodyStr;
  }
  
  public void setRequestBody(byte []httpRequestBodyBytes) {
    this.httpRequestBodyBytes = httpRequestBodyBytes;
  }
  
  public void zeroRequestBody() {
      this.httpRequestBodyStr = null;
  }
  
  public void sendRequest(String request) {
      outPrintStream.print(request);
  }    

  public String getMainHeader() {
      return mainHeader;
  }
  
  public int getContentLength() {
      int contLength = 0;
      for (int i = 0; i < responseHeaders.size(); i++) {
          String curHeader = (String)responseHeaders.get(i);
          if (curHeader.startsWith(Const.HttpHeaders.CONTENTLENGTH)) {
              String []params = curHeader.split(":");
              String lenValue = params[1];
              lenValue = lenValue.trim();                    
              contLength = new Integer(lenValue);                    
          }
      }
      return contLength;
  }
  
  public ArrayList<String> getResponseHeadersNames() {
      ArrayList<String> result = new ArrayList<String>();
      for (int i = 0; i < responseHeaders.size(); i++) {
          String curHeader = (String)responseHeaders.get(i);
          result.add(curHeader.split(":")[0]);
      }
      return result;
  }
  
  public String getResponseHeader(String headerName) {
      for (int i = 0; i < responseHeaders.size(); i++) {
          String curHeader = (String)responseHeaders.get(i);          
          if (curHeader.startsWith(headerName + ": ")) {
            return curHeader.substring(curHeader.indexOf(": ") + 2);
          }
      }
      return "";
  }

  public int getReplyCode() throws Exception {
      int replyCode = 0;
      String []mPathes = mainHeader.split(" ");
      replyCode = new Integer(mPathes[1]);
      return replyCode;
  }

  public String getResponseBody() {
    String contentString = "";
    for (int i = 0; i < contentBytes.length; i++) {
      contentString += (char)contentBytes[i];
    }
    return contentString;
  }

  public byte []getResponseBytes() {
    return contentBytes;
  }
  
  public InputStream getResponseStream() {
      return new ByteArrayInputStream(contentBytes);
  }

  public int execute() throws Exception {
    // REQUEST
    
    String escapedHttpCommand = TextUtils.Escape(httpRequestStr, '%', true);    
    String httpLine = httpCommand + " " + escapedHttpCommand + " " + Const.Http.VERSION;
    
    outPrintStream.println(httpLine);
    
    if (enableTrace) {
      log.info(httpLine);
    }
    
    int reqContLength = 0;
    
    if (httpRequestBodyStr != null) {
      reqContLength = httpRequestBodyStr.length();
    } else if (httpRequestBodyBytes != null) {
      reqContLength = httpRequestBodyBytes.length;
    }

    if (reqContLength != 0) {
      setRequestHeader(Const.HttpHeaders.CONTENTLENGTH, String.format("%s", reqContLength));
    }
    
    setRequestHeader(Const.HttpHeaders.HOST, String.format("%s:%s", server, port));
    setRequestHeader(Const.HttpHeaders.USERAGENT, Const.Http.CLIENTDESCR);
    
    for (int i = 0; i < requestHeaders.size(); i++) {
        String curHeader = (String)requestHeaders.get(i);
        outPrintStream.println(curHeader);
        if (enableTrace) {
          log.info(curHeader);
        }
    }
    
    outPrintStream.println();
    
    if (reqContLength != 0) {
      if (httpRequestBodyStr != null) {
        outPrintStream.print(httpRequestBodyStr);
      } else {
        outStream.write(httpRequestBodyBytes);
      }
    }
    
    // RESPONSE
    mainHeader = readLine();
    while (true) {        
        String nextHeader = readLine();
        if (nextHeader.equals("")) {
            break;
        } else {
            responseHeaders.add(nextHeader);
        }
    }

    int contentLength = getContentLength();
    
    ByteArrayOutputStream outStream = new ByteArrayOutputStream();
    byte []buffer = new byte[16 * 1024];
    
    if (contentLength != 0 && !Const.DavCommand.HEAD.equals(httpCommand)) {
        int received = 0;
        
        while (received < contentLength) {
          int readed = inputStream.read(buffer);
          
          if (readed < 0) {
            break;
          }
          
          outStream.write(buffer, 0, readed);            
          received += readed;
          
        }
        
    }
    
    contentBytes = outStream.toByteArray();
    
    try {
        clientSocket.close();
    } catch (Exception exc) {}        

    return getReplyCode();
  }  
  
  protected String readLine() throws Exception {
    byte []buffer = new byte[4*1024];
    int bufPos = 0;
    byte prevByte = 0;

    while (true) {
      int received = inputStream.read();
      if (received < 0) {
        return null;
      }
      
      buffer[bufPos] = (byte)received;
      bufPos++;
      
      if (prevByte == '\r' && received == '\n') {
        String resultLine = "";
        for (int i = 0; i < bufPos - 2; i++) {
          resultLine += (char)buffer[i];
        }
        return resultLine;
      }
      
      prevByte = (byte)received;
    }
  }  
  
}
