/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient.commands;

import java.io.ByteArrayOutputStream;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.exoplatform.frameworks.davclient.Const;
import org.exoplatform.frameworks.davclient.HttpClient;
import org.exoplatform.frameworks.davclient.ServerLocation;
import org.exoplatform.frameworks.davclient.Const.HttpHeaders;
import org.exoplatform.services.log.ExoLogger;
import org.w3c.dom.Document;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public abstract class DavCommand {
  
  public static final String AUTH_BASIC = "Basic";
  
  private static Log log = ExoLogger.getLogger("jcr.DavCommand");

  protected Document xmlDoc = null;  
  protected String commandName;

  protected ServerLocation location = null;
  
  protected String resourcePath;
  
  protected HttpClient client;
  protected byte []requestDataBytes = null;
  protected byte []responseDataBytes = null;
  
  public DavCommand(ServerLocation location) throws Exception {
    this.location = location;
    
    client = new HttpClient(location.getHost(), location.getPort());
    
    client.setRequestHeader(Const.HttpHeaders.CONNECTION, Const.HttpHeaders.TE);
    client.setRequestHeader(Const.HttpHeaders.TE, "trailers, deflate, gzip, compress");
    client.setRequestHeader(Const.HttpHeaders.DEPTH, "1");
    client.setRequestHeader(Const.HttpHeaders.TRANSLATE, "f");
    client.setRequestHeader(Const.HttpHeaders.ACCEPTENCODING, "deflate, gzip, x-gzip, compress, x-compress");
    client.setRequestHeader(Const.HttpHeaders.CONTENTTYPE, "text/xml");                
  }
  
  public void setResourcePath(String resourcePath) {
    this.resourcePath = resourcePath;
  }
  
  public void setRequestDataBuffer(byte []requestDataBytes) {
    this.requestDataBytes = requestDataBytes;
  }
  
  public void setLockToken(String lockToken) throws Exception {
    client.setRequestHeader(Const.HttpHeaders.LOCKTOKEN, "<" + lockToken + ">");
  }
  
  public byte []getResponseDataBuffer() {
    return client.getResponseBytes();
  }
  
  public int execute() throws Exception {    
    xmlDoc = toXml(DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument());
    if (xmlDoc != null) {
      serializeXml(xmlDoc);      
    }
        
    if (requestDataBytes != null) {
      client.setRequestBody(requestDataBytes);
    }
    
    client.setHttpCommand(commandName);
    
    String path = location.getServletPath();
    if (resourcePath != null) {
      path += resourcePath;
    }

    client.setRequestPath(path);
    
    if (location.getUserId() != null) {      
      String userId = location.getUserId();
      String userPass = location.getUserPass();
      
      byte []encoded = Base64.encodeBase64(new String(userId + ":" + userPass).getBytes());
      String encodedAuth = new String(encoded);
  
      client.setRequestHeader(Const.HttpHeaders.AUTHORIZATION, AUTH_BASIC + " " + encodedAuth);
    }
    
    client.conect();
    
    int status = client.execute();    
    finalExecute();
    return status;
  }
  
  public abstract void finalExecute();
  
  public abstract Document toXml(Document xmlDocument);
  
  
  protected boolean traceEnable = false;
  
  public void setTrace(boolean traceEnable) {
    this.traceEnable = traceEnable;
  }
  
  public void serializeXml(Document xmlDocument) throws Exception {
    ByteArrayOutputStream outStream = new ByteArrayOutputStream();
    OutputFormat format = new OutputFormat("xml", "UTF-8", true);
    XMLSerializer serializer = new XMLSerializer(outStream, format);
    serializer.asDOMSerializer().serialize(xmlDocument);
    requestDataBytes = outStream.toByteArray();
    
    if (traceEnable) {
      try {
        Thread.sleep(1000);
        for (int i = 0; i < requestDataBytes.length; i++) {
          System.out.write((char)requestDataBytes[i]);
        }
        Thread.sleep(1000);
      } catch (Exception exc) {
        log.info("Unhandled exception. " + exc.getMessage(), exc);
      }
    }
    
  }
  
}
