/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient.commands;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import org.exoplatform.frameworks.davclient.Const;
import org.exoplatform.frameworks.davclient.ServerLocation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class DavPropPatch extends DavCommand {
  
  protected HashMap<String, String> propSet = new HashMap<String, String>();
  protected ArrayList<String> propRemove = new ArrayList<String>();
  
  public DavPropPatch(ServerLocation location) throws Exception {
    super(location);
    commandName = Const.DavCommand.PROPPATCH;
  }

  public void setProperty(String propertyName, String propertyValue) {
    propSet.put(propertyName, propertyValue);
  }
  
  public void removeProperty(String propertyName) {
    propRemove.add(propertyName);
  }
  
  public void finalExecute() {
  }
  
  public Document toXml(Document xmlDocument) {
    Element propUpdateEl = xmlDocument.createElementNS(Const.Dav.NAMESPACE,
        Const.Dav.PREFIX + Const.StreamDocs.PROPERTYUPDATE);
    xmlDocument.appendChild(propUpdateEl);
    propUpdateEl.setAttribute(Const.Dav.NAMESPACEATTR, Const.Dav.NAMESPACE);    
    
    Element removeEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.REMOVE);
    propUpdateEl.appendChild(removeEl);
    Element propRemoveEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.PROP);
    removeEl.appendChild(propRemoveEl);
    
    for (int i = 0; i < propRemove.size(); i++) {
      String propName = propRemove.get(i);
      
      Element curPropEl = xmlDocument.createElement(Const.Dav.PREFIX + propName);
      propRemoveEl.appendChild(curPropEl);
    }
    
    Element setEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.SET);
    propUpdateEl.appendChild(setEl);
    Element propSetEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.PROP);
    setEl.appendChild(propSetEl);
    
    Iterator<String> keyIter = propSet.keySet().iterator();
    while (keyIter.hasNext()) {
      String propName = keyIter.next();
      String propVal = propSet.get(propName);
      
      Element curPropEl = xmlDocument.createElement(Const.Dav.PREFIX + propName);
      propSetEl.appendChild(curPropEl);
      curPropEl.setTextContent(propVal);
    }
    
    return xmlDocument;
  }
  
}
