/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient.commands;

import java.util.ArrayList;
import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.Const;
import org.exoplatform.frameworks.davclient.ServerLocation;
import org.exoplatform.frameworks.davclient.documents.DocumentApi;
import org.exoplatform.frameworks.davclient.documents.DocumentManager;
import org.exoplatform.services.log.ExoLogger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class DavReport extends DavCommand {

  private static Log log = ExoLogger.getLogger("jcr.DavReport");

  protected static final String []defReqProps = {
    Const.DavProp.VERSIONNAME,
    Const.DavProp.CREATORDISPLAYNAME,
    Const.DavProp.GETLASTMODIFIED,
    Const.DavProp.GETCONTENTLENGTH,
    Const.DavProp.SUCCESSORSET,
    Const.DavProp.CHECKEDIN,
    Const.DavProp.CHECKEDOUT
  };
  
  protected boolean isRequireAllProps = true;
  protected ArrayList<String> requiredProperties = new ArrayList<String>();
  
  protected DocumentApi multistatusDocument = null;
  
  public DavReport(ServerLocation location) throws Exception {
    super(location);
    commandName = Const.DavCommand.REPORT;
  }
  
  public void requireAllProperties() {
    isRequireAllProps = true;
  }
  
  public void setRequiredProperty(String propertyName) {
    isRequireAllProps = false;
    requiredProperties.add(propertyName);
  }  

  public void finalExecute() {
    try {
      if (client.getReplyCode() == Const.HttpStatus.NOTFOUND) {
        return;
      }
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
    multistatusDocument = DocumentManager.getResponseDocument(client.getResponseStream());
  }
  
  public DocumentApi getMultistatus() {
    return multistatusDocument; 
  }
  
  public Document toXml(Document xmlDocument) {
    Element versionTreeEl = xmlDocument.createElementNS(Const.Dav.NAMESPACE,
          Const.Dav.PREFIX + Const.StreamDocs.VERSION_TREE);
    versionTreeEl.setAttribute(Const.Dav.NAMESPACEATTR, Const.Dav.NAMESPACE);
    xmlDocument.appendChild(versionTreeEl);
    
    Element propEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.PROP);
    versionTreeEl.appendChild(propEl);
    
    if (isRequireAllProps) {
      for (int i = 0; i < defReqProps.length; i++) {
        Element propValEl = xmlDocument.createElement(Const.Dav.PREFIX + defReqProps[i]);
        propEl.appendChild(propValEl);
      }
    } else {
      for (int i = 0; i < requiredProperties.size(); i++) {
        String curPropName = requiredProperties.get(i);
        Element propValEl = xmlDocument.createElement(Const.Dav.PREFIX + curPropName);
        propEl.appendChild(propValEl);
      }
    }
    
    return xmlDocument;    
  }
  
}
