/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient;

import java.util.ArrayList;
import java.util.HashMap;
import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.commands.DavCheckIn;
import org.exoplatform.frameworks.davclient.commands.DavCheckOut;
import org.exoplatform.frameworks.davclient.commands.DavCopy;
import org.exoplatform.frameworks.davclient.commands.DavDelete;
import org.exoplatform.frameworks.davclient.commands.DavGet;
import org.exoplatform.frameworks.davclient.commands.DavHead;
import org.exoplatform.frameworks.davclient.commands.DavLock;
import org.exoplatform.frameworks.davclient.commands.DavMkCol;
import org.exoplatform.frameworks.davclient.commands.DavMove;
import org.exoplatform.frameworks.davclient.commands.DavOptions;
import org.exoplatform.frameworks.davclient.commands.DavPropFind;
import org.exoplatform.frameworks.davclient.commands.DavPropPatch;
import org.exoplatform.frameworks.davclient.commands.DavPut;
import org.exoplatform.frameworks.davclient.commands.DavReport;
import org.exoplatform.frameworks.davclient.commands.DavUnCheckOut;
import org.exoplatform.frameworks.davclient.commands.DavUnLock;
import org.exoplatform.frameworks.davclient.commands.DavVersionControl;
import org.exoplatform.frameworks.davclient.documents.DocumentApi;
import org.exoplatform.frameworks.davclient.documents.Multistatus;
import org.exoplatform.frameworks.davclient.documents.ResponseDoc;
import org.exoplatform.services.log.ExoLogger;
import junit.framework.TestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class DavTest extends TestCase {
  
  private Log getLogger(String unitName) {
    return ExoLogger.getLogger("jcr.TEST_" + unitName);
  }
  
  private final String WORKSPACE = "/production";
  private final String DEST_WORKSPACE = "/draft";
  private final int FCOUNT = 3;
  
  public void testOPTIONS() {
    Log log = getLogger(Const.DavCommand.OPTIONS);
    try {
      log.info("test...");
      
      DavOptions options = new DavOptions(TestConst.getTestServerLocation());
      int status = options.execute();
      
      assertEquals(status, Const.HttpStatus.OK);
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testPROPFIND() {
    Log log = getLogger(Const.DavCommand.PROPFIND);
    try {
      log.info("test...");
      
      DavPropFind propFind = new DavPropFind(TestConst.getTestServerLocation());
      propFind.setResourcePath(WORKSPACE);
      //propFind.setResourcePath("/");
      
      int status = propFind.execute();
      
      assertEquals(status, Const.HttpStatus.MULTISTATUS);
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testMKCOL() {
    Log log = getLogger(Const.DavCommand.MKCOL);
    try {
      log.info("test...");
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "/";
        
        DavMkCol mkCol = new DavMkCol(TestConst.getTestServerLocation());
        mkCol.setResourcePath(WORKSPACE + resName);
        
        int status = mkCol.execute();
        assertEquals(status, Const.HttpStatus.CREATED);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testCOPY() {
    Log log = getLogger(Const.DavCommand.COPY);
    try {
      log.info("test...");
      
      // copy into same workspace
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "/";
        String destName = "/test_folder_" + i + "_copy/";
        
        DavCopy copy = new DavCopy(TestConst.getTestServerLocation());
        copy.setResourcePath(WORKSPACE + resName);
        copy.setDestinationPath(WORKSPACE + destName);
        
        assertEquals(copy.execute(), Const.HttpStatus.CREATED);
      }
      
      //log.info("Copty to same workspace complete.");
      
      // copy to another workspace
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "_copy/";
        
        DavCopy copy = new DavCopy(TestConst.getTestServerLocation());
        copy.setResourcePath(WORKSPACE + resName);
        copy.setDestinationPath(DEST_WORKSPACE + resName);
        
        assertEquals(copy.execute(), Const.HttpStatus.CREATED);
      }
      
      //log.info("Copty to another workspace complete.");
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testMOVE() {
    Log log = getLogger(Const.DavCommand.MOVE);
    try {
      log.info("test...");
      
      // move into same workspace
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "_copy/";
        String destName = "/test_folder_" + i + "_moved/";
        
        DavMove move = new DavMove(TestConst.getTestServerLocation());
        move.setResourcePath(WORKSPACE + resName);
        move.setDestinationPath(WORKSPACE + destName);
        
        assertEquals(move.execute(), Const.HttpStatus.CREATED);
      }
      
      //log.info("Move into same workspace complete.");
      
      // move into another workspace
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "_moved/";
        String destName = "/test_folder_" + i + "_moved_another_workspace/";
        
        DavMove move = new DavMove(TestConst.getTestServerLocation());
        move.setResourcePath(WORKSPACE + resName);
        move.setDestinationPath(DEST_WORKSPACE + destName);
        
        assertEquals(move.execute(), Const.HttpStatus.CREATED);
      }
      
      //log.info("Move into another workspace complete.");
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }  
  
  private byte []getDataByte(String text) {    
    byte []dataBytes = new byte[text.length()];
    for (int i = 0; i < text.length(); i++) {
      dataBytes[i] = (byte)text.charAt(i);
    }
    return dataBytes;
  }
  
  public void testPUT() {
    Log log = getLogger(Const.DavCommand.PUT);
    try {
      log.info("test...");
      
      for (int i = 0; i < FCOUNT; i++) {
        String srcPath = "/test_file_" + i + ".info";
        
        DavPut put = new DavPut(TestConst.getTestServerLocation());
        put.setResourcePath(WORKSPACE + srcPath);
        
        byte []dataByte = getDataByte("FILEDATA_" + i);
        put.setRequestDataBuffer(dataByte);
        
        int status = put.execute();
        
        assertEquals(status, Const.HttpStatus.CREATED);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testGET() {
    Log log = getLogger(Const.DavCommand.GET);
    try {
      log.info("test...");
      
      for (int i = 0; i < FCOUNT; i++) {
        String srcPath = "/test_file_" + i + ".info";
        
        DavGet get = new DavGet(TestConst.getTestServerLocation());
        get.setResourcePath(WORKSPACE + srcPath);
        
        int status = get.execute();
        assertEquals(status, Const.HttpStatus.OK);
        
        byte []dataBytes = get.getResponseDataBuffer();
        String datka = "";
        for (int di = 0; di < dataBytes.length; di++) {
          datka += (char)dataBytes[di];
        }
        
        String defFileContent = "FILEDATA_" + i;
        assertEquals(defFileContent, datka);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testHEAD() {
    Log log = getLogger(Const.DavCommand.HEAD);
    try {
      log.info("test...");
      
      // founds      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_file_" + i + ".info";
        
        DavHead head = new DavHead(TestConst.getTestServerLocation());
        head.setResourcePath(WORKSPACE + resName);
        
        int status = head.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }
      
      // not founds
      for (int i = 0; i < FCOUNT - 2; i++) {
        String resName = "/absent_file_" + i + ".info";
        
        DavHead head = new DavHead(TestConst.getTestServerLocation());
        head.setResourcePath(WORKSPACE + resName);
        
        int status = head.execute();
        assertEquals(status, Const.HttpStatus.NOTFOUND);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
    
  }  

  protected static HashMap<String, String> lockTokens = new HashMap<String, String>();
  
  public void testLOCK() {
    Log log = getLogger(Const.DavCommand.LOCK);
    try {
      log.info("test...");
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_file_" + i + ".info";
        
        DavLock lock = new DavLock(TestConst.getTestServerLocation());
        lock.setResourcePath(WORKSPACE + resName);
        
        int status = lock.execute();
        assertEquals(status, Const.HttpStatus.OK);
        
        String lockToken = lock.getLockToken();
        lockTokens.put(resName, lockToken);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testUNLOCK() {
    Log log = getLogger(Const.DavCommand.UNLOCK);
    try {
      log.info("test...");

      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_file_" + i + ".info";
        String lockToken = lockTokens.get(resName);
        
        DavUnLock unLock = new DavUnLock(TestConst.getTestServerLocation());
        unLock.setResourcePath(WORKSPACE + resName);
        unLock.setLockToken(lockToken);
        
        int status = unLock.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }

  public void testPROPPATCH() {
    Log log = getLogger(Const.DavCommand.PROPPATCH);
    try {
      log.info("test...");
      
      String resName = "/test_file_0.info";
      
      DavPropPatch propPatch = new DavPropPatch(TestConst.getTestServerLocation());
      propPatch.setResourcePath(WORKSPACE + resName);
      
      propPatch.setProperty(Const.DavProp.DISPLAYNAME, "NEW_DISPLAYNAME");
      propPatch.setProperty(Const.DavProp.OWNER, "GAVRIK-VETAL@UKR.NET");
      
      propPatch.removeProperty(Const.DavProp.CHECKEDIN);
      propPatch.removeProperty(Const.DavProp.COLLECTION);
      
      int status = propPatch.execute();      
      assertEquals(status, Const.HttpStatus.MULTISTATUS);
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }
  
  public void testDELETE() {
    Log log = getLogger(Const.DavCommand.DELETE);
    try {
      log.info("test...");
      
      for (int i = 0; i < FCOUNT; i++) {        
        String resName = "/test_folder_" + i + "/";
        
        DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
        delete.setResourcePath(WORKSPACE + resName);
        
        int status = delete.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "_copy";

        DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
        delete.setResourcePath(DEST_WORKSPACE + resName);
        
        int status = delete.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }

      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_folder_" + i + "_moved_another_workspace";

        DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
        delete.setResourcePath(DEST_WORKSPACE + resName);
        
        int status = delete.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }
      
      for (int i = 0; i < FCOUNT; i++) {
        String resName = "/test_file_" + i + ".info";
        
        DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
        delete.setResourcePath(WORKSPACE + resName);
        
        int status = delete.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
  }  
  
  // DeltaV tests
  
  public void testDeltaV() {
    Log log = getLogger("DeltaV");
    log.info("test...");
    
    String fileName = "/versionfile.txt";
    
    try {

      // delete if exist
      
      {
        DavPropFind propFind = new DavPropFind(TestConst.getTestServerLocation());
        propFind.setResourcePath(WORKSPACE + fileName);
        
        int status = propFind.execute();
        
        if (status == Const.HttpStatus.MULTISTATUS) {
          DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
          delete.setResourcePath(WORKSPACE + fileName);
          int delStatus = delete.execute();
          
          assertEquals(delStatus, Const.HttpStatus.NOCONTENT);
        }
      }      

      // create new
      
      {
        DavPut put = new DavPut(TestConst.getTestServerLocation());
        put.setResourcePath(WORKSPACE + fileName);

        byte []dataByte = getDataByte("BASE CONTENT");
        put.setRequestDataBuffer(dataByte);
        
        int status = put.execute();
        
        assertEquals(status, Const.HttpStatus.CREATED);
      }

      // verify
      
      {
        DavPropFind propFind = new DavPropFind(TestConst.getTestServerLocation());
        propFind.setResourcePath(WORKSPACE + fileName);
        
        int status = propFind.execute();
        assertEquals(status, Const.HttpStatus.MULTISTATUS);
      }

      // set versionable
      
      {
        DavVersionControl verControl = new DavVersionControl(TestConst.getTestServerLocation());
        verControl.setResourcePath(WORKSPACE + fileName);
        
        int status = verControl.execute();        
        assertEquals(status, Const.HttpStatus.OK);
      }
      
      // check - out
      
      {
        DavCheckOut checkOut = new DavCheckOut(TestConst.getTestServerLocation());
        checkOut.setResourcePath(WORKSPACE + fileName);
        
        int status = checkOut.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }
      
      // check - in
      
      {
        DavCheckIn checkIn = new DavCheckIn(TestConst.getTestServerLocation());
        checkIn.setResourcePath(WORKSPACE + fileName);
        
        int status = checkIn.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }

      // report
      
      {
        DavReport report = new DavReport(TestConst.getTestServerLocation());
        report.setResourcePath(WORKSPACE + fileName);
        
        int status = report.execute();
        assertEquals(status, Const.HttpStatus.MULTISTATUS);

        DocumentApi doc = report.getMultistatus();
        ArrayList<ResponseDoc> responses = ((Multistatus)doc).getResponses();
        
        assertEquals(responses.size(), 1);
      }
      
      // check - out
      
      {
        DavCheckOut checkOut = new DavCheckOut(TestConst.getTestServerLocation());
        checkOut.setResourcePath(WORKSPACE + fileName);
        
        int status = checkOut.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }      
      
      // check - in

      {
        DavCheckIn checkIn = new DavCheckIn(TestConst.getTestServerLocation());
        checkIn.setResourcePath(WORKSPACE + fileName);
        
        int status = checkIn.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }

      // report
      
      {
        DavReport report = new DavReport(TestConst.getTestServerLocation());
        report.setResourcePath(WORKSPACE + fileName);
        
        int status = report.execute();
        assertEquals(status, Const.HttpStatus.MULTISTATUS);

        DocumentApi doc = report.getMultistatus();
        ArrayList<ResponseDoc> responses = ((Multistatus)doc).getResponses();

        assertEquals(responses.size(), 2);
      }
      
      // check - out
      
      {
        DavCheckOut checkOut = new DavCheckOut(TestConst.getTestServerLocation());
        checkOut.setResourcePath(WORKSPACE + fileName);
        
        int status = checkOut.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }            
      
      // undo check - out
      
      {
        DavUnCheckOut unCheckOut = new DavUnCheckOut(TestConst.getTestServerLocation());
        unCheckOut.setResourcePath(WORKSPACE + fileName);
        
        int status = unCheckOut.execute();
        assertEquals(status, Const.HttpStatus.OK);
      }
      
      // report
      
      {
        DavReport report = new DavReport(TestConst.getTestServerLocation());
        report.setResourcePath(WORKSPACE + fileName);
        
        int status = report.execute();
        assertEquals(status, Const.HttpStatus.MULTISTATUS);

        DocumentApi doc = report.getMultistatus();
        ArrayList<ResponseDoc> responses = ((Multistatus)doc).getResponses();
        
        assertEquals(responses.size(), 2);
      }
      
      // delete
      
      {
        DavDelete delete = new DavDelete(TestConst.getTestServerLocation());
        delete.setResourcePath(WORKSPACE + fileName);
        
        int status = delete.execute();
        assertEquals(status, Const.HttpStatus.NOCONTENT);
      }
      
      log.info("complete.");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }    
    
  }

}
