/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient;

import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.commands.DavGet;
import org.exoplatform.frameworks.davclient.commands.DavLock;
import org.exoplatform.frameworks.davclient.commands.DavOptions;
import org.exoplatform.frameworks.davclient.commands.DavPropFind;
import org.exoplatform.frameworks.davclient.commands.DavPut;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class DavVSOfficeTest {

  private static Log log = ExoLogger.getLogger("jcr.DavVSOfficeTest");
  
  public static final String TEST_HOST = "localhost";
  public static final int TEST_PORT = 8080;

  //public static final String TEST_SERVLET_PATH = "/webdav";
  public static final String TEST_SERVLET_PATH = "/jcr-webdav/webdavserver";
  
  protected void showResponse(byte []data) {
    if (data != null && data.length != 0) {
      log.info("SOMEDATA PRESENT");
      
      String allResp = "";
      for (int i = 0; i < data.length; i++) {
        allResp += (char)data[i];
      }
      log.info("REPLY BODY - " + allResp);
    }    
  }
  
  private byte []getDataByte(String text) {    
    byte []dataBytes = new byte[text.length()];
    for (int i = 0; i < text.length(); i++) {
      dataBytes[i] = (byte)text.charAt(i);
    }
    return dataBytes;
  }
  
  public void testDavVSOffice() throws Exception {
    log.info("Test...");
    log.info("SERVLET - " + TEST_SERVLET_PATH);
    
    ServerLocation serverLocation = new ServerLocation();
    serverLocation.setHost(TEST_HOST);
    serverLocation.setPort(TEST_PORT);
    serverLocation.setServletPath(TEST_SERVLET_PATH);

    String folderName = "/draft/";
    String fileName = folderName + "testFile_" + System.currentTimeMillis() + ".txt";
    
    log.info("TEST FILE NAME - " + fileName);
    
    DavGet davGet = new DavGet(serverLocation);
    davGet.setResourcePath(fileName);
    
    int status = davGet.execute();
    
    log.info("GET STATUS - " + status);
    
    DavOptions davOptions = new DavOptions(serverLocation);
    davOptions.setResourcePath(folderName);
    
    status = davOptions.execute();
    
    log.info("OPTIONS STATUS - " + status);

    davGet = new DavGet(serverLocation);
    davGet.setResourcePath(fileName);
    
    status = davGet.execute();
    
    log.info("GET STATUS - " + status);
    
    DavPropFind davPropFind = new DavPropFind(serverLocation);
    davPropFind.setResourcePath(fileName);
    
    status = davPropFind.execute();
    log.info("PROPFIND STATUS - " + status);
    
    DavLock davLock = new DavLock(serverLocation);
    davLock.setResourcePath(fileName);
    
    status = davLock.execute();
    log.info("LOCK STATUS - " + status);

    String lockToken = "";
    
    if (status == Const.HttpStatus.OK) {

      lockToken = davLock.getLockToken();
      log.info("RETRIEVED LOCK TOKEN = " + lockToken);
    }
    
    davOptions = new DavOptions(serverLocation);
    davOptions.setResourcePath(fileName);
    
    status = davOptions.execute();
    log.info("OPTIONS STATUS - " + status);
    
    DavPut davPut = new DavPut(serverLocation);
    
    davPut.setLockToken(lockToken);
    
    davPut.setResourcePath(fileName);
    
    byte []dataBuffer = getDataByte("THIS FILE " + fileName);
    davPut.setRequestDataBuffer(dataBuffer);
    
    status = davPut.execute();
    log.info("PUT STATUS - " + status);
    
    log.info("Complete.");
  }
  
}
