/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.frameworks.davclient;

import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.commands.DavDelete;
import org.exoplatform.frameworks.davclient.commands.DavGet;
import org.exoplatform.frameworks.davclient.commands.DavLock;
import org.exoplatform.frameworks.davclient.commands.DavMkCol;
import org.exoplatform.frameworks.davclient.commands.DavOptions;
import org.exoplatform.frameworks.davclient.commands.DavPropFind;
import org.exoplatform.frameworks.davclient.commands.DavPut;
import org.exoplatform.frameworks.davclient.commands.DavUnLock;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class LockUnLockTest extends TestCase {

  private static Log log = ExoLogger.getLogger("jcr.LockUnLockTest");
  
  public static final String TEST_WORKSPACE = "/production";
  public static final String TEST_FOLDER = TEST_WORKSPACE + "/test_folder_" + System.currentTimeMillis();
  public static final String TEST_FILE = TEST_FOLDER + "/test_file.txt";
  public static final String TEST_FILE_CONTENT = "THIS FILE - " + TEST_FILE;
  
  public void testLockUnLock() throws Exception {
    log.info("Test...");
    
    ServerLocation serverLocation = TestConst.getTestServerLocation();
    
    DavOptions davOptions = new DavOptions(serverLocation);
    davOptions.setResourcePath(TEST_WORKSPACE);
    
    assertEquals(Const.HttpStatus.OK, davOptions.execute());
    
    DavPropFind davPropFind = new DavPropFind(serverLocation);
    davPropFind.setResourcePath(TEST_WORKSPACE);

    assertEquals(Const.HttpStatus.MULTISTATUS, davPropFind.execute());
    
    DavMkCol davMkCol = new DavMkCol(serverLocation);
    davMkCol.setResourcePath(TEST_FOLDER);

    assertEquals(Const.HttpStatus.CREATED, davMkCol.execute());
    
    davPropFind = new DavPropFind(serverLocation);
    davPropFind.setResourcePath(TEST_FOLDER);

    assertEquals(Const.HttpStatus.MULTISTATUS, davPropFind.execute());
    
    DavPut davPut = new DavPut(serverLocation);
    davPut.setResourcePath(TEST_FILE);
    
    byte []putBytes = TestUtils.getDataByte(TEST_FILE_CONTENT);
    davPut.setRequestDataBuffer(putBytes);

    assertEquals(Const.HttpStatus.CREATED, davPut.execute());
    
    DavGet davGet = new DavGet(serverLocation);
    davGet.setResourcePath(TEST_FILE);
    
    assertEquals(Const.HttpStatus.OK, davGet.execute());
    
    byte []getBytes = davGet.getResponseDataBuffer();
    for (int i = 0; i < putBytes.length; i++) {
      if (getBytes[i] != putBytes[i]) {
        log.info("PUT AND GET FILES ARE NOT EQUALS!!!");
        fail();
      }
    }
    
    DavLock davLock = new DavLock(serverLocation);
    davLock.setResourcePath(TEST_FILE);
    
    assertEquals(Const.HttpStatus.OK, davLock.execute());
    
    String lockToken = davLock.getLockToken();
    
    DavUnLock davUnLock = new DavUnLock(serverLocation);
    davUnLock.setResourcePath(TEST_FILE);
    
    davUnLock.setLockToken(lockToken);
    
    assertEquals(Const.HttpStatus.NOCONTENT, davUnLock.execute());
    
    DavDelete davDelete = new DavDelete(serverLocation);
    davDelete.setResourcePath(TEST_FILE);
    
    assertEquals(Const.HttpStatus.NOCONTENT, davDelete.execute());
    
    davDelete = new DavDelete(serverLocation);
    davDelete.setResourcePath(TEST_FOLDER);
    
    assertEquals(Const.HttpStatus.NOCONTENT, davDelete.execute());
    
    log.info("Complete.");    
  }
  
}
