/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.frameworks.davclient;

import java.util.ArrayList;
import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.Const.HttpStatus;
import org.exoplatform.frameworks.davclient.commands.DavCheckIn;
import org.exoplatform.frameworks.davclient.commands.DavCheckOut;
import org.exoplatform.frameworks.davclient.commands.DavDelete;
import org.exoplatform.frameworks.davclient.commands.DavMkCol;
import org.exoplatform.frameworks.davclient.commands.DavOptions;
import org.exoplatform.frameworks.davclient.commands.DavPropFind;
import org.exoplatform.frameworks.davclient.commands.DavReport;
import org.exoplatform.frameworks.davclient.commands.DavVersionControl;
import org.exoplatform.frameworks.davclient.documents.Multistatus;
import org.exoplatform.frameworks.davclient.documents.ResponseDoc;
import org.exoplatform.services.log.ExoLogger;
import junit.framework.TestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class NameWithSpaceTest extends TestCase {
  
  private static Log log = ExoLogger.getLogger("jcr.NameWithSpaceTest");
  
  public static final String TEST_REPO = "/production";
  public static final String TEST_FOLDER = TEST_REPO + "/Tf " + System.currentTimeMillis();
  
  public void testNameWithSpace() throws Exception {
    log.info("NameSpace test.");
    
    ServerLocation location = TestConst.getTestServerLocation();

    {
      DavOptions davOptions = new DavOptions(location);    
      assertEquals(Const.HttpStatus.OK, davOptions.execute());      
    }
    
    {
      DavPropFind davPropFind = new DavPropFind(location);
      davPropFind.setResourcePath(TEST_REPO);

      assertEquals(Const.HttpStatus.MULTISTATUS, davPropFind.execute());      
    }
    
    {
      DavMkCol davMkCol = new DavMkCol(location);
      davMkCol.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.CREATED, davMkCol.execute());
    }
    
    {
      DavPropFind davPropFind = new DavPropFind(location);
      davPropFind.setResourcePath(TEST_REPO);
      
      assertEquals(Const.HttpStatus.MULTISTATUS, davPropFind.execute());
//      Multistatus multistatus = (Multistatus)davPropFind.getMultistatus();
//      
//      ArrayList<ResponseDoc> resp = multistatus.getResponses();  
//      for (int i = 0; i < resp.size(); i++) {
//        VisualResponse.visualResponse(resp.get(i));
//      }
    }
    
    {
      DavVersionControl davVersionControl = new DavVersionControl(location);
      davVersionControl.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.OK, davVersionControl.execute());
    }
    
    {
      DavCheckIn davCheckIn = new DavCheckIn(location);
      davCheckIn.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.OK, davCheckIn.execute());
    }

    {
      DavCheckOut davCheckOut = new DavCheckOut(location);
      davCheckOut.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.OK, davCheckOut.execute());
    }

    {
      DavCheckIn davCheckIn = new DavCheckIn(location);
      davCheckIn.setResourcePath(TEST_FOLDER);

      assertEquals(Const.HttpStatus.OK, davCheckIn.execute());
    }

    {
      DavCheckOut davCheckOut = new DavCheckOut(location);
      davCheckOut.setResourcePath(TEST_FOLDER);

      assertEquals(Const.HttpStatus.OK, davCheckOut.execute());
    }

    {
      DavCheckIn davCheckIn = new DavCheckIn(location);
      davCheckIn.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.OK, davCheckIn.execute());
    }

    {
      DavReport davReport = new DavReport(location);     
      davReport.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.MULTISTATUS, davReport.execute());
//      Multistatus multistatus = (Multistatus)davReport.getMultistatus();
//      
//      ArrayList<ResponseDoc> resp = multistatus.getResponses();  
//      for (int i = 0; i < resp.size(); i++) {
//        VisualResponse.visualResponse(resp.get(i));
//      }
      
      assertEquals(3, ((Multistatus)davReport.getMultistatus()).getResponses().size());
    }
    
    {
      DavDelete davDelete = new DavDelete(location);
      davDelete.setResourcePath(TEST_FOLDER);
      
      assertEquals(Const.HttpStatus.NOCONTENT, davDelete.execute());
    }
    
    log.info("NameSpace complete.");    
  }

}
