package org.exoplatform.frameworks.jcr.webdav;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.chain.impl.ContextBase;
import org.apache.commons.chain.web.WebContext;
import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.WebDavCommandContext;
import org.exoplatform.services.webdav.WebDavServiceImpl;
import org.exoplatform.services.webdav.commands.WebDavCommand;

public class ExoJCRWebdavServlet extends HttpServlet {
    
    private static Log log = ExoLogger.getLogger("jcr.ExoJCRWebdavServlet");    
    
    protected ExoContainer container = null;
    protected WebDavServiceImpl davService = null;
    
    @Override
    public void init() throws ServletException {
      try {
        container = ExoContainerContext.getCurrentContainer();
              
        //FtpServiceImpl ftp = (FtpServiceImpl)container.getComponentInstanceOfType(FtpServiceImpl.class);        
        
        davService = (WebDavServiceImpl)container.getComponentInstanceOfType(WebDavServiceImpl.class);
      } catch (Exception exc) {
        log.info("Unhandled exception. " + exc.getMessage());
        exc.printStackTrace();
      }
    }    
    
    protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
      String commandName = request.getMethod();
      log.info("DAVCOMMAND - " + commandName);
      
      WebDavCommand command = davService.getCommand(commandName);
      if (command == null) {
        response.setStatus(405);
        log.info("Command " + commandName + " not supplied.");
        return;
      }
      
      WebDavCommandContext context = new WebDavCommandContext(
          getServletContext(), request, response, davService);
      
      long timeStart = System.currentTimeMillis();
      try {
        command.execute(context);        
      } catch (Exception exc) {
        log.info("EXECUTE COMMAND EXCEPTION. " + exc.getMessage());
        exc.printStackTrace();
      }    
      try {
        long timeEnd = System.currentTimeMillis();
        long allTime = timeEnd - timeStart;
        long msec = allTime % 1000;
        long sec = allTime / 1000;
        long min = sec / 60;
        log.info(String.format("Command %s completed at MM:SS:MS %d:%d:%d", commandName, min, sec, msec));          
      } catch (Exception exc) {
        log.info("Unhandled exception. " + exc.getMessage());
        exc.printStackTrace();
      }
    }
    
}
