/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.storage.swap;

import java.io.File;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.util.io.BLOBUtil;
import org.exoplatform.services.jcr.util.io.FSUtil;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 18.09.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: BinaryValueTempStorage.java 8556 2006-09-04 15:32:25Z peterit $
 */
public class BinaryValueTempStorage extends BinaryValueSwapStorage {

  static protected Log log = ExoLogger.getLogger("jcr.BinaryValueTempStorage");
  
  static private ThreadLocal<BinaryValueTempStorage> instance = new ThreadLocal<BinaryValueTempStorage>(); 
  
  static public String DEFAULT_ROOT_DIR = "temp/binary"; 
  static public String DEFAULT_MIN_FILE_SIZE = String.valueOf(BLOBUtil.BLOB_MIN_SIZE);
  
  static private String configRootDir = null; 
  static private String configMinFileSize = null;
  static private String configCopyLocalFiles = null;
  
  static {
    DEFAULT_ROOT_DIR = FSUtil.getTempDir() + File.separator + "exo_jcr_binary_temp";
  }
  
  BinaryValueTempStorage() {
    super(configRootDir != null ? configRootDir : DEFAULT_ROOT_DIR, 
        configMinFileSize != null ? configMinFileSize : DEFAULT_MIN_FILE_SIZE,
            configCopyLocalFiles);
  }
  
  static public void configure(String newConfigRootDir, String newConfigMinFileSize, String copyLocalFilesText) {
    
    if (configRootDir == null)
      configRootDir = newConfigRootDir;
    else
      throw new IllegalStateException("BinaryValueTempStorage already configured. It's try to set RootDir: " 
          + newConfigRootDir + ". Currently used: " + configRootDir);
    
    if (configMinFileSize == null)
      configMinFileSize = newConfigMinFileSize;
    else
      throw new IllegalStateException("BinaryValueTempStorage already configured. It's try to set MinFileSize: " 
          + newConfigMinFileSize + ". Currently used: " + configMinFileSize);
    
    if (copyLocalFilesText == null)
      configCopyLocalFiles = "false"; // optional, null means 'false'
    else 
      configCopyLocalFiles = copyLocalFilesText;
  }
  
  /**
   * Return instance of BinaryValueTempStorage mapped to the current thread.
   * If no instance is available then one will be created with existing configuration (or by default if hasn't ones).  
   */
  static public BinaryValueTempStorage getInstance() {
    if (instance.get() == null)
      instance.set(new BinaryValueTempStorage());
    return instance.get();
  }

}
