/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.dataflow.persistent;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.SessionMoveStorage;
import org.exoplatform.services.jcr.impl.core.observation.ActionLauncher;
import org.exoplatform.services.jcr.impl.dataflow.ItemsPersistenceListener;
import org.exoplatform.services.jcr.storage.WorkspaceDataContainer;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: BatcheableWorkspaceDataManager.java 7271 2006-07-21 11:58:01Z peterit $
 */

public class BatcheableWorkspaceDataManager extends VersionableWorkspaceDataManager {
  
  public BatcheableWorkspaceDataManager(WorkspaceDataContainer dataContainer,
      WorkspaceStorageCacheImpl cache) {
    super(dataContainer, cache);
  }
  
  public synchronized void beginSessionBatch(SessionImpl session) {
    savedItems.put(session, new ArrayList<ItemState>());
  }
  
  public synchronized void endSessionBatch(SessionImpl session) throws RepositoryException {
    if (savedItems.containsKey(session)) {
      List<ItemState> itemStates = savedItems.get(session);
      for (ItemsPersistenceListener listener : listeners) {
        if (listener instanceof ActionLauncher) {
          // Collect item states for observation manager, i.e. 
          // without move operation descendants source/destination both
          //long startAction = System.currentTimeMillis();
          List<ItemState> observationStates = new ArrayList<ItemState>();
          SessionMoveStorage movedStorage = session.getTransientNodesManager().getMovedStorage();
          for (ItemState state : itemStates) {
            ItemData itemData = state.getData();
            //            System.err.println("======= endSessionBatch(): item: "
            //                + itemData.getQPath().getAsString() + " " + state.getState());
            if (!movedStorage.isMoveDescendant(itemData.getQPath())) {
              if (!containsState(itemData.getQPath(), state.getState(), observationStates)) {
                observationStates.add(state);
                //                System.err.println("======= endSessionBatch(): for observation: "
                //                    + itemData.getQPath().getAsString() + " " + state.getState());
              }
            }
          }
          //log.info("   action: " + (System.currentTimeMillis() - startAction) + "ms");
          //startAction = System.currentTimeMillis();
          listener.onBatchSaveItems(observationStates, session);
          //log.info("   onBatchSave: " + (System.currentTimeMillis() - startAction) + "ms");
        } else {
          listener.onBatchSaveItems(itemStates, session);
        }
      }
      savedItems.remove(session);
    }
  }
}
