/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.session;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.MutableItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;

/**
 * Created by The eXo Platform SARL        .<br/>
 * 
 * Responsible for managing session changes log.
 * Relying on fact that ItemData inside ItemState SHOULD be TransientItemData  
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SessionChangesLog.java 7657 2006-08-07 13:23:11Z peterit $
 */
 final class SessionChangesLog extends MutableItemDataChangesLog {
   
  public SessionChangesLog(SessionImpl session) {
    super(session);
  }

  /**
   * Removes the item at the rootPath and all descendants from the log
   * @param changes
   */
  public void remove(InternalQPath rootPath) {
    List <ItemState> tmpList = new ArrayList <ItemState> ();
    tmpList.addAll(items);
    for(int i=0; i<tmpList.size(); i++) {
      TransientItemData item = (TransientItemData)tmpList.get(i).getData();
      if(item.getQPath().equals(rootPath) || 
          item.getQPath().isDescendantOf(rootPath, false))
        items.remove(i);
    }
  }
  

  /**
   * @param rootPath
   * @return item state at the rootPath and its descendants
   */
  public List <ItemState> getChanges(InternalQPath rootPath) {
    List <ItemState> list = new ArrayList <ItemState> ();
    for(int i=0; i<items.size(); i++) {
      TransientItemData item = (TransientItemData)items.get(i).getData();
      if(item.getQPath().equals(rootPath) || 
          item.getQPath().isDescendantOf(rootPath, false))
        list.add(items.get(i));
    }
    return list;
  }
  
  /**
   * @param qpath
   * @return current state (i.e. last change) of item data with qpath
   * or null if not found
   */
  public ItemState getState(InternalQPath qpath) {
    for(int i=0; i<items.size(); i++) {
      TransientItemData item = (TransientItemData)items.get(i).getData();
      if(item.getQPath().equals(qpath))
        return items.get(i);
    }
    return null;
  }

  /**
   * @param id
   * @return current state (i.e. last change) of item data with UUID == id
   * or null if not found
   */
  public ItemState getState(String id) {
    ItemState state = null;
    for(int i=0; i<items.size(); i++) {
      TransientItemData item = (TransientItemData)items.get(i).getData();
      if(item.getUUID().equals(id))
        state = items.get(i);
    }
    return state;
  }
  
  public List <ItemState> getChildStates(InternalQPath qpath, boolean forNodes) {
    List <ItemState> children = new ArrayList<ItemState>();
    List <ItemState> changes = getChanges(qpath);
    for(ItemState child: changes) {
      ItemData data = child.getData();
      children.remove(child);
      if(data.isNode() == forNodes && !data.getQPath().equals(qpath))
        children.add(child);
    }
    return children;
  }


}
