/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.ConstraintViolationException;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.JCRPath.PathElement;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.util.io.BLOBUtil;
import org.exoplatform.services.log.ExoLogger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SysNodeImporter.java 9660 2006-10-21 16:14:41Z peterit $
 */

class SysNodeImporter extends ImporteBase {

  protected Log log = ExoLogger.getLogger("jcr.SysNodeImporter");
  
  private Stack<NodeInfo>     tree;

  private String    curPropName;

  private String    curPropType;

  private List<StringBuffer> curPropValues;

  private List<NodeInfo>      nodeInfos;

  private NodeImpl parent;

  SysNodeImporter(NodeImpl parent, int uuidBehavior) throws RepositoryException {
    super(parent, uuidBehavior);
    this.parent = parent;
    this.tree = new Stack<NodeInfo>();
    this.nodeInfos = new ArrayList<NodeInfo>();
    this.curPropValues = new ArrayList<StringBuffer>();
  }

  NodeImpl getNode() throws RepositoryException {
    
    return rootNode;
  }
  
  private NodeImpl buildNode() throws RepositoryException {

    // build node from parsed infos
    NodeImpl node = null;

    Map<String, NodeImpl> parents = new LinkedHashMap<String, NodeImpl>();
    
    for (int i = 0; i < nodeInfos.size(); i++) {
      NodeInfo info = nodeInfos.get(i);
               
      SessionImpl sessionImpl = session;
      LocationFactory loFactory = sessionImpl.getLocationFactory();
      
      NodeImpl parentNode = null;
      String relPathStr = info.getRelPath();
      // serach for parent of this rel path
      int lastPathElem = relPathStr.lastIndexOf("/");
      if (lastPathElem > 0) {
        String relPathParentStr = relPathStr.substring(0, lastPathElem);
        // in case of empty last elem of rel path - JCR will throw an exception below
        relPathStr = lastPathElem < relPathStr.length() - 1 ? relPathStr.substring(lastPathElem + 1) : "";
        NodeImpl pathParent = parents.get(relPathParentStr);
        parentNode = pathParent != null ? pathParent : parent;
      } else {
        parentNode = parent;
      }
      
      // Traversing node info for properties, primaryType, mixins... 
      InternalQName primaryTypeName = null;
      InternalQName[] mixinTypeNames = null;
      
      String uuid = null;
      List<NodeTypeImpl> nodeTypes = new ArrayList<NodeTypeImpl>();
      
      List<PropertyInfo> props = info.getProperties();
      List<ParsedPropertyInfo> propsParsed = new ArrayList<ParsedPropertyInfo>();
      
      for (PropertyInfo prop: props) {
        InternalQName propName = loFactory.parseJCRName(prop.getName()).getInternalName();
        List<StringBuffer> valueList = prop.getValues();
        
        if (propName.equals(Constants.JCR_PRIMARYTYPE)) {
          if (valueList.size() > 0) {
            primaryTypeName = loFactory.parseJCRName(
                new String(valueList.get(0))).getInternalName();
            nodeTypes.add(findNodeType(primaryTypeName));
          } else
            log.warn("Imported property " + parentNode.getPath() + "/jcr:primaryType has empty value");
        } else if (propName.equals(Constants.JCR_MIXINTYPES)) {
          if (valueList.size() > 0) {
            mixinTypeNames = new InternalQName[valueList.size()];
          } else
            log.warn("Imported property " + parentNode.getPath() + "/jcr:mixinTypes has empty value(s)");
        } else if (propName.equals(Constants.JCR_UUID)) {
          if (valueList.size() > 0)
            uuid = new String(valueList.get(0));
          else
            log.warn("Imported property " + parentNode.getPath() + "/jcr:uuid has empty value");
        }
        
        Value[] values = new Value[valueList.size()];
        String valStr = "";
        for (int k = 0; k < valueList.size(); k++) {
          String val = new String(valueList.get(k));
          if (prop.getType() == PropertyType.BINARY) {
            values[k] = session.getValueFactory().createValue(
                new ByteArrayInputStream(Base64.decodeBase64(val.getBytes())));
          } else {
            values[k] = session.getValueFactory().createValue(val, prop.getType());
            if (propName.equals(Constants.JCR_MIXINTYPES)) {
              //mixinTypeNames[k] = ((NameValue) values[k]).getQName(); // isn't safe 
              mixinTypeNames[k] = loFactory.parseJCRName(val).getInternalName();
              nodeTypes.add(findNodeType(mixinTypeNames[k]));
            }
            valStr += val + " ";
          }
        }
        
        if (log.isDebugEnabled())
          log.debug("prop(1): " + prop.getName() + ", [" 
              + valStr.trim() + "], " + PropertyType.nameFromValue(prop.getType()));
        
        propsParsed.add(new ParsedPropertyInfo(propName, prop.getType(), values));
      }
      
      // check UUID Behavior of the import
      boolean hasMixReferenceable = isReferenceable(nodeTypes);
      if (hasMixReferenceable && uuid != null) {
        try {
          NodeImpl sameUuidNode = (NodeImpl) session.getNodeByUUID(uuid);
          switch (uuidBehavior) {
          case ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW:
            // Incoming referenceable nodes are assigned newly created UUIDs
            // upon addition to the workspace. As a result UUID collisions
            // never occur.

            // reset UUID and it will be autocreated in session
            uuid = null;
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace then the already existing
            // node (and its subtree) is removed from wherever it may be in
            // the workspace before the incoming node is added. Note that this
            // can result in nodes �disappearing� from locations in the
            // workspace that are remote from the location to which the
            // incoming subtree is being written.

            NodeIterator samePatterns = sameUuidNode.getNodes(parentNode.getName());
            if (samePatterns.hasNext()) {
              throw new ConstraintViolationException(
                  "A uuidBehavior is set to IMPORT_UUID_COLLISION_REMOVE_EXISTING and an incoming node has the same UUID as the node at parentAbsPath or one of its ancestors");
            }
            // remove node
            sameUuidNode.remove();
            sameUuidNode.getParent().save();
            sameUuidNode = null;
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace, then the already existing
            // node is replaced by the incoming node in the same position as
            // the existing node. Note that this may result in the incoming
            // subtree being disaggregated and �spread around� to different
            // locations in the workspace. In the most extreme case this
            // behavior may result in no node at all being added as child of
            // parentAbsPath. This will occur if the topmost element of the
            // incoming XML has the same UUID as an existing node elsewhere in
            // the workspace.

            // replace in same location
            parentNode = (NodeImpl) sameUuidNode.getParent();
            sameUuidNode.remove();
            parentNode.save();
            sameUuidNode = null;
            break;
          case ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW:
            // If an incoming referenceable node has the same UUID as a node
            // already existing in the workspace then a SAXException is thrown
            // by the ContentHandler during deserialization.
            throw new ItemExistsException(
                "An incoming referenceable node has the same UUID as a node already existing in the workspace!");
          default:
          }
        } catch (ItemNotFoundException e) {
          // node not found, it's ok - willing create one new
        }
      }
      // [PN] 26.09.06 Imposible state as we will fix path for sns below
//      else {
//        // other cases with same names etc.
//        try {
//          // Check for node with this name in session
//          NodeImpl sameNameNode = (NodeImpl) parentNode.getNode(path.getName().getAsString());
//          if (!sameNameNode.getDefinition().allowsSameNameSiblings()) {
//            String msg = "A node with this name (" + parentNode.getPath() + "/" 
//              + path.getName().getAsString() + ") is already exists. Same-name sibling name nodes is not allowed."
//              + "Overwriting an existing node during import is impossible.";
//            throw new ItemExistsException(msg);
//          }
//        } catch (PathNotFoundException e) {
//          // ok
//        }
//      }      
      
      // build current path of the imported node
      JCRPath path = loFactory.createJCRPath(parentNode.getLocation(), relPathStr);
      
      // same-name sibling nodes path compilation (path indexes fix, current parent calculate???)
      int depthIndex = path.getDepth() - parentNode.getLocation().getDepth();
      PathElement[] relPathElems = path.getRelPath(depthIndex);
      if (log.isDebugEnabled())
        log.debug("BUILD NODE, PREPARE: '" + parentNode.getLocation().getAsString(true) 
            + "' + '" + relPathStr + "', check from depth: " + depthIndex );
      for (int depth = 0; depth < relPathElems.length; depth++) {
        PathElement pathElement = relPathElems[depth];
        if (log.isDebugEnabled())
          log.debug("BUILD NODE, NODE: '" + pathElement.getAsString(true) + "', depth: " + depth );
        try {
          NodeIterator snsNodes = parentNode.getNodes(pathElement.getAsString());
          if (depth < relPathElems.length - 1) {
            pathElement.setIndex((int) snsNodes.getSize());
            try {
              parentNode = (NodeImpl) parentNode.getNode(pathElement.getAsString(false));
              if (log.isDebugEnabled())
                log.debug("BUILD NODE, <<< NEW ANCESTOR for RELATIVE path >>> : '" + parentNode.getPath() + "', depth: " + depth + ", path: '" + path.getAsString(true) + "'");
            } catch (PathNotFoundException e) {
              log.warn("Next parent not found: '" + parentNode.getPath() + "', depth: " + depth
                  + ", relPath: '" + path.getAsString(true) + "'");
            }
          } else if (depth == relPathElems.length - 1) {
            pathElement.setIndex((int) snsNodes.getSize() + 1);
          }
          if (log.isDebugEnabled())
            log.debug("BUILD NODE, FIX ANCESTOR: '" + parentNode.getPath() + "', depth: " + depth + ", path: '" + path.getAsString(true) + "'");
        } catch(PathNotFoundException e) {
          log.warn("Node not found: '" + parentNode.getPath() + "', depth: " + depth 
              + ", relPath: '" + path.getAsString(true) + "'");
        } catch(RepositoryException e) {
          log.warn("Error of same-name-sibling nodes processing: " + e.getMessage() 
              + ". Node: " + parentNode.getPath());
          // if exception will be thrown a node will be created at relPath
        }
      }

      InternalQName name = path.getInternalPath().getName();

      //log.info("BUILD NODE: " + name.getAsString() + ":" + relPath.getIndex() + " " + primaryTypeName.getAsString() + " " + parentNode.getPath());
      
      if (mixinTypeNames == null)
        mixinTypeNames = new InternalQName[0];
      
      NodeImpl newNode = parentNode.createChildNodeInmemory(name, primaryTypeName, mixinTypeNames,
          hasMixReferenceable ? uuid : null,   // uuid 
          path.getIndex(),      // [PN] 01.06.06 index, relPath.getIndex()
          true,  // registerInDataManager
          true,  // doExternalValidation
          false);  // addAutoCreated
      
      if (log.isDebugEnabled())
        log.debug("node: " + newNode.getPath() + ", " + path.getIndex() 
          + ", " + uuid
          + ", " + primaryTypeName.getAsString()
          + ", " + (mixinTypeNames.length > 0 ? mixinTypeNames[0].getAsString() + "..." : ""));
      
      for (ParsedPropertyInfo prop: propsParsed) {
        if (log.isDebugEnabled()) {
          String vals = "";
          for (int m = 0; m<prop.getValues().length; m++) {
            BaseValue v = (BaseValue) prop.getValues()[m];
            try {
              vals += new String((BLOBUtil.readValue(v.getData()))
                + (m < prop.getValues().length - 1 ? "," : ""));
            } catch(IOException e) {
              log.error("Debug eror: ", e);
            }
          }
          log.debug("prop(2): " + prop.getName().getAsString() + ", [" + vals + "], " 
              + PropertyType.nameFromValue(prop.getType()));
        }
        
        Value[] values = null;
        if (prop.getName().equals(Constants.JCR_UUID) && hasMixReferenceable && uuid == null) {
          // as a result of ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW algorithm
          values = new Value[1];
          values[0] = sessionImpl.getValueFactory().createValue(newNode.getInternalUUID());
        } else {
          values = prop.getValues();
        }
        
        newNode.createChildPropertyInmemory(prop.getName(), prop.getType(), values, true, false); 
      }
      
      parents.put(info.getRelPath(), newNode); // add one new parent
      
      if (i == 0) {
        node = newNode;
      }
    }
    return node;
  }

  public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
      throws SAXException {
    if (qName.equals("sv:node")) {
      try {
        String name = atts.getValue("sv:name");
        String relPath;
        if (!tree.isEmpty())
          relPath = (tree.peek()).getRelPath() + "/" + name;
        else
          relPath = name;
        
        NodeInfo info = new NodeInfo(relPath);
        tree.push(info);
        nodeInfos.add(info);
      } catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:property")) {
      // TODO manage UUID
      try {
        curPropType = atts.getValue("sv:type");
        curPropName = atts.getValue("sv:name");
        curPropValues = new ArrayList<StringBuffer>();        
      } catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
      }
    } else if (qName.equals("sv:value")) {
      // [PN] 30.05.06
      StringBuffer curPropValue = new StringBuffer();
      curPropValues.add(curPropValue);
    } else {
      throw new SAXException("'" + qName
          + "' is not allowed. Only sv:node, sv:property and sv:value are allowed");
    }
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    if (qName.equals("sv:node")) {
      // nodeInfos.add(tree.pop());
      tree.pop();
    } else if (qName.equals("sv:property")) {
      tree.peek().addProperty(curPropName, curPropType, curPropValues);
    } else if (qName.equals("sv:value")) {
      // no thing to do 
    }
  }

  public void characters(char[] ch, int start, int length) throws SAXException {
    // [PN] 30.05.06
    // property values
    if (curPropValues.size()>0) {
      StringBuffer curPropValue = curPropValues.get(curPropValues.size()-1);
      curPropValue.append(ch, start, length);
      //curPropValues.add(curPropValue);
    } else {
      // wrong XML, no sv:value visited before
      log.warn("Wrong XML content. Element 'sv:value' expected, but SAX event 'characters' occured. characters:["
          + new String(ch) + "]");
    }
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
    nodeInfos.clear();
  }

  public void endDocument() throws SAXException {
    // so, build the node parsed before
    // and put it in the session
    try {
      rootNode = buildNode();
    } catch(Exception e) {
      e.printStackTrace();
      //causeException = e;
      throw new SAXException("Error create node from System View document. Exception: " + 
          e.getMessage(), e);
    }
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  private class NodeInfo {
    private String    relPath;

    private List<PropertyInfo> properties = new ArrayList<PropertyInfo>();

    public NodeInfo(String relPath) {
      this.relPath = relPath;
    }

    public void addProperty(String name, String typeName, List<StringBuffer> strValues) {
      int type = ExtendedPropertyType.valueFromName(typeName);
      PropertyInfo prop = new PropertyInfo(name, type, strValues);
      properties.add(prop);
    }

    public List<PropertyInfo> getProperties() {
      return properties;
    }

    public String getRelPath() {
      return relPath;
    }

    @Deprecated
    public String getPrimaryTypeName() throws RepositoryException {
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = properties.get(i);
        if (prop.getName().equals("jcr:primaryType"))
          return new String(prop.getValues().get(0));
      }
      throw new RepositoryException("No primary type ");
    }

    @Deprecated
    public String[] getMixinTypeNames() {
      List<String> mixins = new ArrayList<String>();
      for (int i = 0; i < properties.size(); i++) {
        PropertyInfo prop = properties.get(i);
        if (prop.getName().equals("jcr:mixinTypes")) {
          for (int j = 0; j < prop.getValues().size(); j++)
            mixins.add(new String(prop.getValues().get(j)));
        }
      }

      String[] mixinProps = new String[mixins.size()];
      for (int i = 0; i < mixins.size(); i++) {
        mixinProps[i] = mixins.get(i);
      }
      return mixinProps;
    }

  }

  private class ParsedPropertyInfo {
    
    private final InternalQName name;
    private final int    type;
    private final Value[]   values;
    
    public ParsedPropertyInfo(InternalQName name, int type, Value[] values) {
      super();
      this.name = name;
      this.type = type;
      this.values = values;
    }

    public InternalQName getName() {
      return name;
    }

    public int getType() {
      return type;
    }

    public Value[] getValues() {
      return values;
    }    
  }
  
  private class PropertyInfo {

    private String name;

    private int    type;

    private List<StringBuffer>   values;

    public PropertyInfo(String name, int type, List<StringBuffer> values) {
      super();
      this.name = name;
      this.type = type;
      this.values = values;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
      return name;
    }

    /**
     * @return Returns the type.
     */
    public int getType() {
      return type;
    }

    /**
     * @return Returns the values.
     */
    public List<StringBuffer> getValues() {
      return values;
    }
  }

}
