/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.rmi.impl;

import java.net.InetAddress;
import java.net.URL;
import java.rmi.Naming;
import java.rmi.UnknownHostException;
import java.util.List;

import org.apache.jackrabbit.rmi.remote.RemoteRepository;
import org.apache.jackrabbit.rmi.server.RemoteAdapterFactory;
import org.apache.jackrabbit.rmi.server.ServerAdapterFactory;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.rmi.RMIBinder;
import org.picocontainer.Startable;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: RMIBinderImpl.java 7164 2006-07-19 07:31:09Z peterit $
 */

public class RMIBinderImpl implements RMIBinder, Startable {

  private int rmiPort;
  private boolean autoBind;
  private boolean bound = false;
  private RemoteAdapterFactory factory = null;
  private RepositoryService service = null;

  public RMIBinderImpl(InitParams params, RepositoryService service) {
    rmiPort = Integer.valueOf((String) params.getValueParam("rmiPort").getValue()).intValue();
    autoBind = Boolean.valueOf((String) params.getValueParam("autoBind").getValue()).booleanValue();
    factory = new ServerAdapterFactory();
    this.service = service;
  }

  public void bind() {
    if (bound)
      return;
    String bindName = "";
    try {
      List config = service.getConfig().getRepositoryConfigurations();
      for (int i = 1; i <= config.size(); i++) {
        RepositoryEntry conf = (RepositoryEntry) config.get(i - 1);
        RemoteRepository remote = factory.getRemoteRepository(service.getRepository(conf.getName()));
        
        InetAddress addr = InetAddress.getLocalHost();
        String hostname = addr.getHostName();
        
        bindName = "//" + hostname;
        if (rmiPort > 0)
          bindName = bindName + ":" + String.valueOf(rmiPort);
        bindName = bindName + "/" + conf.getName();
        Naming.bind(bindName, remote);
      }
      bound = true;
    } catch (Exception e) {
      throw new RuntimeException(bindName + ": " + e.toString(), e);
    }
  }

  public void unbind() {
    String bindName = "";
    try {
      List config = service.getConfig().getRepositoryConfigurations();
      for (int i = 1; i <= config.size(); i++) {
        RepositoryEntry conf = (RepositoryEntry) config.get(i - 1);
        
        InetAddress addr = InetAddress.getLocalHost();
        String hostname = addr.getHostName();
        
        bindName = "//" + hostname;
        if (rmiPort > 0)
          bindName = bindName + ":" + String.valueOf(rmiPort);
        bindName = bindName + "/" + conf.getName();
        Naming.unbind(bindName);
      }
      bound = false;
    } catch (Exception e) {
      throw new RuntimeException(bindName + ": " + e.toString(), e);
    }
  }

  public void start() {
    if (autoBind)
      bind();
  }

  public void stop() {
    if (autoBind)
      unbind();
  }
}
