/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav;

import org.exoplatform.services.webdav.request.AclPrincipalPropSetDoc;
import org.exoplatform.services.webdav.request.AclPrincipalPropsDoc;
import org.exoplatform.services.webdav.request.ExpandPropertyDoc;
import org.exoplatform.services.webdav.request.LockInfoDoc;
import org.exoplatform.services.webdav.request.PropFindDoc;
import org.exoplatform.services.webdav.request.PropertyBehaviorDoc;
import org.exoplatform.services.webdav.request.PropertyUpdateDoc;
import org.exoplatform.services.webdav.request.VersionTreeDoc;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class DavConst {

    public static final String WDBDAV_COMMAND_CATALOG = "WEBDAV";
  
    public static final String DAVHTTPVERSION = "HTTP/1.1";

    public static final String DAV_MSAUTHORVIA = "DAV";
    public static final String DAV_NAMESPACE = "DAV:";
    public static final String DAV_PREFIX = "D:";
    public static final String DAV_SERVER = "eXo-Webdav Server /1.0";
    public static final String DAV_VERSIONS = "1, 2";
    public static final String DAV_VERSIONIDENTIFIER = "VERSIONID";
    public static final String DAV_VERSIONPREFIX = "?" + DAV_VERSIONIDENTIFIER + "=";
    public static final String DAV_DEFAULT_MIME_TYPE = "text/plain";
    
    public static final String [][]AVAILABLE_DOCUMENTS = {
      {DavConst.DavDocument.ACLPRINCIPALPROPS,
          AclPrincipalPropsDoc.class.getCanonicalName()},
      {DavConst.DavDocument.ACLPRINCIPALPROPSET,
          AclPrincipalPropSetDoc.class.getCanonicalName()},
      {DavConst.DavDocument.EXPANDPROPERTY,
          ExpandPropertyDoc.class.getCanonicalName()},
      {DavConst.DavDocument.LOCKINFO,
          LockInfoDoc.class.getCanonicalName()},
      {DavConst.DavDocument.PROPERTYBEHAVIOR,
          PropertyBehaviorDoc.class.getCanonicalName()},
      {DavConst.DavDocument.PROPERTYUPDATE,
          PropertyUpdateDoc.class.getCanonicalName()},
      {DavConst.DavDocument.PROPFIND,
          PropFindDoc.class.getCanonicalName()},
      {DavConst.DavDocument.VERSIONTREE,
          VersionTreeDoc.class.getCanonicalName()}
    };    
    
    public class HttpStatus {
      public static final int OK = 200;
      public static final int CREATED = 201;
      public static final int NOCONTENT = 204; 
      public static final int MULTISTATUS = 207;
      public static final int AUTHNEEDED = 401;
      public static final int NOTFOUND = 404;
      public static final int PRECONDITIONFAILED = 412;     
      public static final int CONFLICT = 409;    
      public static final int FORBIDDEN = 403;
      public static final int METHODNOTALLOWED = 405;
      public static final int UNSUPPORTEDMEDIATYPE = 415;
      public static final int INSUFFICIENTSTORAGE = 507;
      public static final int NOTIMPLEMENTED = 501;
    }
    
    public class StatusDescr {
      public static final String OK = DAVHTTPVERSION + " 200 Ok.";
      public static final String NOTFOUND = DAVHTTPVERSION + " 404 Not Found.";
      public static final String CONFLICT = DAVHTTPVERSION + " 409 Conflict.";       
    }

    public class Headers {
      public static final String ACCEPTENCODING = "Accept-Encoding";
      public static final String ALLOW = "Allow";
      public static final String CONNECTION = "Connection";
      public static final String CONTENTLENGTH = "Content-Length";
      public static final String CONTENTTYPE = "Content-type";
      public static final String DAV = "DAV";
      public static final String DEPTH = "Depth";
      public static final String DESTINATION = "Destination";
      public static final String HOST = "Host";
      public static final String LOCKTOKEN = "Lock-Token";
      public static final String LASTMODIFIED = "Last-Modified";
      public static final String MSAUTHORVIA = "MS-Author-Via";
      public static final String OVERWRITE = "Overwrite";
      public static final String TE = "TE";
      public static final String TIMEOUT = "Timeout";
      public static final String TRANSLATE = "Translate";
      public static final String USERAGENT = "User-Agent";
      public static final String XFEATURES = "X-Features";
      public static final String DATE = "Date";
      public static final String SERVER = "Server";
      public static final String WWWAUTHENTICATE = "WWW-Authenticate";
      public static final String AUTHORIZATION = "Authorization";
      public static final String CACHECONTROL = "Cache-Control";
      public static final String IF = "If";
    }
    
    public class DavDocument {
      public static final String ACLPRINCIPALPROPS = "acl-principal-props";
      public static final String ACLPRINCIPALPROPSET = "acl-principal-prop-set";
      public static final String EXPANDPROPERTY = "";
      public static final String LOCKINFO = "lockinfo";
      public static final String PROPERTYBEHAVIOR = "";
      public static final String PROPERTYUPDATE = "propertyupdate";
      public static final String PROPFIND = "propfind";
      public static final String VERSIONTREE = "version-tree";      
    }
    
    public class DavProperty {
      public static final String ACTIVELOCK = "activelock";
      public static final String ALLPROP = "allprop";
      public static final String CHECKEDIN = "checked-in";
      public static final String CHECKEDOUT = "checked-out";
      public static final String COLLECTION = "collection";    
      public static final String CREATIONDATE = "creationdate";
      public static final String CREATORDISPLAYNAME = "creator-displayname";
      public static final String DEPTH = "depth";
      public static final String DISPLAYNAME = "displayname";
      public static final String EXCLUSIVE = "exclusive";
      public static final String GETCONTENTLENGTH = "getcontentlength";
      public static final String GETCONTENTTYPE = "getcontenttype";
      public static final String GETLASTMODIFIED = "getlastmodified";
      public static final String HREF = "href";
      public static final String INCLUDE = "include";
      public static final String LOCKDISCOVERY = "lockdiscovery";
      public static final String LOCKINFO = "lockinfo";
      public static final String LOCKSCOPE = "lockscope";
      public static final String LOCKTOKEN = "locktoken";
      public static final String LOCKTYPE = "locktype";
      public static final String MULTISTATUS = "multistatus";
      public static final String OWNER = "owner";
      public static final String PROP = "prop";
      public static final String PROPFIND = "propfind";
      public static final String PROPSTAT = "propstat";
      public static final String REMOVE = "remove";
      public static final String RESOURCETYPE = "resourcetype";
      public static final String RESPONSE = "response";
      public static final String SET = "set";
      public static final String SHARED = "shared";
      public static final String STATUS = "status";
      public static final String SUCCESSORSET = "successor-set";
      public static final String TIMEOUT = "timeout";
      public static final String VERSIONHISTORY = "version-history";
      public static final String VERSIONNAME = "version-name";
      public static final String WRITE = "write";
    }
     
    public class ResourceType {
      public static final String COLLECTION = "collection";
      public static final String RESOURCE = "resource";
      
    }

    public class Lock {
      public static final String SCOPE_SHARED = "shared";
      public static final String SCOPE_EXCLUSIVE = "exclusive";
      public static final String TYPE_WRITE = "write";      
    }

    public class NodeTypes {
      public static final String JCR_CONTENT = "jcr:content";
      public static final String JCR_DATA = "jcr:data";
      public static final String JCR_FROZENNODE = "jcr:frozenNode";
      public static final String JCR_LOCKOWNER = "jcr:lockOwner";
      public static final String NT_VERSION = "nt:version";
      public static final String JCR_CREATED = "jcr:created";
      public static final String NT_FILE = "nt:file";
      public static final String JCR_ROOTVERSION = "jcr:rootVersion";
      public static final String JCR_LASTMODIFIED = "jcr:lastModified";
      public static final String JCR_MIMETYPE = "jcr:mimeType";
      public static final String NT_RESOURCE = "nt:resource";
      public static final String MIX_LOCKABLE = "mix:lockable";
      public static final String MIX_VERSIONABLE = "mix:versionable";
      public static final String NT_FOLDER = "nt:folder";
    }
    
    public class DateFormat {
      public static final String CREATION = "yyyy-MM-dd'T'HH:mm:ss'Z'";
      public static final String MODIFICATION = "EEE, dd MMM yyyy HH:mm:ss z";
    }
    
}
