/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav;

import org.apache.commons.logging.Log;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class WebDavConfigImpl implements WebDavConfig {
  
  private static Log log = ExoLogger.getLogger("jcr.DavConfig");

  public static final String INIT_PARAM_DEFAULT_IDENTITY = "default-identity";
  public static final String INIT_PARAM_AUTH_HEADER = "auth-header";
  public static final String INIT_PARAM_AUTO_MIX_LOCKABLE = "auto-mix-lockable";
  public static final String INIT_PARAM_DEF_FOLDER_NODE_TYPE = "def-folder-node-type";
  public static final String INIT_PARAM_DEF_FILE_NODE_TYPE = "def-file-node-type";
  public static final String INIT_PARAM_DEF_FILE_MIME_TYPE = "def-file-mimetype";
  public static final String INIT_PARAM_CATALOG_NAME = "catalog-name";
  public static final String INIT_PARAM_REPOSITORY_NAME = "repository-name";
  public static final String INIT_PARAM_UPDATE_POLICY = "update-policy";
  
  public static final String INIT_VALUE_CREATE_VERSION = "create-version";
  public static final String INIT_VALUE_REPLACE = "replace";
  public static final String INIT_VALUE_ADD = "add"; 
  
  protected String _defIdentity = "anonymous:anonymous";
  protected String _authHeader = "";
  protected boolean _autoMixLockable = false;
  protected String _defFolderNodeType = DavConst.NodeTypes.NT_FOLDER;
  protected String _defFileNodeType = DavConst.NodeTypes.NT_FILE;
  protected String _defFileMimeType = DavConst.DAV_DEFAULT_MIME_TYPE;
  protected String _catalogName = "";
  protected String _repositoryName = "";
  protected String _updatePolicyType = INIT_VALUE_ADD;

  public WebDavConfigImpl(InitParams params) {
  
    ValueParam pIdentity = params.getValueParam(INIT_PARAM_DEFAULT_IDENTITY);
    if (pIdentity != null) {
      _defIdentity = pIdentity.getValue();
      log.info(INIT_PARAM_DEFAULT_IDENTITY + " = " + _defIdentity);
    }
    
    ValueParam pAuthHeader = params.getValueParam(INIT_PARAM_AUTH_HEADER);
    if (pAuthHeader != null) {
      _authHeader = pAuthHeader.getValue();
      log.info(INIT_PARAM_AUTH_HEADER + " = " + _authHeader);
    }
    
    ValueParam pAutoMixLocable = params.getValueParam(INIT_PARAM_AUTO_MIX_LOCKABLE);
    if (pAutoMixLocable != null) {
      _autoMixLockable = true;
      log.info(INIT_PARAM_AUTO_MIX_LOCKABLE + " = " + _autoMixLockable);
    }

    ValueParam pDefFolderNodeType = params.getValueParam(INIT_PARAM_DEF_FOLDER_NODE_TYPE);
    if (pDefFolderNodeType != null) {
      _defFolderNodeType = pDefFolderNodeType.getValue();
      log.info(INIT_PARAM_DEF_FOLDER_NODE_TYPE + " = " + _defFolderNodeType);
    } 
    
    ValueParam pDefFileNodeType = params.getValueParam(INIT_PARAM_DEF_FILE_NODE_TYPE);
    if (pDefFileNodeType != null) {
      _defFileNodeType = pDefFileNodeType.getValue();
      log.info(INIT_PARAM_DEF_FILE_NODE_TYPE + " = " + _defFileNodeType);
    }
    
    ValueParam pDefFileMimeType = params.getValueParam(INIT_PARAM_DEF_FILE_MIME_TYPE);
    if (pDefFileMimeType != null) {
      _defFileMimeType = pDefFileMimeType.getValue();
      log.info(INIT_PARAM_DEF_FILE_MIME_TYPE + " = " + _defFileMimeType);
    }
    
    ValueParam pCatalogName = params.getValueParam(INIT_PARAM_CATALOG_NAME);
    if (pCatalogName != null) {
      _catalogName = pCatalogName.getValue();
      log.info(INIT_PARAM_CATALOG_NAME + " = " + _catalogName);
    }
    
    ValueParam pRepositoryName = params.getValueParam(INIT_PARAM_REPOSITORY_NAME);
    if (pRepositoryName != null) {
      _repositoryName = pRepositoryName.getValue();
      log.info(INIT_PARAM_REPOSITORY_NAME + " = " + _repositoryName);
    }
    
    ValueParam pUpdatePolicy = params.getValueParam(INIT_PARAM_UPDATE_POLICY);
    if (pUpdatePolicy != null) {
      _updatePolicyType = pUpdatePolicy.getValue();
      log.info(INIT_PARAM_UPDATE_POLICY + " = " + _updatePolicyType);
    }
    
  }
  
  public String getDefIdentity() {
    return _defIdentity;
  }
  
  public String getAuthHeader() {
    return _authHeader;
  }
  
  public boolean isAutoMixLockable() {
    return _autoMixLockable;
  }
  
  public String getDefFolderNodeType() {
    return _defFolderNodeType;
  }
  
  public String getDefFileNodeType() {
    return _defFileNodeType;
  }
  
  public String getDefFileMimeType() {
    return _defFileMimeType;
  }

  public String getCataogName() {
    return _catalogName;
  }
  
  public String getRepositoryName() {
    return _repositoryName; 
  }
  
  public String getUpdatePolicyType() {
    return _updatePolicyType;
  }
  

}
