/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import javax.jcr.RepositoryException;
import org.apache.commons.chain.Catalog;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.command.impl.CommandService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.webdav.commands.WebDavCommand;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class WebDavServiceImpl implements WebDavService {
  
  public static final String COMMANDS_PATH = "/conf/webdav-commands.xml";
  
  protected RepositoryService  repositoryService = null;
  protected WebDavSessionProvider sessionProvider = null;
  protected Catalog commandCatalog = null;
  protected WebDavConfig config = null;
  protected FakeLockTable lockTable = null;
  
  public WebDavServiceImpl (
      InitParams params,
      CommandService commandService, 
      RepositoryService repositoryService) throws Exception {
    
    this.repositoryService = repositoryService;
    
    InputStream commandsStream = getClass().getResourceAsStream(COMMANDS_PATH);    
    commandService.putCatalog(commandsStream);
    
    commandCatalog = commandService.getCatalog(DavConst.WDBDAV_COMMAND_CATALOG);
    
    config = new WebDavConfigImpl(params);

    sessionProvider = new WebDavSessionProvider(this, repositoryService.getRepository());
    
    lockTable = new FakeLockTable();
  }

  public WebDavConfig getConfig() {
    return config;
  }

  public ArrayList<String> getAvailableCommands() {
    ArrayList<String> commands = new ArrayList<String>();    
    Iterator<String> commandIter = commandCatalog.getNames();
    while (commandIter.hasNext()) {
      String curCommand = commandIter.next();
      commands.add(curCommand);
    }
    return commands;
  }

  public WebDavCommand getCommand(String commandName) {
    WebDavCommand command = (WebDavCommand)commandCatalog.getCommand(commandName);
    return command;
  }
  
  public ManageableRepository getRepository() throws RepositoryException, RepositoryConfigurationException {
    String repositoryName = config.getRepositoryName();
    if (repositoryName == null) {
      return repositoryService.getRepository();
    } else {
      return repositoryService.getRepository(repositoryName);
    }
  }
  
  public WebDavSessionProvider getSessionProvider() {
    return sessionProvider;
  }
  
  public FakeLockTable getLockTable() {
    return lockTable;
  }
  
}
