/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.commands;

import java.io.InputStream;
import java.util.ArrayList;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.response.DavFileInfo;
import org.exoplatform.services.webdav.response.HtmlContentManager;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class GetCommand extends WebDavCommand {
  
  private static Log log = ExoLogger.getLogger("jcr.GetCommand");

  public GetCommand() {
    super();
    needSrcSession = false;
  }
  
  @Override
  protected boolean process() {
    DavFileInfo resInfo = null;
    if ("".equals(davRequest().getSrcWorkspace())) {
      resInfo = new DavFileInfo();
      resInfo.setInputStream(getHtmlWorkspaces());      
    } else {
      srcSession.set(davRequest().getSourceSession(davContext().getSessionProvider()));
      if (srcSession.get() == null) {
        String defAuthHeader = davContext().getConfig().getAuthHeader();
        davResponse().answerAuthNeeded(defAuthHeader);
        return false;
      }
      resInfo = getFile();
    }

    if (resInfo == null || resInfo.getInputStream() == null) {
      davResponse().answerNotFound();
      return false;
    }
    
    if (resInfo.getContentType() != null) {
      davResponse().addHeader(DavConst.Headers.CONTENTTYPE, resInfo.getContentType());
    }
    
    if (resInfo.getLastModified() != null) {
      davResponse().addHeader(DavConst.Headers.LASTMODIFIED, resInfo.getLastModified());
    }
    int contentLength = resInfo.getContentLength();
    davResponse().addHeader(DavConst.Headers.CONTENTLENGTH, String.format("%s", contentLength));
    
    davResponse().answerOk();
    
    try {
      byte []buff = new byte[4096];
      int res = 0;
      while ((res = resInfo.getInputStream().read(buff)) > 0) {
        davResponse().writeToResponse(buff, res);
      }
    } catch (Exception exc) {
      exc.printStackTrace();
      return false;
    }
    
    return true;    
  }  
  
  protected DavFileInfo getFile() {
    try {
      Node resNode = (Node)jcrSrcSession().getItem(davRequest().getSrcPath());
  
      if (!resNode.isNodeType(DavConst.NodeTypes.NT_FILE)) {
        return getFolder(resNode);
      }

      if (davRequest().getSrcVersion() != null) {
        VersionHistory vHistory = resNode.getVersionHistory();
        Version requiredVersion = vHistory.getVersion(davRequest().getSrcVersion());
        resNode = requiredVersion.getNode(DavConst.NodeTypes.JCR_FROZENNODE);
      }

      if (!resNode.hasNode(DavConst.NodeTypes.JCR_CONTENT)) {
        log.info("node has no jcr:content");
        return null;
      }
  
      Node fileNode = resNode.getNode(DavConst.NodeTypes.JCR_CONTENT);
      DavFileInfo fileInfo = new DavFileInfo();
    
      if (fileNode.hasProperty(DavConst.NodeTypes.JCR_MIMETYPE)) {
        fileInfo.setContentType(fileNode.getProperty(DavConst.NodeTypes.JCR_MIMETYPE).getString());
      }
      
      if (fileNode.hasProperty(DavConst.NodeTypes.JCR_LASTMODIFIED)) {
        fileInfo.setLatstModified(fileNode.getProperty(DavConst.NodeTypes.JCR_LASTMODIFIED).getString());
      }
      
      if (fileNode.hasProperty(DavConst.NodeTypes.JCR_DATA)) {
        fileInfo.setInputStream(fileNode.getProperty(DavConst.NodeTypes.JCR_DATA).getStream());
      }

      return fileInfo;
    } catch (PathNotFoundException pexc) {
    } catch (Exception exc) {
      log.info("Can't get content for " + davRequest().getSrcPath() + ". " + exc.getMessage());
      exc.printStackTrace();
    }
    return null;
  }
  
  protected DavFileInfo getFolder(Node resourceNode) throws Exception {
    DavFileInfo folderInfo = new DavFileInfo();
    
    ArrayList<DavFileInfo> files = new ArrayList<DavFileInfo>();
    
    NodeIterator nodeIter = resourceNode.getNodes();
    while (nodeIter.hasNext()) {
      Node curNode = nodeIter.nextNode();
      files.add(getResourceHeader(curNode));
    }
    
    HtmlContentManager htmlManager = new HtmlContentManager(davRequest().getServerPrefix(), davRequest().getServerApp());
    folderInfo.setInputStream(htmlManager.getHtmlStream("/" + davRequest().getSrcWorkspace() + davRequest().getSrcPath(), files));    
    return folderInfo;
  }

  public InputStream getHtmlWorkspaces() {
    try {
      String []workspaces = davContext().getAvailableWorkspaces(); 
      
      ArrayList<DavFileInfo> files = new ArrayList<DavFileInfo>();
      for (int i = 0; i < workspaces.length; i++) {
        DavFileInfo workspaceInfo = new DavFileInfo();
        workspaceInfo.setName(workspaces[i]);
        files.add(workspaceInfo);
      }
      
      HtmlContentManager htmlManager = new HtmlContentManager(davRequest().getServerPrefix(), davRequest().getServerApp());
      return htmlManager.getHtmlStream("", files);      
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage());
      exc.printStackTrace();
    }
    return null;
  }
  
}
