/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.commands;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.lock.Lock;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.FakeLockTable;
import org.exoplatform.services.webdav.props.LockDiscoveryProp;
import org.exoplatform.services.webdav.request.LockInfoDoc;
import org.exoplatform.services.webdav.request.RequestDoc;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class LockCommand extends WebDavCommand {
  
  private static Log log = ExoLogger.getLogger("jcr.LockCommand");

  protected boolean process() {    
    RequestDoc requestDoc = davRequest().getDocumentFromRequest();
    
    try {
      Node srcNode = (Node)jcrSrcSession().getItem(davRequest().getSrcPath());

      try {
        if (!srcNode.isNodeType(DavConst.NodeTypes.MIX_LOCKABLE)) {
          srcNode.addMixin(DavConst.NodeTypes.MIX_LOCKABLE);
          jcrSrcSession().save();
        }        
      } catch (Exception lockExc) {
        log.info("Can't add MIX:LOCKABLE to " + davRequest().getSrcPath());
      }
      
      Lock lockResult = srcNode.lock(true, false);
      jcrSrcSession().save();

      if (lockResult == null) {
        davResponse().answerPreconditionFailed();
        return false;
      }
      
      LockDiscoveryProp lockDiscovery = new LockDiscoveryProp();
      lockDiscovery.fillFromNode(srcNode, "");

      lockDiscovery.setLockToken(lockResult.getLockToken());
      
      davResponse().setResponseHeader(DavConst.Headers.LOCKTOKEN, "<" + lockResult.getLockToken() + ">");
      davResponse().setProperty(lockDiscovery);
      
      return true;
    } catch (PathNotFoundException pexc) {
      LockInfoDoc lockDoc = (LockInfoDoc)requestDoc;

      FakeLockTable lockTable = davContext().getLockTable();
      
      String resourcePath = davRequest().getSrcWorkspace() + davRequest().getSrcPath();
      
      log.info("LOCK RESOURCE PATh - [" + resourcePath + "]");
      
      String lockToken = lockTable.lockResource(resourcePath);
      
      LockDiscoveryProp lockDiscovery = new LockDiscoveryProp();
      lockDiscovery.setLocked(true);
      lockDiscovery.setOwner(lockDoc.getLockOwner());
      lockDiscovery.setLockToken(lockToken);      
      lockDiscovery.setStatus(DavConst.HttpStatus.OK);
      
      davResponse().setResponseHeader(DavConst.Headers.LOCKTOKEN, "<" + lockToken + ">");      
      davResponse().setProperty(lockDiscovery);            
      return true;
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
    return false;    
  }
  
}
