/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.commands;

import java.util.ArrayList;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.props.DisplayNameProp;
import org.exoplatform.services.webdav.props.PropManager;
import org.exoplatform.services.webdav.props.WebDavProperty;
import org.exoplatform.services.webdav.request.CommonPropDoc;
import org.exoplatform.services.webdav.request.PropFindDoc;
import org.exoplatform.services.webdav.request.RequestDoc;
import org.exoplatform.services.webdav.response.DMultistatus;
import org.exoplatform.services.webdav.response.DResponse;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class PropfindCommand extends WebDavCommand {
  
  private static Log log = ExoLogger.getLogger("jcr.PropfindCommand"); 

  public PropfindCommand() {
    super();
    needSrcSession = false;
  }
  
  protected boolean process() {    
    PropFindDoc propFindDocument = (PropFindDoc)davRequest().getDocumentFromRequest();

    DMultistatus multistatus = null;

    if ("".equals(davRequest().getSrcWorkspace())) {
      multistatus = getPresetWorkspaces(propFindDocument);
    } else {
      srcSession.set(davRequest().getSourceSession(davContext().getSessionProvider()));
      
      if (jcrSrcSession() == null) {
        String defAuthHeader = davContext().getConfig().getAuthHeader();
        davResponse().answerAuthNeeded(defAuthHeader);
        return false;
      }
      multistatus = getProperties(propFindDocument);
    }
    
    if (multistatus == null) {
      davResponse().answerNotFound();
      return false;
    }
    
    davResponse().setMultistatus(multistatus);    
    return true;    
  }
  
  public DMultistatus getPresetWorkspaces(PropFindDoc propFindDocument) {
    String []workspaces = davContext().getAvailableWorkspaces();
    
    String serverPrefix = davRequest().getServerPrefix();
    
    ArrayList<DResponse> responses = new ArrayList<DResponse>();
    
    for (String curWorkspace : workspaces) {
      DResponse response = new DResponse();
        response.setHref(serverPrefix + "/" + curWorkspace + "/");
   
        ArrayList<String> propNames = propFindDocument.getProperties();
        for (int i = 0; i < propNames.size(); i++) {
          String curPropertyName = propNames.get(i);
          WebDavProperty davProp = PropManager.getPropertyByName(curPropertyName);
      
          while (true) {
            if (DavConst.DavProperty.DISPLAYNAME.equals(curPropertyName)) {
              ((DisplayNameProp)davProp).setDisplayName(curWorkspace);
              davProp.setStatus(DavConst.HttpStatus.OK);
              break;
            }
            if (DavConst.DavProperty.RESOURCETYPE.equals(curPropertyName)) {
              davProp.setStatus(DavConst.HttpStatus.OK);
              break;
            }
            break;
          }
      
          response.AddProperty(davProp);
        }
        responses.add(response);
    }
    
    return new DMultistatus(responses);
  }
  
  public DMultistatus getProperties(RequestDoc propsInfo) {
    if (!(propsInfo instanceof PropFindDoc)) {
      return null;
    }

    ArrayList<DResponse> responses = new ArrayList<DResponse>();
    
    String srcPath = davRequest().getSrcPath();
    
    try {
      Node resNode = null;
      
      try {
        resNode = (Node)jcrSrcSession().getItem(srcPath);
      } catch (PathNotFoundException pexc) {
        return null;
      }
      
      String pathNormalized = davRequest().getServerPrefix() + "/" + jcrSrcSession().getWorkspace().getName() + srcPath;
      if (pathNormalized.endsWith("/")) {
        pathNormalized = pathNormalized.substring(0, pathNormalized.length() - 1);
      }

      if (resNode.isNodeType(DavConst.NodeTypes.NT_FILE)) {
        responses.add(getResponseByNode(resNode, pathNormalized, (CommonPropDoc)propsInfo));
      } else {
        responses.add(getResponseByNode(resNode, pathNormalized + "/", (CommonPropDoc)propsInfo));        
        NodeIterator nodeIter = resNode.getNodes();
        while (nodeIter.hasNext()) {
          Node curNode = nodeIter.nextNode();
          String href = pathNormalized + "/" + curNode.getName();
          if (!curNode.isNodeType(DavConst.NodeTypes.NT_FILE)) {
            href += "/"; 
          }
          responses.add(getResponseByNode(curNode, href, (CommonPropDoc)propsInfo));
        }
      }
      
    } catch (Exception exc) {
      log.info("Can't find properties. " + exc.getMessage());
      exc.printStackTrace();
      return null;
    }
    
    return new DMultistatus(responses);
  }  

}
