/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.commands;

import java.util.Calendar;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.FakeLockTable;
import org.exoplatform.services.webdav.WebDavConfigImpl;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class PutCommand extends WebDavCommand {
  
  private static Log log = ExoLogger.getLogger("jcr.DavPut");

  protected boolean process() {
    try {      
      String mimeType = davRequest().getContentType();
      
      if (mimeType == null) {
        String fileName = getFileName();
        
        MimeTypeResolver mimeTypeResolver = new MimeTypeResolver();
        mimeTypeResolver.setDefaultMimeType(davContext().getConfig().getDefFileMimeType());
        mimeType = mimeTypeResolver.getMimeType(fileName);        
      }
      
      if (davRequest().getRequestStream() == null) {
        log.info("Request Input stream is NULL.");
        davResponse().answerPreconditionFailed();
        return false;
      }
      
      String resourcePath = davRequest().getSrcWorkspace() + davRequest().getSrcPath();      
      FakeLockTable lockTable = davContext().getLockTable();      
      String presentLockToken = lockTable.getLockToken(resourcePath);
      
      if (presentLockToken != null) {        
        String requestLockToken = davRequest().getLockToken();
        
        if (!presentLockToken.equals(requestLockToken)) {          
          davResponse().answerForbidden();
          return false;
        }
        
        createNtFile(mimeType);
        
        lockTable.unLockResource(resourcePath);
      } else {        
        if (jcrSrcSession().itemExists(davRequest().getSrcPath())) {          
          String updatePolicyType = davContext().getConfig().getUpdatePolicyType();
          
          if (updatePolicyType.equals(WebDavConfigImpl.INIT_VALUE_ADD)) {
            createNtFile(mimeType);
          } else if (updatePolicyType.equals(WebDavConfigImpl.INIT_VALUE_CREATE_VERSION)) {
            createNtFileVersion(mimeType);
          } else {
            updateNtFile(mimeType);
          }
          
        } else {
          createNtFile(mimeType);
        }
      }
      
      davResponse().answerCreated();
      
      return true;
    } catch (PathNotFoundException pexc) {
      log.info("Resource path not found. ");
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);      
    }
    
    davResponse().answerForbidden();    
    return false; 
  }
  
  protected String getFilePath() {
    String fullPath = davRequest().getSrcPath();
    String filePath = fullPath.substring(0, fullPath.lastIndexOf("/"));

    if ("".equals(filePath)) {
      filePath = "/";
    }
    return filePath;
  }
  
  protected String getFileName() {
    String fullPath = davRequest().getSrcPath();

    String fileName = fullPath.substring(fullPath.lastIndexOf("/"));
    if (fileName.startsWith("/")) {
      fileName = fileName.substring(1);
    }
    return fileName;
  }
  
  protected void createNtFile(String mimeType) throws Exception {
    Node srcNode = (Node)jcrSrcSession().getItem(getFilePath());

    Node ntFileNode = srcNode.addNode(getFileName(), davContext().getConfig().getDefFileNodeType());
    Node contentNode = ntFileNode.addNode(DavConst.NodeTypes.JCR_CONTENT, DavConst.NodeTypes.NT_RESOURCE);
    
    contentNode.setProperty(DavConst.NodeTypes.JCR_MIMETYPE, mimeType);
    contentNode.setProperty(DavConst.NodeTypes.JCR_LASTMODIFIED, jcrSrcSession().getValueFactory().createValue(Calendar.getInstance()));
    contentNode.setProperty(DavConst.NodeTypes.JCR_DATA, davRequest().getRequestStream());
    
    jcrSrcSession().save();
  }
  
  protected void updateNtFile(String mimeType) throws Exception {    
    Node fileNode = (Node)jcrSrcSession().getItem(davRequest().getSrcPath());
    Node contentNode = fileNode.getNode(DavConst.NodeTypes.JCR_CONTENT);
    
    contentNode.setProperty(DavConst.NodeTypes.JCR_MIMETYPE, mimeType);
    contentNode.setProperty(DavConst.NodeTypes.JCR_LASTMODIFIED, jcrSrcSession().getValueFactory().createValue(Calendar.getInstance()));
    contentNode.setProperty(DavConst.NodeTypes.JCR_DATA, davRequest().getRequestStream());
    
    jcrSrcSession().save();
  }
  
  protected void createNtFileVersion(String mimeType) throws Exception {
    Node fileNode = (Node)jcrSrcSession().getItem(davRequest().getSrcPath());
    
    if (!fileNode.isNodeType(DavConst.NodeTypes.MIX_VERSIONABLE)) {
      
      if (davContext().getConfig().isAutoMixLockable()) {
        fileNode.addMixin(DavConst.NodeTypes.MIX_LOCKABLE);
      }
      
      fileNode.addMixin(DavConst.NodeTypes.MIX_VERSIONABLE);
      
      jcrSrcSession().save();
      
      fileNode.checkout();
      fileNode.checkin();
    }
    
    fileNode.checkout();
    
    updateNtFile(mimeType);
    
    fileNode.checkin();
    
  }
  
}
