/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.props;

import java.util.Locale;
import java.util.TimeZone;
import javax.jcr.Property;
import javax.jcr.Node;
import java.text.SimpleDateFormat;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class CreationDateProp extends CommonProp {

  private static Log log = ExoLogger.getLogger("jcr.CreationDateProp");
  protected String creationDate = "";
  
  public CreationDateProp() {
    super(DavConst.DavProperty.CREATIONDATE);
  }
  
  @Override
  public boolean fillFromNode(Node node, String href) {
    try {
      if (!node.hasProperty(DavConst.NodeTypes.JCR_CREATED)) {
        return false;
      }
      Property createdProp = node.getProperty(DavConst.NodeTypes.JCR_CREATED);
      
      SimpleDateFormat dateFormat = new SimpleDateFormat(DavConst.DateFormat.CREATION, Locale.ENGLISH);
      dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));      
      creationDate = dateFormat.format(createdProp.getDate().getTime());
      
      status = DavConst.HttpStatus.OK;
    } catch (Exception exc) {
      log.info("Can't get creationDate from node. " + exc.getMessage());
      exc.printStackTrace();
    }
    return false;
  }  
  
  @Override
  public void toXml(Document rootDoc, Element parentElement) {
    super.toXml(rootDoc, parentElement);
    if (status != DavConst.HttpStatus.OK) {
      return;
    }
    propertyElement.setTextContent(creationDate);
  }
  
}
