/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.props;

import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.TimeZone;
import javax.jcr.Node;
import javax.jcr.Property;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class LastModifiedProp extends CommonProp {
  
  private static Log log = ExoLogger.getLogger("jcr.LastModifiedProp");
  protected String lastModified = "";
  
  public LastModifiedProp() {
    super(DavConst.DavProperty.GETLASTMODIFIED);
  }
  
  @Override
  public boolean fillFromNode(Node node, String href) {
    try {
      if (!node.isNodeType(DavConst.NodeTypes.NT_FILE)) {
        return false;
      }      
      if (!node.hasNode(DavConst.NodeTypes.JCR_CONTENT)) {
        return false;
      }
      
      Node contentNode = node.getNode(DavConst.NodeTypes.JCR_CONTENT);
      
      if (!contentNode.hasProperty(DavConst.NodeTypes.JCR_LASTMODIFIED)) {
        return false;
      }
      
      Property lastModifiedProp = contentNode.getProperty(DavConst.NodeTypes.JCR_LASTMODIFIED);
      
      SimpleDateFormat dateFormat = new SimpleDateFormat(DavConst.DateFormat.MODIFICATION, Locale.ENGLISH);
      dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));      
      lastModified = dateFormat.format(lastModifiedProp.getDate().getTime());
      
      status = DavConst.HttpStatus.OK;
      return true;
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage());
      exc.printStackTrace();
    }
    return false;
  }  
  
  @Override
  public void toXml(Document rootDoc, Element parentElement) {
    super.toXml(rootDoc, parentElement);
    if (status != DavConst.HttpStatus.OK) {
      return;
    }
    propertyElement.setTextContent(lastModified);
  }

}
