/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.request;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class LockInfoDoc implements RequestDoc {

  private static Log log = ExoLogger.getLogger("jcr.LockInfoDoc");
  
  protected String lockType = DavConst.Lock.TYPE_WRITE;
  protected String lockScope = DavConst.Lock.SCOPE_EXCLUSIVE;
  protected String lockOwner = "";
  protected String lockOwnerHref = "";
  
  public LockInfoDoc() {
  }
  
  public boolean initFromDocument(Document requestDocument) {
    try {
      Node nLockInfo = DavUtil.getChildNode(requestDocument, DavConst.DavDocument.LOCKINFO);
      
      Node nLockType = DavUtil.getChildNode(nLockInfo, DavConst.DavProperty.LOCKTYPE);
      if (nLockType != null) {
        Node nWrite = DavUtil.getChildNode(nLockType, DavConst.DavProperty.WRITE);
        if (nWrite != null) {
          lockType = DavConst.Lock.TYPE_WRITE;
        }
      }
      
      Node nScope = DavUtil.getChildNode(nLockInfo, DavConst.DavProperty.LOCKSCOPE);
      if (nScope != null) {
        Node nExclusive = DavUtil.getChildNode(nScope, DavConst.DavProperty.EXCLUSIVE);
        if (nExclusive != null) {
          lockScope = DavConst.Lock.SCOPE_EXCLUSIVE;
        }
        Node nShared = DavUtil.getChildNode(nScope, DavConst.DavProperty.SHARED);
        if (nShared != null) {
          lockScope = DavConst.Lock.SCOPE_SHARED;
        }
      }

      
      Node nOwner = DavUtil.getChildNode(nLockInfo, DavConst.DavProperty.OWNER);
      if (nOwner != null) {
        lockOwner = nOwner.getTextContent();
        Node nOwnerHref = DavUtil.getChildNode(nOwner, DavConst.DavProperty.HREF);
        if (nOwnerHref != null) {
          lockOwnerHref = nOwnerHref.getTextContent();          
        }
      }
      
      return true;
    } catch (Exception exc) {
      log.info("Can't fill document data. " + exc.getMessage());
      exc.printStackTrace();      
    }
    return false;
  }

  public String getLockType() {
    return lockType;
  }
  
  public String getLockScope() {
    return lockScope;
  }
  
  public String getLockOwner() {
    return lockOwner;
  }
  
  public String getLockOwnerHref() {
    return lockOwnerHref;
  }
  
}
