/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav.request;

import java.util.ArrayList;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class PropertyUpdateDoc implements RequestDoc {
  
  private static Log log = ExoLogger.getLogger("jcr.PropertyUpdateDoc");
  
  protected ArrayList<String> needsToSet = new ArrayList<String>();
  protected ArrayList<String> needsToRemove = new ArrayList<String>();

  public PropertyUpdateDoc() {
  }
  
  public boolean initFromDocument(Document requestDocument) {
    try {
      Node propertyUpdate = DavUtil.getChildNode(requestDocument, DavConst.DavDocument.PROPERTYUPDATE);
      
      Node nodeSet = DavUtil.getChildNode(propertyUpdate, DavConst.DavProperty.SET);
      if (nodeSet != null) {
        Node nodeProps = DavUtil.getChildNode(nodeSet, DavConst.DavProperty.PROP);        
        NodeList nodes = nodeProps.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
          Node curNode = nodes.item(i);
          String localName = curNode.getLocalName();
          if (localName != null && DavConst.DAV_NAMESPACE.equals(curNode.getNamespaceURI())) {
            addPropertyToSet(localName);
          }
        }
      }
      
      Node nodeRemove = DavUtil.getChildNode(propertyUpdate, DavConst.DavProperty.REMOVE);
      if (nodeRemove != null) {
        Node nodeProps = DavUtil.getChildNode(nodeRemove, DavConst.DavProperty.PROP);
        NodeList nodes = nodeProps.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
          Node curNode = nodes.item(i);          
          String localName = curNode.getLocalName();
          if (localName != null && DavConst.DAV_NAMESPACE.equals(curNode.getNamespaceURI())) {
            addPropertyToRemove(localName);
          }
          
        }
      }

      return true;
    } catch (Exception exc) {
      log.info("Can't fill document data. " + exc.getMessage());
      exc.printStackTrace();      
    }
    return false;
  }  
  
  public void addPropertyToSet(String propertyName) {
    needsToSet.add(propertyName);
  }
  
  public void addPropertyToRemove(String propertyName) {
    needsToRemove.add(propertyName);
  }

  public ArrayList<String> getPropertiesToSet() {
    return needsToSet;
  }
  
  public ArrayList<String> getPropertiesToRemove() {
    return needsToRemove;
  }  
  
}
