/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.webdav.response;

import java.io.ByteArrayOutputStream;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.commons.logging.Log;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavConst.HttpStatus;
import org.exoplatform.services.webdav.props.WebDavProperty;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class WebDavResponse extends HttpServletResponseWrapper {

  private static Log log = ExoLogger.getLogger("jcr.WebDavResponse");

  public WebDavResponse(HttpServletResponse response) {
    super(response);
  }

  public void setMultistatus(DMultistatus multistatus) {
    try {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      Document responseDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      
      multistatus.toXml(responseDoc);
      
      OutputFormat format = new OutputFormat("xml", "UTF-8", true);
      XMLSerializer serializer = new XMLSerializer(out, format);
      serializer.setNamespaces(true);
      serializer.asDOMSerializer().serialize(responseDoc);
      
      byte []bytes = out.toByteArray();

      setStatus(DavConst.HttpStatus.MULTISTATUS);      
      addHeader(DavConst.Headers.CONTENTTYPE, "text/xml;charset=UTF-8 ");
      addHeader(DavConst.Headers.CONTENTLENGTH, String.format("%s", bytes.length));
      
      for (int i = 0; i < bytes.length; i++) {
        getOutputStream().write(bytes[i]);
      }

    } catch (Exception exc) {
      log.info("Can't send multistatus. " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  public void setProperty(WebDavProperty property) {
    try {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      Document responseDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      
      Element propElement = responseDoc.createElementNS(DavConst.DAV_NAMESPACE, DavConst.DAV_PREFIX + DavConst.DavProperty.PROP);
      responseDoc.appendChild(propElement);
      
      property.toXml(responseDoc, propElement);
      
      OutputFormat format = new OutputFormat("xml", "UTF-8", true);
      XMLSerializer serializer = new XMLSerializer(out, format);
      serializer.setNamespaces(true);
      serializer.asDOMSerializer().serialize(responseDoc);
      
      byte []bytes = out.toByteArray();
      
      setStatus(DavConst.HttpStatus.OK);
      addHeader(DavConst.Headers.CONTENTTYPE, "text/xml;charset=UTF-8 ");
      addHeader(DavConst.Headers.CONTENTLENGTH, String.format("%s", bytes.length));
  
      for (int i = 0; i < bytes.length; i++) {
        getOutputStream().write(bytes[i]);
      }

    } catch (Exception exc) {
      log.info("Can't send property. " + exc.getMessage());
      exc.printStackTrace();
    }    
  }  

  public void setResponseHeader(String headerName, String headerValue) {
    addHeader(headerName, headerValue);
  }
  
  public void answerAuthNeeded(String defAuthHeader) {
    addHeader(DavConst.Headers.WWWAUTHENTICATE, defAuthHeader);
    setStatus(DavConst.HttpStatus.AUTHNEEDED);
  }  
  
  public void answerOk() {
    addHeader(DavConst.Headers.CACHECONTROL, "no-cache");
    setStatus(DavConst.HttpStatus.OK);
  }
  
  public void answerCreated() {
    setStatus(DavConst.HttpStatus.CREATED);
  }  
  
  public void answerNoContent() {
    setStatus(DavConst.HttpStatus.NOCONTENT);
  }  
  
  public void answerNotFound() {
    setStatus(DavConst.HttpStatus.NOTFOUND);
  }  

  public void answerForbidden() {
    setStatus(DavConst.HttpStatus.FORBIDDEN);
  }  

  public void answerPreconditionFailed() {
    setStatus(DavConst.HttpStatus.PRECONDITIONFAILED);
  }  
  
  public void answerNotImplemented() {
    setStatus(DavConst.HttpStatus.NOTIMPLEMENTED);
  }
  
  public void writeToResponse(byte []data, int len) throws Exception {
    getOutputStream().write(data, 0, len);
  }

}
